/*
	flash programming helper routines
*/
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "lamps.h"
#include "comm.h"
#include "flash.h"

#define	MAX_FIELDS	90

static BYTE dataBuf[BYTE_PAGE_SIZE];
static int dataPtr;
static int dataSize;
static void *currPage;
static char flashBuf[RX_MAX+1];
// for parsing of light bar data
static char *fields[MAX_FIELDS+1];
static volatile int fldPtr;
static PROGRAM *programPtr;
static LAMP_PROGRAM *lampProgramPtr;
static LAMP_GROUP *lampGroupPtr;
static int prgNdx;
static int pgmNdx;
static int grpNdx;
static int lmpNdx;

const PARAMS __attribute__ ((aligned (BYTE_PAGE_SIZE),section("PARAMS"),space(prog))) FlashParams =
	{ 0x0 };
const BAR_DATA __attribute__ ((aligned (BYTE_PAGE_SIZE),section("BAR_DATA"),space(prog))) BarData;
PARAMS Params;

void FlashBegin()
{
	currPage = (void *)&BarData;
	dataSize = 0;
	dataPtr = 0;
}

void FlashWriteByte(BYTE b)
{
	if (dataPtr >= BYTE_PAGE_SIZE)
	{
		UserFlashPageWriteBytes(currPage, dataBuf, BYTE_PAGE_SIZE);
		currPage += BYTE_PAGE_SIZE;
		dataPtr = 0;
		dataSize += BYTE_PAGE_SIZE;
	}
	dataBuf[dataPtr++] = b;
}

void FlashWriteBuf(char *buf, int len)
{
	while(len--)
		FlashWriteByte((BYTE) *buf++);
}

void ReadParams()
{
	UserFlashPageReadBytes(&FlashParams, &Params, sizeof(Params));
}

void SaveParams()
{
	UserFlashPageWriteBytes(&FlashParams, (void *)&Params, sizeof(Params));
}


void FlashEnd()
{
	DWORD page;
	int i;

	dataSize += dataPtr;
	UserFlashPageWriteBytes(currPage, dataBuf, dataPtr);


	for(i = 0; i < CYCLE_INPUTS; i++)
		Params.Index[i] = 0;
	Params.flashSize = dataSize;

	SaveParams();
	dataPtr = 0;
}

void UserFlashPageReadBytes(const void * userFlash, void *buff, WORD numBytes)
{
	// Read numBytes from user page into buff.
	memcpy((void *)buff, userFlash, numBytes);
}
 
void UserFlashPageWriteBytes(const void * userFlashPage, BYTE *data, WORD numBytes)
{
	int numBytesLeft = (int)numBytes;
	WORD maxRows;

	while(numBytesLeft > 0)
	{
		// Erase one page (4096 Bytes) of User Program Flash
		NVMErasePage((void *)userFlashPage);
		
		// program this page
		maxRows = NUM_ROWS_PAGE;
		while (numBytesLeft > 0 && maxRows--)
		{
			// Write 512 bytes (one NVM row) from pageBuff. Writes a whole row always.
			NVMWriteRow((void *)(userFlashPage), (void*)data);
			numBytesLeft -= BYTE_ROW_SIZE;
			data += BYTE_ROW_SIZE;
			userFlashPage += BYTE_ROW_SIZE;
		}
	}
}

static void CleanUp()
{
	int prNdx, pNdx, pCnt, lpCnt, lpNdx, lgNdx, lgCnt, lsNdx, lsCnt;

	if (Programs.count > 0)
	{
		for(prNdx = 0; prNdx < Programs.count; prNdx++)
		{
			ClearWDT();
			pCnt = Programs.progs[prNdx].count;
			for(pNdx = 0; pNdx < pCnt; pNdx++)
			{
				lpCnt = Programs.progs[prNdx].lamp_programs[pNdx].count;
				for(lpNdx = 0; lpNdx < lpCnt; lpNdx++)
				{
					lgCnt = Programs.progs[prNdx].lamp_programs[pNdx].lamp_groups[lpNdx].count;
					for(lgNdx = 0; lgNdx < lgCnt; lgNdx++)
					{
						lsCnt = Programs.progs[prNdx].lamp_programs[pNdx].lamp_groups[lpNdx].lamp_seqs[lgNdx].count;
						for(lsNdx = 0; lsNdx < lsCnt; lsNdx++)
							free(Programs.progs[prNdx].lamp_programs[pNdx].lamp_groups[lpNdx].lamp_seqs[lgNdx].lamps);
					}
					free((void *)Programs.progs[prNdx].lamp_programs[pNdx].lamp_groups[lpNdx].lamp_seqs);
				}
				free((void *)Programs.progs[prNdx].lamp_programs[pNdx].lamp_groups);
			}
			free((void *)Programs.progs[prNdx].lamp_programs);
		}
		free((void *)Programs.progs);
		Programs.count = 0;
	}
	if (ProgInputs != NULL)
	{
		free((void *)ProgInputs);
		ProgInputs = NULL;
	}
}

static int ProcFlashLine()
{
	int i, j, cnt, ndx;
	LAMP *lmps;
	LAMP_SEQ *lampSeqPtr;
	PROG_INPUT *piPtr;
	int errCode = 0;

	ClearWDT();
	switch(*flashBuf)
	{
// not for Odyssey
#if ! defined(ODYSSEY)
		case 'Q':		// start of input program data
			if (ProgInputs == NULL)
				ProgInputs = (PROG_INPUT *)malloc(sizeof(PROG_INPUT) * MAX_INPUTS + 1);		// + 1 for the 13 & 14 program
			if (fldPtr < 7)
			{
				errCode = 4;
			}
			else
			{
				i = atoi(fields[1]);
				if (i < MAX_INPUTS + 1)		// +1 for the 13 & 14 program
				{
					piPtr = &ProgInputs[i];
					// # of programs, 1st 4 inputs have 15 otherwise 1
					if (i >= CYCLE_INPUTS)
						cnt = 1;		
					else
						cnt = MAX_CYCLE;
					ndx = 2;
					for(j = 0; j < cnt; j++)
						piPtr->ProgIndexes[j] = atoi(fields[ndx++]);	// program numbers, -1 is no program
					piPtr->InpType = atoi(fields[ndx++]);		// 0-momentary, 1-Toggle, 2-Timer
					piPtr->TimerValue = atoi(fields[ndx++]);
					piPtr->Priority = atoi(fields[ndx++]);
					piPtr->Dimmed = *fields[ndx++] == 'T';
					piPtr->Level = atoi(fields[ndx++]);
					piPtr->Toggle = FALSE;
				}
			}			
			break;
#endif
		case 'L':		// start of light bar config
			if (fldPtr < 6)
			{
				errCode = 4;
			}
			else
			{
				CleanUp();		// free previous data!
				pgmNdx = 0;
				prgNdx = 0;
				grpNdx = 0;
				lmpNdx = 0;
				cnt = atoi(fields[1]);
				Programs.count = cnt;
				Programs.frontLamps = atoi(fields[2]);
				Programs.rearLamps = atoi(fields[3]);
				Programs.dimLevel = atoi(fields[4]);
				Programs.progs = (PROGRAM *)malloc(sizeof(PROGRAM) * cnt);
				cnt = atoi(fields[5]);		// number of dimmable lamps
				if (cnt > MAX_DIMMABLE)
					cnt = MAX_DIMMABLE;
				Programs.dimCount = cnt;
				for(i = 0; i < cnt; i++)
					Programs.dimmable[i] = *fields[i + 6] == 'T';
			}
			break;
		case 'F':		// start of new flash program
			if (fldPtr < 3 || prgNdx >= Programs.count)
			{
				errCode = 2;
			}
			else
			{
				cnt = atoi(fields[2]);
				programPtr = &Programs.progs[prgNdx++];
				programPtr->count = cnt;
				programPtr->Running = FALSE;	// not currently running
				pgmNdx = 0;
				programPtr->lamp_programs = (LAMP_PROGRAM *)malloc(sizeof(LAMP_PROGRAM) * cnt);
				if (programPtr->lamp_programs == NULL)
					errCode = 7;
			}
			break;
		case 'G':		// program group
			if (fldPtr < 3 || pgmNdx >= programPtr->count)		// check line length and pgm index
				errCode = 4;
			else
			{
				grpNdx = 0;
				cnt = atoi(fields[1]);
				lampProgramPtr = &programPtr->lamp_programs[pgmNdx++];
				lampProgramPtr->count = cnt;
				lampProgramPtr->lamp_groups = (LAMP_GROUP *)malloc(sizeof(LAMP_GROUP) * cnt);
				if (lampProgramPtr->lamp_groups == NULL)
					errCode = 7;
			}
			break;
		case 'P':		// lamp group
			if (fldPtr < 4 || grpNdx >= lampProgramPtr->count)
				errCode = 5;
			else
			{
				lmpNdx = 0;
				cnt = atoi(fields[2]);
				lampGroupPtr = &lampProgramPtr->lamp_groups[grpNdx++];
				lampGroupPtr->count = cnt;					// number of lamp_seqs
				lampGroupPtr->rep = atoi(fields[1]);		// repeat 
				lampGroupPtr->lamp_seqs = (LAMP_SEQ *)malloc(sizeof(LAMP_SEQ) * cnt);
				if (lampGroupPtr->lamp_seqs == NULL)
					errCode = 7;
			}
			break;
		case 'N':
			if (fldPtr < 4 || lmpNdx >= lampGroupPtr->count)
				errCode = 6;
			else
			{
				cnt = atoi(fields[1]);
				lampSeqPtr = &lampGroupPtr->lamp_seqs[lmpNdx++];
				lampSeqPtr->count = cnt;
				lmps = (LAMP *)malloc(sizeof(LAMP) * cnt);		// space for array of lamps
				lampSeqPtr->lamps = lmps;			// this array of LAMPs
				lampSeqPtr->delay = atoi(fields[2]);	// these lamps delay
				if (lmps == NULL)
					errCode = 7;
				else
				{
					for(i = 0; i < cnt * 2; i += 2, lmps++)
					{
						lmps->lamp = atoi(fields[i + 3]);
						lmps->on = *fields[i + 4] == 'T';
					}
				}
			}
			break;
	}

	return errCode;
}

static void CalcLamps()
{
	int i, prNdx, pNdx, gNdx, lNdx, sNdx;
	PROGRAM *cProg;
	LAMP_PROGRAM *lProg;
	LAMP_GROUP *lGroup;
	LAMP_SEQ *lSeq;

	for(prNdx = 0; prNdx < Programs.count; prNdx++)
	{
		cProg = &Programs.progs[prNdx];
		cProg->Running = FALSE;
		for(i = 0; i < LAMP_COUNT; i++)
		{
			cProg->LampEn[i] = FALSE;		// used for priority calculations
			cProg->LampList[i] = FALSE;	// list of lamps in this program
		}					
		for(pNdx = 0; pNdx < cProg->count; pNdx++)
		{
			lProg = &cProg->lamp_programs[pNdx];
			for(gNdx = 0; gNdx < lProg->count; gNdx++)
			{
				lGroup = &lProg->lamp_groups[gNdx];
				for(lNdx = 0; lNdx < lGroup->count; lNdx++)
				{
					lSeq = &lGroup->lamp_seqs[lNdx];
					for(sNdx = 0; sNdx < lSeq->count; sNdx++)
						cProg->LampList[lSeq->lamps[sNdx].lamp] = TRUE;
				}
			}
		}
	}
}

int ReadPrograms()
{
	int romptr = 0;
	int ramptr = 0;
	int errCode = 0;
	char c;

	CloseTimer2();		// shutdown intterupt
	CleanUp();			// kill the old stuff
	ReadParams();		// get the parameters from Flash

	fldPtr = 1;
	fields[0] = flashBuf;
	while(romptr < Params.flashSize && ! errCode)
	{
		c = (char)BarData.bytes[romptr++];
		if (c == '|' || c == '^')
		{
			if (fldPtr < MAX_FIELDS)
			{
				flashBuf[ramptr++] = '\0';
				fields[fldPtr++] = flashBuf + ramptr;
			}
		}
		else
		{
			if (c == '\0')
			{
				flashBuf[ramptr++] = '\0';
				errCode = ProcFlashLine();
				ramptr = 0;
				fldPtr = 1;
				fields[0] = flashBuf;
			}
			else
				if (ramptr < RX_MAX)
					flashBuf[ramptr++] = c;
		}
	}

	if (! errCode)
		CalcLamps();		// figure out what lamps we have and the count for priority etc...
	LampInit();		// re-enable the timer

	return errCode;
}

