/*
	lamps.c
	D&R Electronics Lamp Controller Lamp outputs
*/
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "lamps.h"
#include "flash.h"
#include "analog.h"

/* locals */

static int currProgNdx;
static BOOL progDimmed;
static int dimDelay = 0;

typedef struct
{
	IoPortId port;
	unsigned int bits;
} port_bit_t;

static BOOL lampIsDim[LAMP_COUNT];
static BOOL lampStatus[LAMP_COUNT];

#if defined(ODYSSEY)
// for Odyssey the dimmer input is input 3
#define	DIM_INPUT	2
port_bit_t lamp_ports[LAMP_COUNT] = {
	{ IOPORT_A, 0x2 },		// RA1 L21/A (note: for Odyssey, 0 is L21/A (left alley)
	{ IOPORT_A, 0x0 },		// (note, it's a dummy bit because this is a single module in the Windows app
	{ IOPORT_E, 0x8 },		// RE3 L1/A
	{ IOPORT_E, 0x10 },		// RE4 L1/B
	{ IOPORT_G, 0x2000 },	// RG13 L2/A
	{ IOPORT_E, 0x4 },		// RE2 L2/B
	{ IOPORT_G, 0x4000 },	// RG14 L3/A
	{ IOPORT_G, 0x1000 },	// RG12 L3/B
	{ IOPORT_E, 0x1 },		// RE0 L4/A
	{ IOPORT_E, 0x2 },		// RE1 L4/B
	{ IOPORT_A, 0x40 },		// RA6 L5/A
	{ IOPORT_A, 0x80 },		// RA7 L5/B
	{ IOPORT_G, 0x2 },		// RG1 L6/A
	{ IOPORT_G, 0x1 },		// RG0 L6/B
	{ IOPORT_F, 0x1 },		// RF0 L7/A
	{ IOPORT_F, 0x2 },		// RF1 L7/B
	{ IOPORT_D, 0x40 },		// RD6 L8/A
	{ IOPORT_D, 0x80 },		// RD7 L8/B
	{ IOPORT_D, 0x10 },		// RD4 L9/A
	{ IOPORT_D, 0x20 },		// RD5 L9/B
	{ IOPORT_D, 0x1000 },	// RD12 L10/A
	{ IOPORT_D, 0x2000 },	// RD13 L10/B
	{ IOPORT_D, 0x4 },		// RD2 L11/A
	{ IOPORT_D, 0x8 },		// RD3 L11/B
	{ IOPORT_C, 0x4000 },	// RC14 L12/A
	{ IOPORT_D, 0x2 },		// RD1 L12/B
	{ IOPORT_C, 0x2000 },	// RC13 L13/A
	{ IOPORT_D, 0x1 },		// RD0 L13/B
	{ IOPORT_D, 0x800 },	// RD11 L14/A
	{ IOPORT_D, 0x400 },	// RD10 L14/B
	{ IOPORT_A, 0x10 },		// RA4 L15/A
	{ IOPORT_A, 0x20 },		// RA5 L15/B
	{ IOPORT_F, 0x2000 },	// RF13 L21/B (for odyssey, [16] is L21/B alley right
	{ IOPORT_F, 0x0 },		// dummy for [16] B half
	{ IOPORT_A, 0x4 },		// RA2 L16/A
	{ IOPORT_A, 0x8 },		// RA3 L16/B
	{ IOPORT_F, 0x10 },		// RF4 L17/A
	{ IOPORT_F, 0x20 },		// RF5 L17/B
	{ IOPORT_B, 0x8000 },	// RB15 L18/A
	{ IOPORT_D, 0x8000 },	// RD15 L18/B
	{ IOPORT_B, 0x2000 },	// RB13 L19/A
	{ IOPORT_B, 0x4000 },	// RB14 L19/B
	{ IOPORT_F, 0x1000 },	// RF12 L20/A
	{ IOPORT_B, 0x1000 },	// RB12 L20/B

	// this is the T1-T16 for the odyssey bar
	
	{ IOPORT_C, 0x8 },		// RC3 T1 (out 28) J11 Pin 3
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_E, 0x200 },	// RE9 T2 (out 29) J11 Pin 4
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_C, 0x4 },		// RC2 T3 (out 30) J11 Pin 5
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_G, 0x8000 },	// RG15 T4 (out 31) J11 Pin 6
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_G, 0x80 },		// RG7 T5 (out 25) J11 Pin 7
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_A, 0x1 },		// RA0 T6 (out 24) J11 Pin 8
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_C, 0x2 },		// RC1 T7 (out 32) J11 Pin 9
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_E, 0x80 },		// RE7 T8 (out 33) J11 Pin 10
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_E, 0x40 },		// RE6 T9 (out 34) J11 Pin 11
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_E, 0x20 },		// RE5 T10  (out 35) J11 Pin 12
	{ IOPORT_B, 0x0 },		// dummy

	{ IOPORT_C, 0x10 },		// RC4 T11 (out 27) J10 Pin 1
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_G, 0x40 },		// RG6 T12 (out 26) J10 Pin 2
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_B, 0x400 },	// RB10 T13 (L22/A) J10 Pin 3
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_B, 0x800 },	// RB11 T14 (L22/B) J10 Pin 4
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_B, 0x100 },	// RB8 T15 (L23/A) J10 Pin 5
	{ IOPORT_B, 0x0 },		// dummy
	{ IOPORT_B, 0x200 },	// RB9 T16 (L23/B) J10 Pin 6
	{ IOPORT_B, 0x0 },		// dummy
};
#else
// for Standard light bar the dimmer input is input 15
#define	DIM_INPUT	14

port_bit_t lamp_ports[LAMP_COUNT] = {
	{ IOPORT_E, 0x8 },		// RE3 L1/A
	{ IOPORT_E, 0x10 },		// RE4 L1/B
	{ IOPORT_G, 0x2000 },	// RG13 L2/A
	{ IOPORT_E, 0x4 },		// RE2 L2/B
	{ IOPORT_G, 0x4000 },	// RG14 L3/A
	{ IOPORT_G, 0x1000 },	// RG12 L3/B
	{ IOPORT_E, 0x1 },		// RE0 L4/A
	{ IOPORT_E, 0x2 },		// RE1 L4/B
	{ IOPORT_A, 0x40 },		// RA6 L5/A
	{ IOPORT_A, 0x80 },		// RA7 L5/B
	{ IOPORT_G, 0x2 },		// RG1 L6/A
	{ IOPORT_G, 0x1 },		// RG0 L6/B
	{ IOPORT_F, 0x1 },		// RF0 L7/A
	{ IOPORT_F, 0x2 },		// RF1 L7/B
	{ IOPORT_D, 0x40 },		// RD6 L8/A
	{ IOPORT_D, 0x80 },		// RD7 L8/B
	{ IOPORT_D, 0x10 },		// RD4 L9/A
	{ IOPORT_D, 0x20 },		// RD5 L9/B
	{ IOPORT_D, 0x1000 },	// RD12 L10/A
	{ IOPORT_D, 0x2000 },	// RD13 L10/B
	{ IOPORT_D, 0x4 },		// RD2 L11/A
	{ IOPORT_D, 0x8 },		// RD3 L11/B
	{ IOPORT_C, 0x4000 },	// RC14 L12/A
	{ IOPORT_D, 0x2 },		// RD1 L12/B
	{ IOPORT_C, 0x2000 },	// RC13 L13/A
	{ IOPORT_D, 0x1 },		// RD0 L13/B
	{ IOPORT_D, 0x800 },	// RD11 L14/A
	{ IOPORT_D, 0x400 },	// RD10 L14/B
	{ IOPORT_A, 0x10 },		// RA4 L15/A
	{ IOPORT_A, 0x20 },		// RA5 L15/B
	{ IOPORT_A, 0x4 },		// RA2 L16/A
	{ IOPORT_A, 0x8 },		// RA3 L16/B
	{ IOPORT_F, 0x10 },		// RF4 L17/A
	{ IOPORT_F, 0x20 },		// RF5 L17/B
	{ IOPORT_B, 0x8000 },	// RB15 L18/A
	{ IOPORT_D, 0x8000 },	// RD15 L18/B
	{ IOPORT_B, 0x2000 },	// RB13 L19/A
	{ IOPORT_B, 0x4000 },	// RB14 L19/B
	{ IOPORT_F, 0x1000 },	// RF12 L20/A
	{ IOPORT_B, 0x1000 },	// RB12 L20/B
	{ IOPORT_A, 0x2 },		// RA1 L21/A
	{ IOPORT_F, 0x2000 },	// RF13 L21/B
	{ IOPORT_B, 0x400 },	// RB10 L22/A
	{ IOPORT_B, 0x800 },	// RB11 L22/B
	{ IOPORT_B, 0x100 },	// RB8 L23/A
	{ IOPORT_B, 0x200 },	// RB9 L23/B
	{ IOPORT_A, 0x200 },	// RA9 L24/A
	{ IOPORT_A, 0x400 }	// RA10 L24/B
};
#endif

port_bit_t LampEnable = { IOPORT_E, 0x100 };		// RE8, Light Enable as per Rinaldo

#if defined(ODYSSEY)
/*
	Odyssey Motors
*/
port_bit_t OdyLampEnable = { IOPORT_A, 0x8000 };	// RA15, Odyssey Light Enable as Per Rinaldo
int motorState = 0;
int motorTick;
volatile BOOL needSignUp = FALSE;		// true we have modules in a program that are on the sign board
port_bit_t MotorDown = { IOPORT_A, 0x200 };	// RA9 motor down signal
port_bit_t MotorUp = { IOPORT_A, 0x400 };	// RA10 motor up signal
#endif

volatile BOOL softDim;		// software dimmer
volatile BOOL runFast;		// run fast
PROG_LIST Programs = { 0 };	// structure of programs, default to NO programs!
PROG_INPUT *ProgInputs = NULL;		// program inputs for stand-alone bar
static volatile BOOL programRun;		// a program is running!

static void EnableTimer1()
{
	int dim;

	// This statement configured the timer to produce an interrupt with a priority of 4
	ConfigIntTimer1( T1_INT_ON | T1_INT_PRIOR_4);
	// This statement says: turn on timer3 | have it use an internal clock source | have it
	// use a prescaler of 1:256, and use a period of 93, gives 40% of 2ms for dimming
	// 233 is the 2ms period, we're going to try and use a % of that
	dim = (Programs.dimLevel + 1) * 2;
	OpenTimer1( T1_ON | T1_SOURCE_INT | T1_PS_1_256, dim);	
}

static void EnableTimer2()
{
	// This statement says: turn on timer3 | have it use an internal clock source | have it
	// use a prescaler of 1:256, and use a period of 234, give 500 ints / sec for 2ms
	OpenTimer2( T2_ON | T2_SOURCE_INT | T2_PS_1_256, 233);// 585);		// approx 2 ms interrupt
	
	// Timer2 as configured would trigger an interrupt at a frequency of (80MHZ/256/65535), or 4.77
	// times a second.
	
	// This statement configured the timer to produce an interrupt with a priority of 3
	ConfigIntTimer2( T2_INT_ON | T2_INT_PRIOR_3);
}

static void LampOn(int lamp)
{
	if (lamp < LAMP_COUNT + 1 && lamp_ports[lamp].bits)
		PORTSetBits(lamp_ports[lamp].port, lamp_ports[lamp].bits);
}

static void LampOff(int lamp)
{
	if (lamp < LAMP_COUNT + 1 && lamp_ports[lamp].bits)
		PORTClearBits(lamp_ports[lamp].port, lamp_ports[lamp].bits);
}

static void SetLamp(int lamp, BOOL on)
{
	if (lamp < LAMP_COUNT + 1 && lamp_ports[lamp].bits)
	{
		if (on)
			PORTSetBits(lamp_ports[lamp].port, lamp_ports[lamp].bits);
		else
			PORTClearBits(lamp_ports[lamp].port, lamp_ports[lamp].bits);
	}
}

void LampInit(void)
{
	int ndx, lamp;
	PROGRAM *cProg;

	softDim = FALSE;
	runFast = FALSE;
	programRun = FALSE;
	// for Odyssey, setup the motor outputs
#if defined(ODYSSEY)
	PORTSetPinsDigitalOut(MotorUp.port, MotorUp.bits);
	PORTClearBits(MotorUp.port, MotorUp.bits);
	PORTSetPinsDigitalOut(MotorDown.port, MotorDown.bits);
	PORTClearBits(MotorDown.port, MotorDown.bits);
	PORTSetPinsDigitalOut(OdyLampEnable.port, OdyLampEnable.bits);
	PORTClearBits(OdyLampEnable.port, OdyLampEnable.bits);
	needSignUp = FALSE;
#endif

	// stop all programs
	for(ndx = 0; ndx < Programs.count; ndx++)
	{
		cProg = &Programs.progs[ndx];
		cProg->Running = FALSE;
	}
	for(lamp = 0; lamp < LAMP_COUNT; lamp++)
	{
		if (lamp_ports[lamp].bits)		// protection
			PORTSetPinsDigitalOut(lamp_ports[lamp].port, lamp_ports[lamp].bits);
		LampOff(lamp);
		lampIsDim[lamp] = FALSE;	// dimming logic bit
		lampStatus[lamp] = FALSE;	// current lamp status on/off
	}
	// turn off all the lamp outpus as per Rinaldo
	PORTSetPinsDigitalOut(LampEnable.port, LampEnable.bits);
	PORTClearBits(LampEnable.port, LampEnable.bits);
	EnableTimer2();
}

/****************************************************************************
  Function:
    void __attribute__((__interrupt__, auto_psv)) _T2Interrupt(void)

  Description:
    Timer ISR, used for PWM for brightness
  Precondition:
    None

  Return Values:
    None

  ***************************************************************************/

void __ISR(_TIMER_2_VECTOR, ipl3) handlesTimer2Ints(void)
{
	int i, prNdx, pNdx, lNdx;
	PROGRAM *cProg;
	LAMP_PROGRAM *lProg;
	LAMP_GROUP *lGroup;
	LAMP_SEQ *lSeq;
	int lmp;
	BOOL running = FALSE;
//	LAMP *lmp;

	for(prNdx = 0; prNdx < Programs.count; prNdx++)
	{
		cProg = &Programs.progs[prNdx];
		if (cProg->Running)
		{
			running = TRUE;
			for(pNdx = 0; pNdx < cProg->count; pNdx++)
			{
				lProg = &cProg->lamp_programs[pNdx];
				lProg->CurrentDelay += 2;		//  2 ms time base
				if (lProg->CurrentDelay > lProg->NextDelay)
				{
					lGroup = &lProg->lamp_groups[lProg->PatternNdx];
	//	            PatternSimple ps = pc.Patterns[pc.PatternNdx];
		            if (lProg->LampNdx >= lGroup->count)	// pc.LampNdx >= ps.Lamps.Length)
		            {
		                if (++lProg->RepeatCnt >= lGroup->rep)
		                {
		                    lProg->RepeatCnt = 0;
		                    if (++lProg->PatternNdx >= lProg->count)
		                        lProg->PatternNdx = 0;
							lGroup = &lProg->lamp_groups[lProg->PatternNdx];
		                }
		                lProg->LampNdx = 0;
		            }
					lSeq = &lGroup->lamp_seqs[lProg->LampNdx++];
		            //LampSimple ls = ps.Lamps[pc.LampNdx++];
		
					for(lNdx = 0; lNdx < lSeq->count; lNdx++)
					{
						lmp = lSeq->lamps[lNdx].lamp;
						if(cProg->LampEn[lmp])		// lower priority check
						{
							// if this program is dimmed or dim input is high and this lamp can be dimmed, then dim it
							// added software dimming
							if ((ana_inputs[DIM_INPUT] == HIGH || cProg->Dimmed || softDim) && Programs.dimmable[lmp])
								lampIsDim[lmp] = lSeq->lamps[lNdx].on;		// lamp is dim, just set a flag
							else
								lampIsDim[lmp] = FALSE;		// it's not dim
							lampStatus[lmp] = lSeq->lamps[lNdx].on;
						}
					}
					if (runFast && cProg->CanRunFast)		// speed up this pattern?
						lProg->NextDelay = lSeq->delay / 2;	// twice as fast for now!	
					else		
						lProg->NextDelay = lSeq->delay;	
					lProg->CurrentDelay = 0;
				}
			}
		}
	}
	// set the lamps to there appropriate on/off values
	for(i = 0; i < LAMP_COUNT; i++)
		if (! lampIsDim[i])
			SetLamp(i, lampStatus[i]);
	// only if something is running, start the dimming timer
	if (running)
	{
		// dimming logic
		// use a pwm for this!
		if (dimDelay == Programs.dimLevel + 1)
		{
			for(i = 0; i < LAMP_COUNT; i++)
				if (lampIsDim[i])
					LampOff(i);
		}
		else
		{
			if (dimDelay >= 12)
			{
				for(i = 0; i < LAMP_COUNT; i++)
					if (lampIsDim[i])
						LampOn(i);
				dimDelay = 0;
			}
		}
		dimDelay++;
	}

	mT2ClearIntFlag();

} // END Timer2 ISR

/*
	lamp dimming
	timer runs from the timer 2 interrupt above
*/
void __ISR(_TIMER_1_VECTOR, ipl4) handlesTimer1Ints(void)
{
	int i;

	for(i = 0; i < LAMP_COUNT; i++)
		if (lampIsDim[i])
			LampOff(i);
	mT1ClearIntFlag();
	CloseTimer1();		// stop dimming logic from running
}

static void CalcPriority()
{
	int pri, lmp, pgm;
	PROGRAM *cProg;
#if defined(ODYSSEY)
	// for Odyssey, calculate if the sign board needs to be up or not
	BOOL signLamp = FALSE;
#endif

	// check all programs that are running
	// note: += 2 because modules contain two lamps each!
	for(lmp = 0; lmp < LAMP_COUNT; lmp += 2)
	{
		pri = -1;
		// find the highest priority number for this lamp module
		for(pgm = 0; pgm < Programs.count; pgm++)
		{
			cProg = &Programs.progs[pgm];
			cProg->LampEn[lmp] = FALSE;			// disable one 1/2 of the module
			cProg->LampEn[lmp + 1] = FALSE;		// disable the other 1/2
			if (cProg->Running && (cProg->LampList[lmp] || cProg->LampList[lmp + 1]))
			{
				// must make sure other lamps are off
				if (cProg->LampList[lmp])
				{
					lampIsDim[lmp] = FALSE;
					lampStatus[lmp] = FALSE;
#if defined(ODYSSEY)
// for Odyssey, check if there are any sign board lamps
					if (lmp >= 44)		// Lamp module 23 and > are Odyssey
						signLamp = TRUE;
#endif
				}
				if (cProg->LampList[lmp+1])
				{
					lampIsDim[lmp+1] = FALSE;
					lampStatus[lmp+1] = FALSE;
				}
				
				if (cProg->Priority > pri)
					pri = cProg->Priority;
			}
		}
		// is this lamp on anywhere?
		if (pri != -1)
		{
			// loop through again to set the lamp with the highest priority
			for(pgm = 0; pgm < Programs.count; pgm++)
			{
				cProg = &Programs.progs[pgm];
				// set the highest priority lamp we can find
				if (cProg->Running && (cProg->LampList[lmp] || cProg->LampList[lmp + 1]) && cProg->Priority == pri)
				{
 					if (cProg->LampList[lmp])
					{
						lampIsDim[lmp] = FALSE;
						lampStatus[lmp] = FALSE;	// make sure it turns off
						cProg->LampEn[lmp] = TRUE;
					}
					if (cProg->LampList[lmp + 1])
					{
						lampIsDim[lmp+1] = FALSE;
						lampStatus[lmp+1] = FALSE;
						cProg->LampEn[lmp + 1] = TRUE;
					}
					break;
				}
			}
		}
	}
#if defined(ODYSSEY)
	// this will turn on the sign up motor in MotorTasks() below if need be
	needSignUp = signLamp;
#endif

	IEC0bits.T2IE = 1;
//	EnableTimer2();		// restart logic
}

void StopProgram(int pgm)
{
	int i, prNdx;
	BOOL run;
	PROGRAM *cProg;

	cProg = &Programs.progs[pgm];
	if (cProg->Running)		// ignore if not running
	{
		IEC0bits.T2IE = 0;
		cProg->Running = FALSE;
		for(i = 0; i < LAMP_COUNT; i++)
		{
//			if(cProg->LampEn[i])		// if this lamp enabled to run
			{
				lampIsDim[i] = FALSE;	// dimming logic bit
				lampStatus[i] = FALSE;
				LampOff(i);
			}
		}
		// recalculate lamp priority
		CalcPriority();
	}

	// check if anything is still running, if not, disable all lamps!
	run = FALSE;
	for(prNdx = 0; prNdx < Programs.count; prNdx++)
	{
		cProg = &Programs.progs[prNdx];
		if (cProg->Running)
		{
			run = TRUE;
			break;
		}
	}
	programRun = run;
#if defined(ODYSSEY)
	if (! run)	// nothing is still running
		PORTClearBits(LampEnable.port, LampEnable.bits);		// as per Rinaldo, required now to disable all lamps
#endif
}

BOOL StartProgram(int pgm, int pri, BOOL dim, BOOL canfast)
{
	int i;
	BOOL rc = FALSE;
	PROGRAM *cProg;

	if (pgm < Programs.count)		//is it valid?
	{
		rc = TRUE;
		cProg = &Programs.progs[pgm];		// pointer to current program
		if (! cProg->Running)		// ignore if already running
		{
			IEC0bits.T2IE = 0;
//			DisableTimer2();		// stop logic from running
			cProg->Dimmed = dim;
			cProg->Priority = pri;
			cProg->CanRunFast = canfast;
			for(i = 0; i < cProg->count; i++)
			{
				cProg->lamp_programs[i].PatternNdx = 0;
				cProg->lamp_programs[i].LampNdx = 0;
				cProg->lamp_programs[i].RepeatCnt = 0;
				cProg->lamp_programs[i].NextDelay = 0;
				cProg->lamp_programs[i].CurrentDelay = 0;
			}
			cProg->Running = TRUE;
			CalcPriority();
			PORTSetBits(LampEnable.port, LampEnable.bits);		// as per Rinaldo, required now to enable all lamps
			programRun = TRUE;
		}
	}	
	return rc;
}

#if defined(ODYSSEY)

/*
	only for Odyssey
	handle the motor up/down logic
*/
void MotorTasks(void)
{
	if (needSignUp)
	{
		if (motorState)
		{
			PORTClearBits(MotorDown.port, MotorDown.bits);
			PORTSetBits(MotorUp.port, MotorUp.bits);
			PORTSetBits(OdyLampEnable.port, OdyLampEnable.bits);	// we don't want the odyssey lamp power anymore
			motorState = 0;
		}
	}
	else
	{
		switch(motorState)
		{
			case 0:
				PORTClearBits(MotorUp.port, MotorUp.bits);
				PORTSetBits(MotorDown.port, MotorDown.bits);
				motorTick = GetTickCount();
				motorState++;
				break;
			case 1:
				if (HasElapsed(motorTick, 35000))		// 35 seconds for now
				{
					PORTClearBits(MotorUp.port, MotorUp.bits);
					PORTClearBits(MotorDown.port, MotorDown.bits);
					PORTClearBits(OdyLampEnable.port, OdyLampEnable.bits);	// we don't want the odyssey lamp power anymore
					motorState++;
				}
				break;
			case 2:
				if (! programRun)
					PORTClearBits(LampEnable.port, LampEnable.bits);	// we don't want the odyssey lamp power anymore
				break;
		}
	}
}

#endif
