/*
	lamps.h
*/
#ifndef __LAMPS_H
#define __LAMPS_H

#if defined(ODYSSEY)
// for Odyssey it's really 58 lights, the alley lights are doubled up and the 16 Odyssey bar lamps are doubled up for numbering
#define		LAMP_COUNT	76
#define		MAX_DIMMABLE	76	
#define		MAX_INPUTS	4
#else
#define		LAMP_COUNT	48
#define		MAX_DIMMABLE	48
#define		MAX_INPUTS	14
#endif

#define		MAX_CYCLE	15

enum InpState
{
	MOMENTARY,
	TOGGLE,
	TIMER
};

typedef struct
{
	BOOL on;		// turn on/off
	int lamp;		// lamp index
} LAMP;

typedef struct
{
	int count;			// number of lamps
	int delay;			// delay in ms
	LAMP	*lamps;	// lamp array
} LAMP_SEQ;

typedef struct
{
	int count;			// number of sequences
	int rep;			// repeat count for this lamp group
	LAMP_SEQ *lamp_seqs;	// lamp sequences
} LAMP_GROUP;

typedef struct
{
	int count;		// lamp groups in this program
	int PatternNdx;
	int LampNdx;
	int RepeatCnt;
	int NextDelay;
	int CurrentDelay;
	LAMP_GROUP *lamp_groups;	// array of lamp sequences
} LAMP_PROGRAM;

typedef struct
{
	int count;			// number of lamp programs
	BOOL Running;		// this program is running

	BOOL Dimmed;		// running dimmed
	BOOL CanRunFast;	// this function can run fast
	int Priority;		// running priority of this program
	int lampCnt;		// total lamps used by this program all groups
	BOOL LampEn[LAMP_COUNT];	// used for priority
	BOOL LampList[LAMP_COUNT];	// list of all lamps in all groups
	LAMP_PROGRAM *lamp_programs;	// array of lamp programs
} PROGRAM;

typedef struct
{
	int count;
	int frontLamps;
	int rearLamps;
	int dimLevel;
	int dimCount;
	BOOL dimmable[MAX_DIMMABLE];
	PROGRAM *progs;		// array of programs
} PROG_LIST;

typedef struct
{
	int ProgIndexes[MAX_CYCLE];		// program number
	enum InpState InpType;	// 0-momentary, 1-toggle, 2-timer
	int TimerValue;		// from windows app
	int RunTimer;		// current active running timer 
	int Priority;
	BOOL Dimmed;
	int Level;
	BOOL IsOn;		// input state
	BOOL Toggle;	// used by toggle input type
} PROG_INPUT;		// program from input 

extern PROG_LIST Programs;
extern PROG_INPUT *ProgInputs;		// program inputs for stand-alone bar

void LampInit(void);
void StopProgram(int pgm);
BOOL StartProgram(int pgm, int pri, BOOL dim, BOOL canfast);
#if defined(ODYSSEY)
void MotorTasks(void);
#endif

extern volatile BOOL softDim;		// software dimmer
extern volatile BOOL runFast;		// run fast patterns

#endif
