/*
	leds.c flash leds in a 3 bit pattern in the background
	rb Jan 6/2010
	Copyright (c) 2010 DM&T Services Ltd.
*/
#include "Compiler.h"
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "timer.h"
#include "leds.h"

//******************************************************************************
// Constants
//******************************************************************************

#define	MAX_LED_BUF	8		// maximum of 8 different led patterns buffered
#define	LONG_DELAY	600
#define	SHORT_DELAY	200

//******************************************************************************
// Local Variables
//******************************************************************************

static BYTE ledBuf[MAX_LED_BUF];
static int ledHead = 0;
static int ledTail = 0;
static int ledState = 0;
static BYTE ledBit;
static BYTE ledCurr;
static long ledTick;

void LEDInit()
{
	TRISGbits.TRISG8 = 0;		// output
	LED_1 = 0;					// off
}

/****************************************************************************
  Function:
	void BlinkLED(BYTE pattern)

  Description:
    Blinks a morse code style 3 blip message

  Parameters:
    pattern will be blinked in the background 3 bits long

  Return Values:
    None

  Remarks:
    None
***************************************************************************/

void BlinkLED(BYTE pattern)
{
	ledBuf[ledHead++] = pattern;
	if(ledHead >= MAX_LED_BUF)
		ledHead = 0;
}

void ClearLEDState()
{
	ledState = 0;
}

/****************************************************************************
  Function:
	void LEDTasks(void)

  Description:
    Do all the work here

  Parameters:
    None

  Return Values:
    None

  Remarks:
    None
***************************************************************************/

void LEDTasks(void)
{
	int delay;

	switch(ledState)
	{
		case 0:
			if(ledHead != ledTail)
			{
				ledState++;
				ledBit = 4;
				ledTick = GetTickCount();
				LED_1 = 1;		// LED on
				ledCurr = ledBuf[ledTail++];
				if(ledTail >= MAX_LED_BUF)
					ledTail = 0;
			}
			break;
		case 1:
			if(ledBit & ledCurr)
				delay = LONG_DELAY;
			else
				delay = SHORT_DELAY;
			if(HasElapsed(ledTick, delay))
			{
				LED_1 = 0;			// LED off
				ledTick = GetTickCount();
				ledState++;
				ledBit >>= 1;
			}
			break;
		case 2:
			if(HasElapsed(ledTick, SHORT_DELAY))
			{
				ledTick = GetTickCount();
				if(ledBit)
				{
					LED_1 = 1;		// LED on
					ledState--;
				}
				else
					ledState++;
			}
			break;
		case 3:
			if(HasElapsed(ledTick, LONG_DELAY * 2))
				ledState = 0;
			break;
	}	
}
