/********************************************************************
 FileName:     main.c
 Dependencies: See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  1.0   Initial release
  2.1   Updated for simplicity and to use common
                     coding style
********************************************************************/
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "timer.h"
#include "lamps.h"
#include "comm.h"
#include "thumbdrive.h"
#include "analog.h"
#include "leds.h"

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************
#pragma config UPLLEN   = ON            // USB PLL Enabled
#pragma config FPLLMUL  = MUL_15        // PLL Multiplier
#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN   = ON           // Watchdog Timer
#pragma config WDTPS    = PS1024           // Watchdog Timer Postscale, one second timeout
//#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
   
int siz;

int main(void)
{
	int  value;
//	int tick;
	int i;
	BOOL resetFlag;
	int resetTick;

	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
	
	// Enable the cache for the best performance
	CheKseg0CacheOn();

	INTEnableSystemMultiVectoredInt();

	value = OSCCON;
	while (!(value & 0x00000020))
	{
		 value = OSCCON;    // Wait for PLL lock to stabilize
	}
//	AD1PCFG = 0xffff;		// make all analog inputs be digial lines

// disable the JTAG port, solves RA1,RA2,RA3 issues
	mJTAGPortEnable(0);
	Timer3Init();
	AnalogInit();
	LEDInit();
	LampInit();		// set lamp outputs
	CommInit();		// init Uart
	ClearWDT();

    // Enable multi-vector interrupts
    INTEnableInterrupts();

	BlinkLED(0);
	resetFlag = FALSE;

	while(1)
	{
		// monitor communications
		// force reset if it goes down!
		if (resetFlag)
		{
			if (! onLine && watchReset)
			{
				if (HasElapsed(resetTick, 2000))
				{
					while(1);		// force watchdog reset!
				}
			}
			else
			{
				resetTick = GetTickCount();
				onLine = FALSE;
			}
		}
		else
		{
			if (onLine)
				resetFlag = TRUE;
		}
		if (forceReset)
			while(1);		// force watchdog reset!


		ClearWDT();
		CommTasks();
		ThumbTasks();
		AnalogTasks();
		LEDTasks();
#if defined(ODYSSEY)
		// handle motor up/down
		MotorTasks();
#endif
	}

/*	while(1)
	{
		tick = GetTickCount();
		while(! HasElapsed(tick, 2000));

		for(i = 0; i < 36; i++)
		{
			LampOn(i);
			tick = GetTickCount();
			while(! HasElapsed(tick, 300));
		}

		for(i = 0; i < 36; i++)
		{
			LampOff(i);
			tick = GetTickCount();
			while(! HasElapsed(tick, 300));
		}

	} 
*/
    return 0;
}


