/*
	main.c
*/
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "timer.h"
#include "sound.h"
#include "comm.h"
#include "analog.h"
#include "lamps.h"
#include "flash.h"

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************
#pragma config UPLLEN   = ON            // USB PLL Enabled
#pragma config FPLLMUL  = MUL_15        // PLL Multiplier
#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
#pragma config FPLLIDIV = DIV_2         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN   = ON          // Watchdog Timer
#pragma config WDTPS    = PS1024           // Watchdog Timer Postscale, about 1 sec timeout
//#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
   
int main(void)
{
	int  value;
	int resetTick;
	int i;
	BOOL resetFlag;

	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
	
	// Enable the cache for the best performance
	CheKseg0CacheOn();

	INTEnableSystemMultiVectoredInt();

	value = OSCCON;
	while (!(value & 0x00000020))
	{
		 value = OSCCON;    // Wait for PLL lock to stabilize
	}
// disable the JTAG port, solves RA1,RA2,RA3 issues
	mJTAGPortEnable(0);
	Timer1Init();
	AnalogInit();
	SoundInit();
	LampInit();
	CommInit();
	ClearWDT();

    // Enable multi-vector interrupts
    INTEnableInterrupts();
	resetFlag = FALSE;

	i = 0;
	while(1)
	{
//				PORTSetBits(outputs[0].port, outputs[0].bits);
/*		if (i)
		{
			if (HasElapsed(tick, 4000))
			{
				PORTClearBits(outputs[0].port, outputs[0].bits);
				tick = GetTickCount();
				i = 0;
			}
		}
		else
		{
			if (HasElapsed(tick, 10000))
			{
				PORTSetBits(outputs[0].port, outputs[0].bits);
				tick = GetTickCount();
				i = 1;
			}
		} */
		// monitor communications
		// force reset if it goes down!
#if ! defined(__DEBUG)
		if (resetFlag)
		{
			if (! onLine && watchReset)
			{
				if (HasElapsed(resetTick, 1000))
				{
					while(1);		// force watchdog reset!
				}
			}
			else
			{
				resetTick = GetTickCount();
				onLine = FALSE;
			}
		}
		else
		{
			if (onLine)
				resetFlag = TRUE;
		}		
		if (forceReset)
			while(1);		// force watchdog reset!
#endif

		ClearWDT();
		SoundTasks();
		CommTasks();
		AnalogTasks();
	}

    return 0;
}


