/*
	sound.c
*/
#include "Compiler.h"
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "sound.h"
#include "comm.h"

#define	HORN_RELAY			LATGbits.LATG0
#define	HORN_RELAY_TRIS		TRISGbits.TRISG0
#define	DATA_A				LATDbits.LATD6
#define	DATA_A_TRIS			TRISDbits.TRISD6
#define	DATA_B				LATDbits.LATD5
#define	DATA_B_TRIS			TRISDbits.TRISD5
#define	SIREN				LATDbits.LATD4
#define	SIREN_TRIS			TRISDbits.TRISD4
#define	SIREN2				LATDbits.LATD11
#define	SIREN2_TRIS			TRISDbits.TRISD11
#define	MIC_C				PORTDbits.RD7

/* globals */
BOOL AirHornBit[2];
BOOL ManualBit[2];
BOOL ManualStop[2];
int SirenControl[2];
int toneMode[2];
BOOL ManualStart[2];
BOOL StandByMode;
int StandByTone;

/* locals */
static int hornOffDelay;
static BOOL micIsOn;
static int micTick;
static BOOL t4Flag;
static int AirHornCounter[2];
static int SirenFStore[2];		// fixed freq offset
static int SirenFShift[2];		// amount to shift freq
static int SirenSpeed[2];
static int Timer_Count_Up_10ms;
static int toneCounter[2];
static int tone[2];
static int SirenFPoint[2];
static const int AirHornTable[] = {
454,454,234,234,144,401,234,234,144,144,234,144,144,144,122,99 };

static int FreqTable[255] = {
747,715,691,671,647,624,603,587,569,559,541,524,507,495,481,468,461,
453,447,437,426,413,413,401,396,392,380,373,369,360,356,353,349,342,
339,332,329,321,318,314,312,308,305,297,293,289,285,281,281,277,272,
269,269,265,265,258,255,252,252,248,245,245,237,234,234,230,230,228,
228,224,224,217,217,217,217,213,213,210,210,206,206,206,206,203,203,
197,197,197,193,193,189,189,186,186,181,177,174,174,174,173,173,173,
169,169,169,165,165,165,161,161,157,157,153,153,153,153,153,150,150,
150,150,150,146,146,146,146,146,144,144,144,144,144,140,140,140,140,
137,137,137,137,133,133,133,133,129,129,129,129,129,129,126,126,126,
126,126,126,122,122,122,122,122,122,122,119,119,119,119,116,116,116,
116,116,113,113,113,113,113,109,109,109,109,109,113,113,119,126,133,
144,150,161,169,174,186,193,197,203,206,213,217,228,230,237,248,255,
258,269,277,285,289,297,308,314,318,329,336,342,353,360,369,377,384,
396,401,416,423,437,444,457,464,479,485,495,504,516,524,537,545,559,
569,583,590,603,615,627,643,656,674,691,708,727,735,744,744,751,751 };

/*
	initialize the Output Comparator 2 OC2 for continuous output
*/
void SoundInit()
{
	int i, j;

	CNPUEbits.CNPUE16 = 1;		// internal pull-up on MIC_C input
	HORN_RELAY_TRIS = 0;		// assign outputs
	HORN_RELAY = 0;				// Horn relay off
	SIREN_TRIS = 0;
	SIREN2_TRIS = 0;
	DATA_A_TRIS = 0;
	DATA_B_TRIS = 0;
	SelectDevice(0, NO_DEVICE);
	SelectDevice(1, NO_DEVICE);
	micIsOn = FALSE;
	hornOffDelay = 0;
	AirHornCounter[0] = AirHornCounter[1] = 0;
	SirenFPoint[0] = SirenFPoint[1] = 0;

	// output comparator setup for siren output OC2

	T2CON = 0x0050; // Configure Timer2 for 16-bit operation
					// with a prescaler of 32. The Timer2/Timer3
					// pair is accessed via registers associated
					// with the Timer2 register
	OC2CON = 0x3;	// bits 0-2, 011 = toggle output mode
					// bit 3 zero is Timer2 is clock source
					// 16 bit timer source
					// 
	OC2R = 2000; 	// Initialize Compare Register 2
	PR2 = 2000; 	// Set period 

	// for siren output 2
	T3CON = 0x0050; // Configure Timer3 for 16-bit operation
					// with a prescaler of 32. The Timer2/Timer3
					// pair is accessed via registers associated
					// with the Timer2 register
	OC1CON = 0xb;	// bits 0-2, 011 = toggle output mode
					// bit 3 one Timer3 is clock source
					// 16 bit timer source
					// 
	OC1R = 2000; 	// Initialize Compare Register 1
	PR3 = 2000; 	// Set period 


    SirenFShift[0] = SirenFShift[1] = 84;  
    SirenSpeed[0] = SirenSpeed[1] = 30;   //30 = wail   
	SirenFStore[0] = SirenFStore[1] = 187;
	SirenFPoint[0] = SirenFPoint[1] = 100;
	T2CONCLR = 0x8000; // disable Timer2
	OC2CONSET = 0x8000; // Enable OC2
	T3CONCLR = 0x8000; // disable Timer3
	OC1CONSET = 0x8000; // Enable OC1
	SirenControl[0] = SirenControl[1] = 0;
	toneMode[0] = toneMode[1] = MODE_RADIO;
	StandByTone = 0;
	StandByMode = FALSE;
	AirHornBit[0] = AirHornBit[1] = FALSE;
	ManualBit[0] = ManualBit[1] = FALSE;
	ManualStop[0] = ManualStop[1] = FALSE;

	// highest priority int. so far
	ConfigIntOC2(OC_INT_ON | OC_INT_PRIOR_6);

	// highest priority int. so far
	ConfigIntOC1(OC_INT_ON | OC_INT_PRIOR_5);
	
	// This statement says: turn on timer4 | have it use an internal clock source | have it
	// use a prescaler of 1:1, and use a period of 15,000 for 2000 interrupts / sec or every 500us
	OpenTimer4( T4_ON | T4_SOURCE_INT | T4_PS_1_1, 14999);
	
	// Timer4 as configured would trigger an interrupt at a frequency of 2000 hz or every 500us
	// This statement configured the timer to produce an interrupt with a priority of 2
	ConfigIntTimer4( T4_INT_ON | T3_INT_PRIOR_3);
}

void __ISR(_OUTPUT_COMPARE_2_VECTOR, ipl6) handlesOutputCompare2Ints(void)
{
	T2CONCLR = 0x8000; // disable Timer2

	if (AirHornBit[0])		// air horn
	{
		SirenFStore[0] = 970;		// was 10
		AirHornCounter[0] &= 0xf;	// make sure it's valid 0-15
		SirenFShift[0] = AirHornTable[AirHornCounter[0]++]; 
	}
	OC2R = SirenFStore[0] + SirenFShift[0];
	PR2 = SirenFStore[0] + SirenFShift[0];
	T2CONSET = 0x8000; // enable Timer2

	mOC2ClearIntFlag();
}


void __ISR(_OUTPUT_COMPARE_1_VECTOR, ipl5) handlesOutputCompare1Ints(void)
{
	T3CONCLR = 0x8000; // disable Timer3

	if (AirHornBit[1])		// air horn
	{
		SirenFStore[1] = 970;		// was 10
		AirHornCounter[1] &= 0xf;	// make sure it's valid 0-15
		SirenFShift[1] = AirHornTable[AirHornCounter[1]++]; 
	}
	OC1R = SirenFStore[1] + SirenFShift[1];
	PR3 = SirenFStore[1] + SirenFShift[1];
	T3CONSET = 0x8000; // enable Timer3

	mOC1ClearIntFlag();
}

static void HornRelay(BOOL on)
{
	if (on)
	{
		hornOffDelay = 0;
		HORN_RELAY = 1;
	}
	else
	{
		if (HORN_RELAY && ! hornOffDelay)
			hornOffDelay = 400;		// 400 = 200 ms
	}
}

void Siren_On(int ndx, int SirenSpeed_New_Value)
{
	SirenSpeed[ndx]--;
	if(SirenSpeed[ndx] == 0)
	{
//		SirenFShift = 69;  // fix value
		SirenFShift[ndx] = 500;  // fix value
		SirenSpeed[ndx] = SirenSpeed_New_Value;   //30 = wail   2 = yelp  (2= stinger with calling the siren on 3 times)

		SirenFPoint[ndx]++;
		if(SirenFPoint[ndx] >= 251)
			SirenFPoint[ndx] = 0;

		SirenFStore[ndx] = FreqTable[SirenFPoint[ndx]];
	}
}

void Siren_On_Hi_Lo(int ndx)
{
	SirenFPoint[ndx] &= 0x7f;		// max 128

	if(Timer_Count_Up_10ms == 1)
	{
		SirenFPoint[ndx]--;
		if(SirenFPoint[ndx] == 0)
		{
			SirenFPoint[ndx] = 100;
		//setlofrhl
			SirenFShift[ndx] = 253;  // fix value
			SirenFStore[ndx] = 1400;
		}
		else
		{
		//chktonhl
			if(SirenFPoint[ndx] > 50)
			{
			//setlofrhl
				SirenFShift[ndx] = 253;  // fix value
				SirenFStore[ndx] = 1400;		// was 1000
			}
			else
			{
			//sethifrhl
				SirenFShift[ndx] = 253;  // fix value
				SirenFStore[ndx] = 1100;		// was 360
			}
		}
	}
}

void __ISR(_TIMER_4_VECTOR, ipl3) handlesTimer4Ints(void)
{
	t4Flag = TRUE;
	mT4ClearIntFlag();
} // END Timer4 ISR

static void Stinger(int ndx)
{
	Siren_On(ndx, 4);
	Siren_On(ndx, 4);
	Siren_On(ndx, 4);
	SelectDevice(ndx, WAVE_DEVICE);
}

static void Yelp(int ndx)
{
	Siren_On(ndx, 4);
	SelectDevice(ndx, WAVE_DEVICE);
}

static void Quad(int ndx)
{
	Siren_On(ndx, 4);
	Siren_On(ndx, 4);
	Siren_On(ndx, 4);
	Siren_On(ndx, 4);
	SelectDevice(ndx, WAVE_DEVICE);
}

static void Wail(int ndx)
{
	Siren_On(ndx, 70);
	SelectDevice(ndx, WAVE_DEVICE);
}

void CancelStandbyTone(int ndx, BOOL kill)
{
	if (StandByMode)
	{	
		if (ndx == 0)		// only if we started this via horn ring
		{
			if (StandByTone == 3)	// air horn going off?
			{
				AirHornBit[0] = FALSE;
				AirHornBit[1] = FALSE;
				SelectDevice(0, STANDBY_DEVICE);
				SelectDevice(1, STANDBY_DEVICE);
			}
			else
			{
				if (ManualBit[0])
				{
					ManualBit[0] = FALSE;
					ManualStop[0] = TRUE;	// manual going off
					ManualBit[1] = FALSE;
					ManualStop[1] = TRUE;	// manual going off
				}
				else
				{
					if (! ManualStop[0] || kill)
					{
						ManualStop[0] = FALSE;
						ManualStop[1] = FALSE;
						SelectDevice(0, STANDBY_DEVICE);
						SelectDevice(1, STANDBY_DEVICE);
					}
				}
			}
		}
	}
}

void Siren_Main(int ndx)
{
	// new standby mode
	if(ndx == 0 && StandByMode)
	{
		HornRelay(TRUE);		// must be on in standby!
		if (HornRing())
		{
			if (StandByTone == 3)	// air horn going on?
			{
				AirHornBit[0] = TRUE;
				AirHornBit[1] = TRUE;
			}
			else
			{
				if(! ManualBit[0])
				{
					// start first manual output
					ManualStart[0] = TRUE;
					ManualStop[0] = FALSE;
					ManualBit[0] = TRUE;
					// same for 2nd siren output
					ManualStart[1] = TRUE;
					ManualStop[1] = FALSE;
					ManualBit[1] = TRUE;
				}
			}
			SIREN = 1;
			SIREN2 = 1;		// make sure both AMPs are on
		}
		else
			CancelStandbyTone(ndx, FALSE);
	}
//		else
//		{
//		}
//	}
	if(AirHornBit[ndx]) // AIR_HORN
	{
		SelectDevice(ndx, MANUAL_DEVICE);
	}
	else if(ManualBit[ndx]) // MANUAL
	{
		if(ManualStart[ndx])
		{
			ManualStart[ndx] = FALSE;
			SirenFPoint[ndx] = 0;	//start fresh from the begining every time
		}
		
		Siren_On(ndx, 70);		// wail setup
		
		SelectDevice(ndx, MANUAL_DEVICE);
	}
	else
	{	
		if (! ManualStop[ndx])
		{
			switch(SirenControl[ndx])
			{
				case SIREN_OFF:
					SelectDevice(ndx, NO_DEVICE);
					break;
	
				case RADIO:
					if(toneMode[ndx] == MODE_RADIO)
					{
						SelectDevice(ndx, RADIO_DEVICE);
						toneCounter[ndx] = 0;
						tone[ndx] = 0;
					}
					else if(toneMode[ndx] == MODE_MULTITONE)
					{
						toneCounter[ndx]++;
						switch(tone[ndx])
						{
							case 0:
									if(toneCounter[ndx] > TIME_TO_CHANGE_TONE)
									{
										toneCounter[ndx] = 0;
										tone[ndx] = 1;
										Yelp(ndx);
									}
									else
									{
										Wail(ndx);
									}
									break;
							case 1:
									if(toneCounter[ndx] > TIME_TO_CHANGE_TONE)
									{
										toneCounter[ndx] = 0;
										tone[ndx] = 2;
										Stinger(ndx);
									}
									else
									{
										Yelp(ndx);
									}
									break;
							case 2:
									if(toneCounter[ndx] > TIME_TO_CHANGE_TONE)
									{
										toneCounter[ndx] = 0;
										tone[ndx] = 0;
										Wail(ndx);
									}
									else
									{
										Stinger(ndx);
									}
									break;
							}
						}
		
					break;
	
				case WAIL:
					Wail(ndx);
					break;
				case YELP:
					Yelp(ndx);
					break;
				case STINGER:
					Stinger(ndx);
					break;
				case HI_LO:
					Siren_On_Hi_Lo(ndx);
					SelectDevice(ndx, WAVE_DEVICE);
					break;
			  	default:
					SelectDevice(ndx, NO_DEVICE);
	    	        break;
			} // end of switch
		}
	}
}

void SelectDevice(int ndx, DEVICE dev)
{
	// 2nd siren output?
	if (ndx)
	{
		switch(dev)
		{
			case STANDBY_DEVICE:
				T3CONCLR = 0x8000; // disable Timer3
				SIREN2 = 0;
				break;
			case NO_DEVICE:
				T3CONCLR = 0x8000; // disable Timer3
				SIREN2 = 0;
				break;
			default:
				T3CONSET = 0x8000; // enable Timer3
				SIREN2 = 1;
				break;
		}
	}
	else
	{
		switch(dev)
		{
			case NO_DEVICE:
				T2CONCLR = 0x8000; // disable Timer2
				DATA_A = 0;
				DATA_B = 0;
				if (! StandByMode)
					HornRelay(FALSE);		// horn relay off
				SIREN = 0;
				break;
			case WAVE_DEVICE:
				DATA_A = 1;
				DATA_B = 0;
				T2CONSET = 0x8000; // enable Timer2
				HornRelay(TRUE);		// horn relay on
				SIREN = 1;
				break;
			case MANUAL_DEVICE:		// same as wave device but no HORN_RELAY
				DATA_A = 1;
				DATA_B = 0;
				T2CONSET = 0x8000; // enable Timer2
				SIREN = 1;
				break;
			case MIC_DEVICE:
				DATA_A = 0;
				DATA_B = 1;
				T2CONSET = 0x8000; // enable Timer2
				HornRelay(TRUE);		// horn relay on
				SIREN = 1;
				break;
			case RADIO_DEVICE:
				DATA_A = 1;
				DATA_B = 1;
				T2CONSET = 0x8000; // enable Timer2
				HornRelay(TRUE);		// horn relay on
				SIREN = 1;
				break;
			case STANDBY_DEVICE:
				T2CONCLR = 0x8000; // disable Timer2
				HornRelay(TRUE);		// horn relay on
				DATA_A = 0;
				DATA_B = 0;
				SIREN = 0;
				break;
		}
	}
}

BOOL ManualGoingOff(int ndx)
{
	if (ManualStop[ndx])
	{
//		crap = SirenControl[ndx];
//		if(crap == NO_DEVICE)
//		{
			if(Timer_Count_Up_10ms == 1)
			{
				SirenFStore[ndx]++;
				if(SirenFStore[ndx] >= 400)
					ManualStop[ndx] = FALSE;
			}
//		}
//		else
//			ManualStop[ndx] = FALSE;	
	}

	return ManualStop[ndx];
}

void SoundTasks()
{
	if (! t4Flag)		// wait for 500us interrupt
		return;
	t4Flag = FALSE;
	if(++Timer_Count_Up_10ms >= 45)
		Timer_Count_Up_10ms = 0;
	if (hornOffDelay)
	{
		if(! --hornOffDelay)
			HORN_RELAY = 0;
	}

	if (! micIsOn)
	{
		// mic button pressed?
		if (! MIC_C)
		{
			SelectDevice(0, MIC_DEVICE);
			SelectDevice(1, NO_DEVICE);	// turn off amp 2
			micIsOn = TRUE;
			micTick = GetTickCount();
		}
		else
		{
			if(! ManualGoingOff(0))
				Siren_Main(0);		// make noise!
			// 2nd siren output
			Siren_Main(1);
		}
	}
	else
	{
		if (Repeat(&micTick, 200))		// every 200 ms check if the mic button has been released
		{
			// mic button released?
			if (MIC_C)
			{
				micIsOn = FALSE;
			}
		}
	}
}

