/*
	OLE SPI Display Test
	Part: UG2684HMBEG01
*/
#define _SUPPRESS_PLIB_WARNING 1 

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "timer.h"
#include "display.h"

/*
	constants
*/
// Character descriptors for Arial Narrow 12pt
// { [Char width in bits], [Offset into arialNarrow_12ptCharBitmaps in bytes] }
typedef struct {
	int charWidth;
	int address;
} FONT_CHAR_INFO;
	

// Character bitmaps for Arial Narrow 12pt
const BYTE fontBitmaps[] = 
{
	// @0 '!' (2 pixels wide)
	//   
	// ##
	// ##
	//   
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	0x6F, 0x6F, 
	0xFF, 0xFF, 

	// @4 '"' (4 pixels wide)
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	// #  #
	// #  #
	// #  #
	// #  #
	// #  #
	0x00, 0x00, 0x00, 0x00, 
	0x1F, 0x00, 0x00, 0x1F, 

	// @12 '#' (8 pixels wide)
	//         
	//  ## ##  
	//  ## ##  
	//  ## ##  
	//  ## ##  
	// ########
	// ########
	//   #  #  
	//   #  #  
	//   #  #  
	// ########
	// ########
	//   ## ## 
	//   ## ## 
	//   ## ## 
	//         
	0x06, 0x7E, 0x7F, 0x06, 0x7E, 0x7F, 0x06, 0x06, 
	0x30, 0x30, 0xFE, 0x3E, 0x30, 0xFE, 0x3E, 0x30, 

	// @28 '%' (14 pixels wide)
	//               
	//    ##    ###  
	//     #   ##### 
	//     ## ##   ##
	//      # ##   ##
	//      ####   ##
	//       ####  ##
	//   ### ####### 
	//  ##### # ###  
	// ##   ####     
	// ##   ## #     
	// ##   ## ##    
	// ###  ## ##    
	//  #####   #    
	//   ###    ##   
	//               
	0x00, 0x00, 0x01, 0x41, 0x71, 0x1C, 0x07, 0x1F, 0x3F, 0x63, 0x61, 0x61, 0x3F, 0x1E, 
	0x78, 0xFC, 0x8E, 0x86, 0x86, 0xFC, 0x78, 0xC0, 0x78, 0x9E, 0x82, 0x80, 0x00, 0x00, 

	// @56 '&' (11 pixels wide)
	//            
	//   #### ####
	//  ###### ## 
	// ###   ###  
	// ##    ###  
	// ##    ###  
	// ##   ##### 
	//  ## ### ## 
	//   ####     
	//    ###     
	//    ####    
	//   ##  ##   
	//   ##  ##   
	//   ##  ##   
	//   ######   
	//    ###     
	0x1E, 0x3F, 0x71, 0x60, 0x61, 0x63, 0x3F, 0x5E, 0x7F, 0x63, 0x40, 
	0x00, 0x00, 0x9E, 0xFF, 0xE3, 0xE3, 0x3E, 0x1E, 0x00, 0x00, 0x00, 

	// @78 ''' (1 pixels wide)
	//  
	//  
	//  
	//  
	//  
	//  
	//  
	//  
	//  
	//  
	//  
	// #
	// #
	// #
	// #
	// #
	0x00, 
	0x1F, 

	// @80 '*' (5 pixels wide)
	//      
	//      
	//      
	//      
	//      
	//      
	//      
	//      
	//      
	//      
	//   #  
	//  ### 
	//  ### 
	//  ####
	// ###  
	//   #  
	0x00, 0x00, 0x00, 0x00, 0x00, 
	0x02, 0x1E, 0x3F, 0x1C, 0x04, 

	// @90 '+' (10 pixels wide)
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	// ##########
	// ##########
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//           
	//           
	//           
	//           
	0x06, 0x06, 0x06, 0x06, 0xFF, 0xFF, 0x06, 0x06, 0x06, 0x06, 
	0x00, 0x00, 0x00, 0x00, 0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00, 

	// @110 '-' (4 pixels wide)
	//     
	//     
	//     
	//     
	//     
	//     
	// ####
	// ####
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	//     
	0x03, 0x03, 0x03, 0x03, 
	0x00, 0x00, 0x00, 0x00, 

	// @118 '.' (2 pixels wide)
	//   
	// ##
	// ##
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	0x60, 0x60, 
	0x00, 0x00, 

	// @122 '/' (5 pixels wide)
	//      
	// ##   
	// ##   
	//  #   
	//  #   
	//  ##  
	//  ##  
	//  ##  
	//   #  
	//   ## 
	//   ## 
	//   ## 
	//    # 
	//    # 
	//    ##
	//    ##
	0x60, 0x7F, 0x07, 0x00, 0x00, 
	0x00, 0x00, 0xF0, 0x7F, 0x03, 

	// @132 '0' (9 pixels wide)
	//          
	//   #####  
	//  ####### 
	//  ##   ## 
	// ###   ###
	// ##     ##
	// ##     ##
	// ##     ##
	// ##     ##
	// ##     ##
	// ##     ##
	// ###   ###
	//  ##   ## 
	//  ####### 
	//   #####  
	//          
	0x0F, 0x3F, 0x78, 0x60, 0x60, 0x60, 0x78, 0x3F, 0x0F, 
	0xF0, 0xFC, 0x1E, 0x06, 0x06, 0x06, 0x1E, 0xFC, 0xF0, 

	// @150 '1' (5 pixels wide)
	//      
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	//    ##
	// #####
	// #####
	//   ###
	//    ##
	//      
	0x00, 0x00, 0x00, 0x7F, 0x7F, 
	0x18, 0x18, 0x1C, 0xFE, 0xFE, 

	// @160 '2' (8 pixels wide)
	//         
	// ########
	// ########
	// ##      
	// ###     
	//  ###    
	//   ####  
	//     ### 
	//      ###
	//       ##
	// ##    ##
	// ##    ##
	//  ##  ###
	//  ###### 
	//   ####  
	//         
	0x78, 0x7C, 0x6E, 0x66, 0x63, 0x63, 0x61, 0x60, 
	0x30, 0x3C, 0x0E, 0x06, 0x06, 0x8E, 0xFC, 0xF8, 

	// @176 '3' (8 pixels wide)
	//         
	//   ####  
	//  ###### 
	// ###  ###
	// ##    ##
	// ##    ##
	//       ##
	//      ###
	//    #### 
	//    #### 
	// ##    ##
	// ##    ##
	// ###   ##
	//  #######
	//   ####  
	//         
	0x1C, 0x3C, 0x70, 0x60, 0x60, 0x71, 0x3F, 0x1F, 
	0x38, 0x3C, 0x0E, 0xC6, 0xC6, 0xC6, 0xFC, 0x3C, 

	// @192 '4' (9 pixels wide)
	//          
	//      ##  
	//      ##  
	//      ##  
	//      ##  
	// #########
	// #########
	//  #   ##  
	//  ##  ##  
	//   #  ##  
	//   #  ##  
	//    # ##  
	//     ###  
	//     ###  
	//      ##  
	//          
	0x06, 0x07, 0x06, 0x06, 0x06, 0x7F, 0x7F, 0x06, 0x06, 
	0x00, 0x80, 0xE0, 0x10, 0x0C, 0xFE, 0xFE, 0x00, 0x00, 

	// @210 '5' (8 pixels wide)
	//         
	//   ####  
	//  ###### 
	// ###  ###
	// ##    ##
	//       ##
	//       ##
	// ##   ###
	// ####### 
	// ## ###  
	//  #      
	//  #      
	//  #      
	//  ###### 
	//  ###### 
	//         
	0x19, 0x39, 0x70, 0x60, 0x60, 0x71, 0x3F, 0x1F, 
	0xC0, 0xFE, 0x86, 0xC6, 0xC6, 0xC6, 0x86, 0x00, 

	// @226 '6' (9 pixels wide)
	//          
	//    ####  
	//  ####### 
	//  ##   ## 
	// ##     ##
	// ##     ##
	// ##     ##
	// ##     ##
	// ###   ###
	// ######## 
	// ## ####  
	//  ##    ##
	//  ##   ###
	//   ###### 
	//    ####  
	//          
	0x0F, 0x3F, 0x30, 0x60, 0x60, 0x60, 0x70, 0x3F, 0x0F, 
	0xE0, 0xF8, 0xDC, 0x66, 0x66, 0x66, 0xEE, 0xDC, 0x98, 

	// @244 '7' (8 pixels wide)
	//         
	//   ##    
	//   ##    
	//   ##    
	//    ##   
	//    ##   
	//    ##   
	//    ##   
	//     ##  
	//     ##  
	//      #  
	//      ## 
	//      ## 
	// ########
	// ########
	//         
	0x00, 0x00, 0x70, 0x7F, 0x0F, 0x00, 0x00, 0x00, 
	0x06, 0x06, 0x06, 0x06, 0xC6, 0xFE, 0x1E, 0x06, 

	// @260 '8' (9 pixels wide)
	//          
	//   #####  
	//  ####### 
	// ###   ###
	// ##     ##
	// ##     ##
	// ##     ##
	//  ##   ###
	//   ###### 
	//   ###### 
	//  ##    ##
	//  ##    ##
	//  ##    ##
	//   ###### 
	//    ####  
	//          
	0x1E, 0x3F, 0x71, 0x60, 0x60, 0x60, 0x71, 0x3F, 0x1F, 
	0x00, 0x38, 0xFC, 0xC6, 0xC6, 0xC6, 0xC6, 0xFC, 0x38, 

	// @278 '9' (9 pixels wide)
	//          
	//   ####   
	//  ######  
	// ###   ## 
	// ##    ## 
	//   #### ##
	//  ########
	// ###   ###
	// ##     ##
	// ##     ##
	// ##     ##
	// ##     ##
	//  ##   ## 
	//  ####### 
	//   #####  
	//          
	0x19, 0x3B, 0x77, 0x66, 0x66, 0x66, 0x3B, 0x1F, 0x07, 
	0xF0, 0xFC, 0x0E, 0x06, 0x06, 0x06, 0x0E, 0xFC, 0xF0, 

	// @296 ':' (2 pixels wide)
	//   
	// ##
	// ##
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	// ##
	// ##
	//   
	//   
	//   
	//   
	0x60, 0x60, 
	0x30, 0x30, 

	// @300 '<' (8 pixels wide)
	//         
	//        #
	//      ###
	//    #### 
	//  ####   
	// ###     
	// ###     
	//  ###    
	//   ####  
	//     ### 
	//       ##
	//        #
	//         
	//         
	//         
	//         
	0x06, 0x0F, 0x0F, 0x19, 0x18, 0x30, 0x30, 0x60, 
	0x00, 0x00, 0x80, 0x80, 0xC0, 0xC0, 0x60, 0x30, 

	// @316 '=' (9 pixels wide)
	//          
	//          
	//          
	// #########
	// #########
	//          
	//          
	//          
	// #########
	// #########
	//          
	//          
	//          
	//          
	//          
	//          
	0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 
	0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 

	// @334 '>' (8 pixels wide)
	//         
	// #       
	// ###     
	//  ####   
	//    #### 
	//      ###
	//      ###
	//     ### 
	//   ####  
	//  ###    
	// ##      
	// #       
	//         
	//         
	//         
	//         
	0x60, 0x30, 0x30, 0x18, 0x19, 0x0F, 0x0F, 0x06, 
	0x30, 0x60, 0xC0, 0xC0, 0x80, 0x80, 0x00, 0x00, 

	// @350 '?' (8 pixels wide)
	//         
	//    ##   
	//    ##   
	//         
	//         
	//    ##   
	//    ##   
	//    ###  
	//     ##  
	//      ## 
	//       ##
	// ##    ##
	// ##    ##
	//  ##  ###
	//  ###### 
	//   ####  
	0x00, 0x00, 0x00, 0x67, 0x67, 0x01, 0x00, 0x00, 
	0x18, 0x1E, 0x07, 0x03, 0x83, 0xC7, 0x7E, 0x3C, 

	// @366 'A' (11 pixels wide)
	//            
	// ##       ##
	// ###     ###
	//  ##     ## 
	//  ##     ## 
	//  ######### 
	//   ######## 
	//   ##   ##  
	//   ###  ##  
	//    ##  ##  
	//    ## ##   
	//    #####   
	//     ####   
	//     ####   
	//     ###    
	//      ##    
	0x60, 0x7C, 0x3F, 0x07, 0x06, 0x06, 0x06, 0x07, 0x3F, 0x7E, 0x60, 
	0x00, 0x00, 0x80, 0xF0, 0xFE, 0x1F, 0x3F, 0xFC, 0xC0, 0x00, 0x00, 

	// @388 'B' (10 pixels wide)
	//           
	// ########  
	// ######### 
	// ##     ###
	// ##      ##
	// ##      ##
	// ##      ##
	// ##     ###
	// ######### 
	// #######   
	// ##    ##  
	// ##     ## 
	// ##     ## 
	// ##    ### 
	// ########  
	// #######   
	0x7F, 0x7F, 0x60, 0x60, 0x60, 0x60, 0x60, 0x71, 0x3F, 0x1F, 
	0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xE7, 0xBE, 0x9C, 0x00, 

	// @408 'C' (11 pixels wide)
	//            
	//    #####   
	//   #######  
	//  ###   ### 
	//  ##     ## 
	// ##       ##
	// ##       ##
	// ##         
	// ##         
	// ##         
	// ##         
	// ##       ##
	//  ##     ###
	//  ###    ## 
	//   #######  
	//    #####   
	0x07, 0x1F, 0x38, 0x70, 0x60, 0x60, 0x60, 0x70, 0x38, 0x1E, 0x06, 
	0xF0, 0xFC, 0x0E, 0x07, 0x03, 0x03, 0x03, 0x03, 0x0E, 0x1C, 0x18, 

	// @430 'D' (11 pixels wide)
	//            
	// ########   
	// #########  
	// ##     ### 
	// ##      ## 
	// ##       ##
	// ##       ##
	// ##       ##
	// ##       ##
	// ##       ##
	// ##       ##
	// ##       ##
	// ##      ## 
	// ##     ### 
	// #########  
	// ########   
	0x7F, 0x7F, 0x60, 0x60, 0x60, 0x60, 0x60, 0x70, 0x38, 0x1F, 0x07, 
	0xFF, 0xFF, 0x03, 0x03, 0x03, 0x03, 0x03, 0x07, 0x0E, 0xFC, 0xF0, 

	// @452 'E' (9 pixels wide)
	//          
	// #########
	// #########
	// ##       
	// ##       
	// ##       
	// ##       
	// ##       
	// ######## 
	// ######## 
	// ##       
	// ##       
	// ##       
	// ##       
	// #########
	// #########
	0x7F, 0x7F, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 
	0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x03, 

	// @470 'F' (9 pixels wide)
	//          
	// ##       
	// ##       
	// ##       
	// ##       
	// ##       
	// ##       
	// ##       
	// ######## 
	// ######## 
	// ##       
	// ##       
	// ##       
	// ##       
	// #########
	// #########
	0x7F, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0xFF, 0xFF, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0xC3, 0x03, 

	// @488 'G' (12 pixels wide)
	//             
	//     ####   #
	//   ##########
	//  ###    ####
	//  ##      ###
	// ##        ##
	// ##        ##
	// ##    ######
	// ##    ######
	// ##          
	// ##          
	// ###       ##
	//  ##      ###
	//  ###    ### 
	//   ########  
	//     #####   
	0x07, 0x1F, 0x38, 0x30, 0x60, 0x60, 0x61, 0x61, 0x31, 0x39, 0x3F, 0x7F, 
	0xF0, 0xFC, 0x1E, 0x06, 0x03, 0x03, 0x83, 0x83, 0x87, 0x8E, 0x9C, 0x98, 

	// @512 'H' (10 pixels wide)
	//           
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##########
	// ##########
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	0x7F, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 
	0xFF, 0xFF, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xFF, 0xFF, 

	// @532 'I' (2 pixels wide)
	//   
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	// ##
	0x7F, 0x7F, 
	0xFF, 0xFF, 

	// @536 'J' (8 pixels wide)
	//         
	//   ####  
	//  ###### 
	// ##   ###
	// ##    ##
	// ##    ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	//       ##
	0x1C, 0x3C, 0x60, 0x60, 0x60, 0x70, 0x3F, 0x1F, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 

	// @552 'K' (10 pixels wide)
	//           
	// ##     ###
	// ##     ## 
	// ##    ### 
	// ##   ###  
	// ##   ##   
	// ##  ###   
	// ### ##    
	// ######    
	// #####     
	// ## ##     
	// ##  ##    
	// ##   ##   
	// ##   ###  
	// ##    ### 
	// ##     ###
	0x7F, 0x7F, 0x01, 0x00, 0x03, 0x0F, 0x1E, 0x78, 0x70, 0x40, 
	0xFF, 0xFF, 0xC0, 0xE0, 0xF0, 0x9C, 0x0E, 0x07, 0x03, 0x01, 

	// @572 'L' (8 pixels wide)
	//         
	// ########
	// ########
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	// ##      
	0x7F, 0x7F, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 

	// @588 'M' (12 pixels wide)
	//             
	// ##   ##   ##
	// ##  ####  ##
	// ##  ####  ##
	// ##  ####  ##
	// ## ###### ##
	// ## ##  ## ##
	// ## ##  ## ##
	// #####  #####
	// ####    ####
	// ####    ####
	// ####    ####
	// ###      ###
	// ###      ###
	// ###      ###
	// ##        ##
	0x7F, 0x7F, 0x00, 0x07, 0x3F, 0x7C, 0x7C, 0x3F, 0x07, 0x00, 0x7F, 0x7F, 
	0xFF, 0xFF, 0xFE, 0xF0, 0x80, 0x00, 0x00, 0x80, 0xF0, 0xFE, 0xFF, 0xFF, 

	// @612 'N' (10 pixels wide)
	//           
	// ##      ##
	// ##     ###
	// ##     ###
	// ##    ####
	// ##   #####
	// ##   ## ##
	// ##  ### ##
	// ##  ##  ##
	// ## ###  ##
	// ## ##   ##
	// ####    ##
	// ####    ##
	// ###     ##
	// ###     ##
	// ##      ##
	0x7F, 0x7F, 0x00, 0x00, 0x01, 0x07, 0x0F, 0x3C, 0x7F, 0x7F, 
	0xFF, 0xFF, 0x1E, 0x78, 0xE0, 0xC0, 0x00, 0x00, 0xFF, 0xFF, 

	// @632 'O' (12 pixels wide)
	//             
	//     ####    
	//   ########  
	//  ###    ### 
	//  ##      ## 
	// ###      ###
	// ##        ##
	// ##        ##
	// ##        ##
	// ##        ##
	// ##        ##
	// ###      ###
	//  ##      ## 
	//  ###    ### 
	//   ########  
	//     ####    
	0x07, 0x1F, 0x3C, 0x30, 0x60, 0x60, 0x60, 0x60, 0x30, 0x3C, 0x1F, 0x07, 
	0xF0, 0xFC, 0x1E, 0x06, 0x03, 0x03, 0x03, 0x03, 0x06, 0x1E, 0xFC, 0xF0, 

	// @656 'P' (10 pixels wide)
	//           
	// ##        
	// ##        
	// ##        
	// ##        
	// ##        
	// ##        
	// ########  
	// ######### 
	// ##     ###
	// ##      ##
	// ##      ##
	// ##      ##
	// ##     ###
	// ######### 
	// ########  
	0x7F, 0x7F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 
	0xFF, 0xFF, 0x83, 0x83, 0x83, 0x83, 0x83, 0xC7, 0xFE, 0x7C, 

	// @676 'Q' (12 pixels wide)
	//           # 
	//     #### ###
	//   ######### 
	//  ###    ### 
	//  ##    #### 
	// ##      ####
	// ##        ##
	// ##        ##
	// ##        ##
	// ##        ##
	// ##        ##
	// ###      ###
	//  ##      ## 
	//  ###    ### 
	//   ########  
	//     ####    
	0x07, 0x1F, 0x38, 0x30, 0x60, 0x60, 0x60, 0x68, 0x3C, 0x7C, 0xFF, 0x47, 
	0xF0, 0xFC, 0x1E, 0x06, 0x03, 0x03, 0x03, 0x03, 0x06, 0x1E, 0xFC, 0xF0, 

	// @700 'R' (11 pixels wide)
	//            
	// ##      ###
	// ##      ## 
	// ##      ## 
	// ##      ## 
	// ##      ## 
	// ##      ## 
	// #########  
	// #########  
	// ##     ##  
	// ##      ## 
	// ##      ## 
	// ##      ## 
	// ##     ### 
	// #########  
	// ########   
	0x7F, 0x7F, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x7F, 0x7E, 0x40, 
	0xFF, 0xFF, 0x83, 0x83, 0x83, 0x83, 0x83, 0xC7, 0xFE, 0x3C, 0x00, 

	// @722 'S' (9 pixels wide)
	//          
	//   #####  
	//  ####### 
	// ###   ###
	// ##     ##
	// ##     ##
	//        ##
	//      ### 
	//   #####  
	//  ###     
	// ##       
	// ##     ##
	// ##     ##
	// ###   ###
	//  ####### 
	//   #####  
	0x1C, 0x3C, 0x70, 0x60, 0x60, 0x61, 0x71, 0x3F, 0x1E, 
	0x3C, 0x7E, 0xC7, 0xC3, 0x83, 0x83, 0x87, 0x1E, 0x1C, 

	// @740 'T' (10 pixels wide)
	//           
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	//     ##    
	// ##########
	// ##########
	0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x00, 0x00, 0x00, 0x00, 
	0x03, 0x03, 0x03, 0x03, 0xFF, 0xFF, 0x03, 0x03, 0x03, 0x03, 

	// @760 'U' (10 pixels wide)
	//           
	//   #####   
	//  ######## 
	//  ##    ## 
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	// ##      ##
	0x0F, 0x3F, 0x70, 0x60, 0x60, 0x60, 0x60, 0x30, 0x3F, 0x0F, 
	0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 

	// @780 'V' (12 pixels wide)
	//             
	//      ##     
	//     ####    
	//     ####    
	//     ####    
	//    ######   
	//    ##  ##   
	//    ##  ##   
	//   ###  ###  
	//   ##    ##  
	//   ##    ##  
	//  ###    ### 
	//  ##      ## 
	//  ##      ## 
	// ###      ###
	// ###      ###
	0x00, 0x00, 0x00, 0x07, 0x3F, 0x7C, 0x7C, 0x3F, 0x07, 0x00, 0x00, 0x00, 
	0x03, 0x1F, 0xFF, 0xF0, 0x80, 0x00, 0x00, 0x80, 0xF0, 0xFF, 0x1F, 0x03, 

	// @804 'W' (16 pixels wide)
	//                 
	//    ##      ##   
	//    ##      ##   
	//   ####    ####  
	//   ####    ####  
	//   ####    ####  
	//   #####  #####  
	//   ## ##  ## ##  
	//  ### ##  ## ### 
	//  ##  ##  ##  ## 
	//  ##  #####   ## 
	//  ##   ####   ## 
	//  ##   ####   ## 
	// ###   ####   ###
	// ###    ##    ###
	// ##     ##     ##
	0x00, 0x00, 0x1F, 0x7F, 0x7E, 0x1F, 0x03, 0x00, 0x00, 0x03, 0x1F, 0x7E, 0x7F, 0x1F, 0x00, 0x00, 
	0x07, 0xFF, 0xFE, 0x80, 0x00, 0xE0, 0xFC, 0x3F, 0x3F, 0xFC, 0xC0, 0x00, 0x80, 0xFE, 0xFF, 0x07, 

	// @836 'X' (11 pixels wide)
	//            
	// ###      ##
	//  ##     ## 
	//  ###   ### 
	//   ##   ##  
	//    ## ##   
	//    #####   
	//     ###    
	//     ###    
	//     ###    
	//    #####   
	//    ## ###  
	//   ###  ##  
	//  ###   ### 
	//  ##     ## 
	// ###     ###
	0x40, 0x70, 0x78, 0x1E, 0x07, 0x03, 0x07, 0x1E, 0x38, 0x70, 0x40, 
	0x01, 0x07, 0x0F, 0x3C, 0xF8, 0xE0, 0xF0, 0x3C, 0x1F, 0x07, 0x01, 

	// @858 'Y' (12 pixels wide)
	//             
	//      ##     
	//      ##     
	//      ##     
	//      ##     
	//      ##     
	//      ##     
	//      ##     
	//     ####    
	//     ####    
	//    ##  ##   
	//    ##  ##   
	//   ##    ##  
	//  ###    ### 
	//  ##      ## 
	// ###      ###
	0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x01, 0x07, 0x0F, 0x3C, 0xF0, 0xC0, 0xC0, 0xF0, 0x3C, 0x0F, 0x07, 0x01, 

	// @882 'Z' (10 pixels wide)
	//           
	// ##########
	// ##########
	//  ##       
	//  ###      
	//   ##      
	//   ###     
	//    ###    
	//     ##    
	//     ###   
	//      ###  
	//       ##  
	//       ### 
	//        ###
	//  #########
	//  #########
	0x60, 0x78, 0x7E, 0x6F, 0x63, 0x61, 0x60, 0x60, 0x60, 0x60, 
	0x00, 0x03, 0x03, 0x03, 0xC3, 0xE3, 0x7B, 0x3F, 0x0F, 0x07, 

	// @902 '\' (5 pixels wide)
	//      
	//    ##
	//    ##
	//    # 
	//    # 
	//   ## 
	//   ## 
	//   ## 
	//   #  
	//  ##  
	//  ##  
	//  ##  
	//  #   
	//  #   
	// ##   
	// ##   
	0x00, 0x00, 0x07, 0x7F, 0x60, 
	0x03, 0x7F, 0xF0, 0x00, 0x00, 

	// @912 '^' (10 pixels wide)
	//           
	//           
	//           
	//           
	//           
	//           
	// ##      ##
	//  ##     # 
	//  ##    ## 
	//   #    #  
	//   ##  ##  
	//   ##  ##  
	//    ####   
	//    ####   
	//     ##    
	//           
	0x02, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x02, 
	0x00, 0x80, 0xF0, 0x3C, 0x0E, 0x0E, 0x3C, 0xF0, 0x80, 0x00, 

	// @932 '<>' (16 pixels wide)
	//         
	//        ##
	//      ######
	//    #### ####
	//  ####     ####
	// ###         ###
	// ###         ###
	//  ###       ###
	//   ####   ####
	//     ### ###
	//       ####
	//        ##
	//         
	//         
	//         
	//         
	0x06, 0x0F, 0x0F, 0x19, 0x18, 0x30, 0x30, 0x60, 0x60, 0x30, 0x30, 0x18, 0x19, 0x0F, 0x0F, 0x06, 
	0x00, 0x00, 0x80, 0x80, 0xC0, 0xC0, 0x60, 0x30, 0x30, 0x60, 0xC0, 0xC0, 0x80, 0x80, 0x00, 0x00, 


	// @964 ':' (3 pixels wide)
	//   
	// ###
	// ###
	//   
	//   
	//   
	//   
	//   
	//   
	//   
	// ###
	// ###
	//   
	//   
	//   
	//   
	0x70, 0x70, 0x70, 
	0x38, 0x38, 0x38,

	// @964 block (9 pixels wide)
	//          
	//  ####  ##
	// ###### ##
	// ##   ### 
	// ##    ## 
	// ##    ## 
	//   ###### 
	//       ## 
	// ##    ## 
	// ##    ## 
	//  ####### 
	//   #####  
	//          
	//          
	//          
	//          
//	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f,
//	0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};

// Character descriptors for Helvetica Narrow 16pt
// { [Char width in bits], [Offset into helveticaNarrow_16ptCharBitmaps in bytes] }
const FONT_CHAR_INFO fontCharInfo[] = 
{
	{2, 0}, 		// ! 
	{4, 4}, 		// " 
	{8, 12}, 		// # 
	{0, 0}, 		// $ 
	{14, 28}, 		// % 
	{11, 56}, 		// & 
	{1, 78}, 		// ' 
	{0, 0}, 		// ( 
	{0, 0}, 		// ) 
	{5, 80}, 		// * 
	{10, 90}, 		// + 
	{0, 0}, 		// , 
	{4, 110}, 		// - 
	{2, 118}, 		// . 
	{5, 122}, 		// / 
	{9, 132}, 		// 0 
	{5, 150}, 		// 1 
	{8, 160}, 		// 2 
	{8, 176}, 		// 3 
	{9, 192}, 		// 4 
	{8, 210}, 		// 5 
	{9, 226}, 		// 6 
	{8, 244}, 		// 7 
	{9, 260}, 		// 8 
	{9, 278}, 		// 9 
	{2, 296}, 		// : 
	{0, 0}, 		// ; 
	{8, 300}, 		// < 
	{9, 316}, 		// = 
	{8, 334}, 		// > 
	{8, 350}, 		// ? 
	{0, 0}, 		// @ 
	{11, 366}, 		// A 
	{10, 388}, 		// B 
	{11, 408}, 		// C 
	{11, 430}, 		// D 
	{9, 452}, 		// E 
	{9, 470}, 		// F 
	{12, 488}, 		// G 
	{10, 512}, 		// H 
	{2, 532}, 		// I 
	{8, 536}, 		// J 
	{10, 552}, 		// K 
	{8, 572}, 		// L 
	{12, 588}, 		// M 
	{10, 612}, 		// N 
	{12, 632}, 		// O 
	{10, 656}, 		// P 
	{12, 676}, 		// Q 
	{11, 700}, 		// R 
	{9, 722}, 		// S 
	{10, 740}, 		// T 
	{10, 760}, 		// U 
	{12, 780}, 		// V 
	{16, 804}, 		// W 
	{11, 836}, 		// X 
	{12, 858}, 		// Y 
	{10, 882}, 		// Z 
	{0, 0}, 		// [ 
	{5, 902}, 		// \ 
	{0, 0}, 		// ] 
	{10, 912}, 		// ^ 
	{0, 0}, 		// _ 
	{16, 932}, 		// <> 
	{3, 964}, 		// big : 
};

unsigned char Ascii_1[241][5] = {		// Refer to "Times New Roman" Font Database...
	{0x00,0x00,0x00,0x00,0x00},					//   Basic Characters
	{0x00,0x00,0x4F,0x00,0x00},		//   (  1)  ! - 0x0021 Exclamation Mark
	{0x00,0x07,0x00,0x07,0x00},		//   (  2)  " - 0x0022 Quotation Mark
	{0x14,0x7F,0x14,0x7F,0x14},		//   (  3)  # - 0x0023 Number Sign
	{0x24,0x2A,0x7F,0x2A,0x12},		//   (  4)  $ - 0x0024 Dollar Sign
	{0x23,0x13,0x08,0x64,0x62},		//   (  5)  % - 0x0025 Percent Sign
	{0x36,0x49,0x55,0x22,0x50},		//   (  6)  & - 0x0026 Ampersand
	{0x00,0x05,0x03,0x00,0x00},		//   (  7)  ' - 0x0027 Apostrophe
	{0x00,0x1C,0x22,0x41,0x00},		//   (  8)  ( - 0x0028 Left Parenthesis
	{0x00,0x41,0x22,0x1C,0x00},		//   (  9)  ) - 0x0029 Right Parenthesis
	{0x14,0x08,0x3E,0x08,0x14},		//   ( 10)  * - 0x002A Asterisk
	{0x08,0x08,0x3E,0x08,0x08},		//   ( 11)  + - 0x002B Plus Sign
	{0x00,0x50,0x30,0x00,0x00},		//   ( 12)  , - 0x002C Comma
	{0x08,0x08,0x08,0x08,0x08},		//   ( 13)  - - 0x002D Hyphen-Minus
	{0x00,0x60,0x60,0x00,0x00},		//   ( 14)  . - 0x002E Full Stop
	{0x20,0x10,0x08,0x04,0x02},		//   ( 15)  / - 0x002F Solidus
	{0x3E,0x51,0x49,0x45,0x3E},		//   ( 16)  0 - 0x0030 Digit Zero
	{0x00,0x42,0x7F,0x40,0x00},		//   ( 17)  1 - 0x0031 Digit One
	{0x42,0x61,0x51,0x49,0x46},		//   ( 18)  2 - 0x0032 Digit Two
	{0x21,0x41,0x45,0x4B,0x31},		//   ( 19)  3 - 0x0033 Digit Three
	{0x18,0x14,0x12,0x7F,0x10},		//   ( 20)  4 - 0x0034 Digit Four
	{0x27,0x45,0x45,0x45,0x39},		//   ( 21)  5 - 0x0035 Digit Five
	{0x3C,0x4A,0x49,0x49,0x30},		//   ( 22)  6 - 0x0036 Digit Six
	{0x01,0x71,0x09,0x05,0x03},		//   ( 23)  7 - 0x0037 Digit Seven
	{0x36,0x49,0x49,0x49,0x36},		//   ( 24)  8 - 0x0038 Digit Eight
	{0x06,0x49,0x49,0x29,0x1E},		//   ( 25)  9 - 0x0039 Dight Nine
	{0x00,0x36,0x36,0x00,0x00},		//   ( 26)  : - 0x003A Colon
	{0x00,0x56,0x36,0x00,0x00},		//   ( 27)  ; - 0x003B Semicolon
	{0x08,0x14,0x22,0x41,0x00},		//   ( 28)  < - 0x003C Less-Than Sign
	{0x14,0x14,0x14,0x14,0x14},		//   ( 29)  = - 0x003D Equals Sign
	{0x00,0x41,0x22,0x14,0x08},		//   ( 30)  > - 0x003E Greater-Than Sign
	{0x02,0x01,0x51,0x09,0x06},		//   ( 31)  ? - 0x003F Question Mark
	{0x32,0x49,0x79,0x41,0x3E},		//   ( 32)  @ - 0x0040 Commercial At
	{0x7E,0x11,0x11,0x11,0x7E},		//   ( 33)  A - 0x0041 Latin Capital Letter A
	{0x7F,0x49,0x49,0x49,0x36},		//   ( 34)  B - 0x0042 Latin Capital Letter B
	{0x3E,0x41,0x41,0x41,0x22},		//   ( 35)  C - 0x0043 Latin Capital Letter C
	{0x7F,0x41,0x41,0x22,0x1C},		//   ( 36)  D - 0x0044 Latin Capital Letter D
	{0x7F,0x49,0x49,0x49,0x41},		//   ( 37)  E - 0x0045 Latin Capital Letter E
	{0x7F,0x09,0x09,0x09,0x01},		//   ( 38)  F - 0x0046 Latin Capital Letter F
	{0x3E,0x41,0x49,0x49,0x7A},		//   ( 39)  G - 0x0047 Latin Capital Letter G
	{0x7F,0x08,0x08,0x08,0x7F},		//   ( 40)  H - 0x0048 Latin Capital Letter H
	{0x00,0x41,0x7F,0x41,0x00},		//   ( 41)  I - 0x0049 Latin Capital Letter I
	{0x20,0x40,0x41,0x3F,0x01},		//   ( 42)  J - 0x004A Latin Capital Letter J
	{0x7F,0x08,0x14,0x22,0x41},		//   ( 43)  K - 0x004B Latin Capital Letter K
	{0x7F,0x40,0x40,0x40,0x40},		//   ( 44)  L - 0x004C Latin Capital Letter L
	{0x7F,0x02,0x0C,0x02,0x7F},		//   ( 45)  M - 0x004D Latin Capital Letter M
	{0x7F,0x04,0x08,0x10,0x7F},		//   ( 46)  N - 0x004E Latin Capital Letter N
	{0x3E,0x41,0x41,0x41,0x3E},		//   ( 47)  O - 0x004F Latin Capital Letter O
	{0x7F,0x09,0x09,0x09,0x06},		//   ( 48)  P - 0x0050 Latin Capital Letter P
	{0x3E,0x41,0x51,0x21,0x5E},		//   ( 49)  Q - 0x0051 Latin Capital Letter Q
	{0x7F,0x09,0x19,0x29,0x46},		//   ( 50)  R - 0x0052 Latin Capital Letter R
	{0x46,0x49,0x49,0x49,0x31},		//   ( 51)  S - 0x0053 Latin Capital Letter S
	{0x01,0x01,0x7F,0x01,0x01},		//   ( 52)  T - 0x0054 Latin Capital Letter T
	{0x3F,0x40,0x40,0x40,0x3F},		//   ( 53)  U - 0x0055 Latin Capital Letter U
	{0x1F,0x20,0x40,0x20,0x1F},		//   ( 54)  V - 0x0056 Latin Capital Letter V
	{0x3F,0x40,0x38,0x40,0x3F},		//   ( 55)  W - 0x0057 Latin Capital Letter W
	{0x63,0x14,0x08,0x14,0x63},		//   ( 56)  X - 0x0058 Latin Capital Letter X
	{0x07,0x08,0x70,0x08,0x07},		//   ( 57)  Y - 0x0059 Latin Capital Letter Y
	{0x61,0x51,0x49,0x45,0x43},		//   ( 58)  Z - 0x005A Latin Capital Letter Z
	{0x00,0x7F,0x41,0x41,0x00},		//   ( 59)  [ - 0x005B Left Square Bracket
	{0x02,0x04,0x08,0x10,0x20},		//   ( 60)  \ - 0x005C Reverse Solidus
	{0x00,0x41,0x41,0x7F,0x00},		//   ( 61)  ] - 0x005D Right Square Bracket
	{0x04,0x02,0x01,0x02,0x04},		//   ( 62)  ^ - 0x005E Circumflex Accent
	{0x40,0x40,0x40,0x40,0x40},		//   ( 63)  _ - 0x005F Low Line
	{0x01,0x02,0x04,0x00,0x00},		//   ( 64)  ` - 0x0060 Grave Accent
	{0x20,0x54,0x54,0x54,0x78},		//   ( 65)  a - 0x0061 Latin Small Letter A
	{0x7F,0x48,0x44,0x44,0x38},		//   ( 66)  b - 0x0062 Latin Small Letter B
	{0x38,0x44,0x44,0x44,0x20},		//   ( 67)  c - 0x0063 Latin Small Letter C
	{0x38,0x44,0x44,0x48,0x7F},		//   ( 68)  d - 0x0064 Latin Small Letter D
	{0x38,0x54,0x54,0x54,0x18},		//   ( 69)  e - 0x0065 Latin Small Letter E
	{0x08,0x7E,0x09,0x01,0x02},		//   ( 70)  f - 0x0066 Latin Small Letter F
	{0x06,0x49,0x49,0x49,0x3F},		//   ( 71)  g - 0x0067 Latin Small Letter G
	{0x7F,0x08,0x04,0x04,0x78},		//   ( 72)  h - 0x0068 Latin Small Letter H
	{0x00,0x44,0x7D,0x40,0x00},		//   ( 73)  i - 0x0069 Latin Small Letter I
	{0x20,0x40,0x44,0x3D,0x00},		//   ( 74)  j - 0x006A Latin Small Letter J
	{0x7F,0x10,0x28,0x44,0x00},		//   ( 75)  k - 0x006B Latin Small Letter K
	{0x00,0x41,0x7F,0x40,0x00},		//   ( 76)  l - 0x006C Latin Small Letter L
	{0x7C,0x04,0x18,0x04,0x7C},		//   ( 77)  m - 0x006D Latin Small Letter M
	{0x7C,0x08,0x04,0x04,0x78},		//   ( 78)  n - 0x006E Latin Small Letter N
	{0x38,0x44,0x44,0x44,0x38},		//   ( 79)  o - 0x006F Latin Small Letter O
	{0x7C,0x14,0x14,0x14,0x08},		//   ( 80)  p - 0x0070 Latin Small Letter P
	{0x08,0x14,0x14,0x18,0x7C},		//   ( 81)  q - 0x0071 Latin Small Letter Q
	{0x7C,0x08,0x04,0x04,0x08},		//   ( 82)  r - 0x0072 Latin Small Letter R
	{0x48,0x54,0x54,0x54,0x20},		//   ( 83)  s - 0x0073 Latin Small Letter S
	{0x04,0x3F,0x44,0x40,0x20},		//   ( 84)  t - 0x0074 Latin Small Letter T
	{0x3C,0x40,0x40,0x20,0x7C},		//   ( 85)  u - 0x0075 Latin Small Letter U
	{0x1C,0x20,0x40,0x20,0x1C},		//   ( 86)  v - 0x0076 Latin Small Letter V
	{0x3C,0x40,0x30,0x40,0x3C},		//   ( 87)  w - 0x0077 Latin Small Letter W
	{0x44,0x28,0x10,0x28,0x44},		//   ( 88)  x - 0x0078 Latin Small Letter X
	{0x0C,0x50,0x50,0x50,0x3C},		//   ( 89)  y - 0x0079 Latin Small Letter Y
	{0x44,0x64,0x54,0x4C,0x44},		//   ( 90)  z - 0x007A Latin Small Letter Z
	{0x00,0x08,0x36,0x41,0x00},		//   ( 91)  { - 0x007B Left Curly Bracket
	{0x00,0x00,0x7F,0x00,0x00},		//   ( 92)  | - 0x007C Vertical Line
	{0x00,0x41,0x36,0x08,0x00},		//   ( 93)  } - 0x007D Right Curly Bracket
	{0x02,0x01,0x02,0x04,0x02},		//   ( 94)  ~ - 0x007E Tilde
	{0x3E,0x55,0x55,0x41,0x22},		//   ( 95)  C - 0x0080 <Control>
	{0x00,0x00,0x00,0x00,0x00},		//   ( 96)    - 0x00A0 No-Break Space
	{0x00,0x00,0x79,0x00,0x00},		//   ( 97)  ! - 0x00A1 Inverted Exclamation Mark
	{0x18,0x24,0x74,0x2E,0x24},		//   ( 98)  c - 0x00A2 Cent Sign
	{0x48,0x7E,0x49,0x42,0x40},		//   ( 99)  L - 0x00A3 Pound Sign
	{0x5D,0x22,0x22,0x22,0x5D},		//   (100)  o - 0x00A4 Currency Sign
	{0x15,0x16,0x7C,0x16,0x15},		//   (101)  Y - 0x00A5 Yen Sign
	{0x00,0x00,0x77,0x00,0x00},		//   (102)  | - 0x00A6 Broken Bar
	{0x0A,0x55,0x55,0x55,0x28},		//   (103)    - 0x00A7 Section Sign
	{0x00,0x01,0x00,0x01,0x00},		//   (104)  " - 0x00A8 Diaeresis
	{0x00,0x0A,0x0D,0x0A,0x04},		//   (105)    - 0x00AA Feminine Ordinal Indicator
	{0x08,0x14,0x2A,0x14,0x22},		//   (106) << - 0x00AB Left-Pointing Double Angle Quotation Mark
	{0x04,0x04,0x04,0x04,0x1C},		//   (107)    - 0x00AC Not Sign
	{0x00,0x08,0x08,0x08,0x00},		//   (108)  - - 0x00AD Soft Hyphen
	{0x01,0x01,0x01,0x01,0x01},		//   (109)    - 0x00AF Macron
	{0x00,0x02,0x05,0x02,0x00},		//   (110)    - 0x00B0 Degree Sign
	{0x44,0x44,0x5F,0x44,0x44},		//   (111) +- - 0x00B1 Plus-Minus Sign
	{0x00,0x00,0x04,0x02,0x01},		//   (112)  ` - 0x00B4 Acute Accent
	{0x7E,0x20,0x20,0x10,0x3E},		//   (113)  u - 0x00B5 Micro Sign
	{0x06,0x0F,0x7F,0x00,0x7F},		//   (114)    - 0x00B6 Pilcrow Sign
	{0x00,0x18,0x18,0x00,0x00},		//   (115)  . - 0x00B7 Middle Dot
	{0x00,0x40,0x50,0x20,0x00},		//   (116)    - 0x00B8 Cedilla
	{0x00,0x0A,0x0D,0x0A,0x00},		//   (117)    - 0x00BA Masculine Ordinal Indicator
	{0x22,0x14,0x2A,0x14,0x08},		//   (118) >> - 0x00BB Right-Pointing Double Angle Quotation Mark
	{0x17,0x08,0x34,0x2A,0x7D},		//   (119) /4 - 0x00BC Vulgar Fraction One Quarter
	{0x17,0x08,0x04,0x6A,0x59},		//   (120) /2 - 0x00BD Vulgar Fraction One Half
	{0x30,0x48,0x45,0x40,0x20},		//   (121)  ? - 0x00BF Inverted Question Mark
	{0x70,0x29,0x26,0x28,0x70},		//   (122) `A - 0x00C0 Latin Capital Letter A with Grave
	{0x70,0x28,0x26,0x29,0x70},		//   (123) 'A - 0x00C1 Latin Capital Letter A with Acute
	{0x70,0x2A,0x25,0x2A,0x70},		//   (124) ^A - 0x00C2 Latin Capital Letter A with Circumflex
	{0x72,0x29,0x26,0x29,0x70},		//   (125) ~A - 0x00C3 Latin Capital Letter A with Tilde
	{0x70,0x29,0x24,0x29,0x70},		//   (126) "A - 0x00C4 Latin Capital Letter A with Diaeresis
	{0x70,0x2A,0x2D,0x2A,0x70},		//   (127)  A - 0x00C5 Latin Capital Letter A with Ring Above
	{0x7E,0x11,0x7F,0x49,0x49},		//   (128) AE - 0x00C6 Latin Capital Letter Ae
	{0x0E,0x51,0x51,0x71,0x11},		//   (129)  C - 0x00C7 Latin Capital Letter C with Cedilla
	{0x7C,0x55,0x56,0x54,0x44},		//   (130) `E - 0x00C8 Latin Capital Letter E with Grave
	{0x7C,0x55,0x56,0x54,0x44},		//   (131) 'E - 0x00C9 Latin Capital Letter E with Acute
	{0x7C,0x56,0x55,0x56,0x44},		//   (132) ^E - 0x00CA Latin Capital Letter E with Circumflex
	{0x7C,0x55,0x54,0x55,0x44},		//   (133) "E - 0x00CB Latin Capital Letter E with Diaeresis
	{0x00,0x45,0x7E,0x44,0x00},		//   (134) `I - 0x00CC Latin Capital Letter I with Grave
	{0x00,0x44,0x7E,0x45,0x00},		//   (135) 'I - 0x00CD Latin Capital Letter I with Acute
	{0x00,0x46,0x7D,0x46,0x00},		//   (136) ^I - 0x00CE Latin Capital Letter I with Circumflex
	{0x00,0x45,0x7C,0x45,0x00},		//   (137) "I - 0x00CF Latin Capital Letter I with Diaeresis
	{0x7F,0x49,0x49,0x41,0x3E},		//   (138)  D - 0x00D0 Latin Capital Letter Eth
	{0x7C,0x0A,0x11,0x22,0x7D},		//   (139) ~N - 0x00D1 Latin Capital Letter N with Tilde
	{0x38,0x45,0x46,0x44,0x38},		//   (140) `O - 0x00D2 Latin Capital Letter O with Grave
	{0x38,0x44,0x46,0x45,0x38},		//   (141) 'O - 0x00D3 Latin Capital Letter O with Acute
	{0x38,0x46,0x45,0x46,0x38},		//   (142) ^O - 0x00D4 Latin Capital Letter O with Circumflex
	{0x38,0x46,0x45,0x46,0x39},		//   (143) ~O - 0x00D5 Latin Capital Letter O with Tilde
	{0x38,0x45,0x44,0x45,0x38},		//   (144) "O - 0x00D6 Latin Capital Letter O with Diaeresis
	{0x22,0x14,0x08,0x14,0x22},		//   (145)  x - 0x00D7 Multiplcation Sign
	{0x2E,0x51,0x49,0x45,0x3A},		//   (146)  O - 0x00D8 Latin Capital Letter O with Stroke
	{0x3C,0x41,0x42,0x40,0x3C},		//   (147) `U - 0x00D9 Latin Capital Letter U with Grave
	{0x3C,0x40,0x42,0x41,0x3C},		//   (148) 'U - 0x00DA Latin Capital Letter U with Acute
	{0x3C,0x42,0x41,0x42,0x3C},		//   (149) ^U - 0x00DB Latin Capital Letter U with Circumflex
	{0x3C,0x41,0x40,0x41,0x3C},		//   (150) "U - 0x00DC Latin Capital Letter U with Diaeresis
	{0x0C,0x10,0x62,0x11,0x0C},		//   (151) `Y - 0x00DD Latin Capital Letter Y with Acute
	{0x7F,0x12,0x12,0x12,0x0C},		//   (152)  P - 0x00DE Latin Capital Letter Thom
	{0x40,0x3E,0x01,0x49,0x36},		//   (153)  B - 0x00DF Latin Capital Letter Sharp S
	{0x20,0x55,0x56,0x54,0x78},		//   (154) `a - 0x00E0 Latin Small Letter A with Grave
	{0x20,0x54,0x56,0x55,0x78},		//   (155) 'a - 0x00E1 Latin Small Letter A with Acute
	{0x20,0x56,0x55,0x56,0x78},		//   (156) ^a - 0x00E2 Latin Small Letter A with Circumflex
	{0x20,0x55,0x56,0x55,0x78},		//   (157) ~a - 0x00E3 Latin Small Letter A with Tilde
	{0x20,0x55,0x54,0x55,0x78},		//   (158) "a - 0x00E4 Latin Small Letter A with Diaeresis
	{0x20,0x56,0x57,0x56,0x78},		//   (159)  a - 0x00E5 Latin Small Letter A with Ring Above
	{0x24,0x54,0x78,0x54,0x58},		//   (160) ae - 0x00E6 Latin Small Letter Ae
	{0x0C,0x52,0x52,0x72,0x13},		//   (161)  c - 0x00E7 Latin Small Letter c with Cedilla
	{0x38,0x55,0x56,0x54,0x18},		//   (162) `e - 0x00E8 Latin Small Letter E with Grave
	{0x38,0x54,0x56,0x55,0x18},		//   (163) 'e - 0x00E9 Latin Small Letter E with Acute
	{0x38,0x56,0x55,0x56,0x18},		//   (164) ^e - 0x00EA Latin Small Letter E with Circumflex
	{0x38,0x55,0x54,0x55,0x18},		//   (165) "e - 0x00EB Latin Small Letter E with Diaeresis
	{0x00,0x49,0x7A,0x40,0x00},		//   (166) `i - 0x00EC Latin Small Letter I with Grave
	{0x00,0x48,0x7A,0x41,0x00},		//   (167) 'i - 0x00ED Latin Small Letter I with Acute
	{0x00,0x4A,0x79,0x42,0x00},		//   (168) ^i - 0x00EE Latin Small Letter I with Circumflex
	{0x00,0x4A,0x78,0x42,0x00},		//   (169) "i - 0x00EF Latin Small Letter I with Diaeresis
	{0x31,0x4A,0x4E,0x4A,0x30},		//   (170)    - 0x00F0 Latin Small Letter Eth
	{0x7A,0x11,0x0A,0x09,0x70},		//   (171) ~n - 0x00F1 Latin Small Letter N with Tilde
	{0x30,0x49,0x4A,0x48,0x30},		//   (172) `o - 0x00F2 Latin Small Letter O with Grave
	{0x30,0x48,0x4A,0x49,0x30},		//   (173) 'o - 0x00F3 Latin Small Letter O with Acute
	{0x30,0x4A,0x49,0x4A,0x30},		//   (174) ^o - 0x00F4 Latin Small Letter O with Circumflex
	{0x30,0x4A,0x49,0x4A,0x31},		//   (175) ~o - 0x00F5 Latin Small Letter O with Tilde
	{0x30,0x4A,0x48,0x4A,0x30},		//   (176) "o - 0x00F6 Latin Small Letter O with Diaeresis
	{0x08,0x08,0x2A,0x08,0x08},		//   (177)  + - 0x00F7 Division Sign
	{0x38,0x64,0x54,0x4C,0x38},		//   (178)  o - 0x00F8 Latin Small Letter O with Stroke
	{0x38,0x41,0x42,0x20,0x78},		//   (179) `u - 0x00F9 Latin Small Letter U with Grave
	{0x38,0x40,0x42,0x21,0x78},		//   (180) 'u - 0x00FA Latin Small Letter U with Acute
	{0x38,0x42,0x41,0x22,0x78},		//   (181) ^u - 0x00FB Latin Small Letter U with Circumflex
	{0x38,0x42,0x40,0x22,0x78},		//   (182) "u - 0x00FC Latin Small Letter U with Diaeresis
	{0x0C,0x50,0x52,0x51,0x3C},		//   (183) 'y - 0x00FD Latin Small Letter Y with Acute
	{0x7E,0x14,0x14,0x14,0x08},		//   (184)  p - 0x00FE Latin Small Letter Thom
	{0x0C,0x51,0x50,0x51,0x3C},		//   (185) "y - 0x00FF Latin Small Letter Y with Diaeresis
	{0x1E,0x09,0x09,0x29,0x5E},		//   (186)  A - 0x0104 Latin Capital Letter A with Ogonek
	{0x08,0x15,0x15,0x35,0x4E},		//   (187)  a - 0x0105 Latin Small Letter A with Ogonek
	{0x38,0x44,0x46,0x45,0x20},		//   (188) 'C - 0x0106 Latin Capital Letter C with Acute
	{0x30,0x48,0x4A,0x49,0x20},		//   (189) 'c - 0x0107 Latin Small Letter C with Acute
	{0x38,0x45,0x46,0x45,0x20},		//   (190)  C - 0x010C Latin Capital Letter C with Caron
	{0x30,0x49,0x4A,0x49,0x20},		//   (191)  c - 0x010D Latin Small Letter C with Caron
	{0x7C,0x45,0x46,0x45,0x38},		//   (192)  D - 0x010E Latin Capital Letter D with Caron
	{0x20,0x50,0x50,0x7C,0x03},		//   (193) d' - 0x010F Latin Small Letter D with Caron
	{0x1F,0x15,0x15,0x35,0x51},		//   (194)  E - 0x0118 Latin Capital Letter E with Ogonek
	{0x0E,0x15,0x15,0x35,0x46},		//   (195)  e - 0x0119 Latin Small Letter E with Ogonek
	{0x7C,0x55,0x56,0x55,0x44},		//   (196)  E - 0x011A Latin Capital Letter E with Caron
	{0x38,0x55,0x56,0x55,0x18},		//   (197)  e - 0x011B Latin Small Letter E with Caron
	{0x00,0x44,0x7C,0x40,0x00},		//   (198)  i - 0x0131 Latin Small Letter Dotless I
	{0x7F,0x48,0x44,0x40,0x40},		//   (199)  L - 0x0141 Latin Capital Letter L with Stroke
	{0x00,0x49,0x7F,0x44,0x00},		//   (200)  l - 0x0142 Latin Small Letter L with Stroke
	{0x7C,0x08,0x12,0x21,0x7C},		//   (201) 'N - 0x0143 Latin Capital Letter N with Acute
	{0x78,0x10,0x0A,0x09,0x70},		//   (202) 'n - 0x0144 Latin Small Letter N with Acute
	{0x7C,0x09,0x12,0x21,0x7C},		//   (203)  N - 0x0147 Latin Capital Letter N with Caron
	{0x78,0x11,0x0A,0x09,0x70},		//   (204)  n - 0x0148 Latin Small Letter N with Caron
	{0x38,0x47,0x44,0x47,0x38},		//   (205) "O - 0x0150 Latin Capital Letter O with Double Acute
	{0x30,0x4B,0x48,0x4B,0x30},		//   (206) "o - 0x0151 Latin Small Letter O with Double Acute
	{0x3E,0x41,0x7F,0x49,0x49},		//   (207) OE - 0x0152 Latin Capital Ligature Oe
	{0x38,0x44,0x38,0x54,0x58},		//   (208) oe - 0x0153 Latin Small Ligature Oe
	{0x7C,0x15,0x16,0x35,0x48},		//   (209)  R - 0x0158 Latin Capital Letter R with Caron
	{0x78,0x11,0x0A,0x09,0x10},		//   (210)  r - 0x0159 Latin Small Letter R with Caron
	{0x48,0x54,0x56,0x55,0x20},		//   (211) 'S - 0x015A Latin Capital Letter S with Acute
	{0x20,0x48,0x56,0x55,0x20},		//   (212) 's - 0x015B Latin Small Letter S with Acute
	{0x48,0x55,0x56,0x55,0x20},		//   (213)  S - 0x0160 Latin Capital Letter S with Caron
	{0x20,0x49,0x56,0x55,0x20},		//   (214)  s - 0x0161 Latin Small Letter S with Caron
	{0x04,0x05,0x7E,0x05,0x04},		//   (215)  T - 0x0164 Latin Capital Letter T with Caron
	{0x08,0x3C,0x48,0x22,0x01},		//   (216) t' - 0x0165 Latin Small Letter T with Caron
	{0x3C,0x42,0x45,0x42,0x3C},		//   (217)  U - 0x016E Latin Capital Letter U with Ring Above
	{0x38,0x42,0x45,0x22,0x78},		//   (218)  u - 0x016F Latin Small Letter U with Ring Above
	{0x3C,0x43,0x40,0x43,0x3C},		//   (219) "U - 0x0170 Latin Capital Letter U with Double Acute
	{0x38,0x43,0x40,0x23,0x78},		//   (220) "u - 0x0171 Latin Small Letter U with Double Acute
	{0x0C,0x11,0x60,0x11,0x0C},		//   (221) "Y - 0x0178 Latin Capital Letter Y with Diaeresis
	{0x44,0x66,0x55,0x4C,0x44},		//   (222) 'Z - 0x0179 Latin Capital Letter Z with Acute
	{0xff,0xff,0xff,0xff,0xff},		//   (223) 'z - 0x017A Latin Small Letter Z with Acute
	{0x44,0x64,0x55,0x4C,0x44},		//   (224)  Z - 0x017B Latin Capital Letter Z with Dot Above
	{0x48,0x68,0x5A,0x48,0x00},		//   (225)  z - 0x017C Latin Small Letter Z with Dot Above
	{0x44,0x65,0x56,0x4D,0x44},		//   (226)  Z - 0x017D Latin Capital Letter Z with Caron
	{0x48,0x69,0x5A,0x49,0x00},		//   (227)  z - 0x017E Latin Small Letter Z with Caron
	{0x00,0x02,0x01,0x02,0x00},		//   (228)  ^ - 0x02C6 Modifier Letter Circumflex Accent
	{0x00,0x01,0x02,0x01,0x00},		//   (229)    - 0x02C7 Caron
	{0x00,0x01,0x01,0x01,0x00},		//   (230)    - 0x02C9 Modifier Letter Macron
	{0x01,0x02,0x02,0x01,0x00},		//   (231)    - 0x02D8 Breve
	{0x00,0x00,0x01,0x00,0x00},		//   (232)    - 0x02D9 Dot Above
	{0x00,0x02,0x05,0x02,0x00},		//   (233)    - 0x02DA Ring Above
	{0x02,0x01,0x02,0x01,0x00},		//   (234)  ~ - 0x02DC Small Tilde
	{0x7F,0x05,0x15,0x3A,0x50},		//   (235) Pt - 0x20A7 Peseta Sign
	{0x3E,0x55,0x55,0x41,0x22},		//   (236)  C - 0x20AC Euro Sign
	{0x18,0x14,0x08,0x14,0x0C},		//   (237)    - 0x221E Infinity
	{0x44,0x4A,0x4A,0x51,0x51},		//   (238)  < - 0x2264 Less-Than or Equal to
	{0x51,0x51,0x4A,0x4A,0x44},		//   (239)  > - 0x2265 Greater-Than or Equal to
	{0xff,0xff,0xff,0xff,0xff},		//   (240)    - 0x2302 Block
};

int dispBrightness = 4;

#define	MANUAL_IO

// this RD_LINE should be tied low by D&R
#define	RD_LINE		LATBbits.LATB1
// /CMD Data Line
#define	CD_LINE		LATBbits.LATB0		
// chip select
#define	CS_LINE		LATFbits.LATF12
#define	SDIN		LATFbits.LATF5
#define	SCLK		LATFbits.LATF13

#define	VDD_LINE	LATGbits.LATG15
#define	VCC_LINE	LATAbits.LATA0
#define	RES_LINE	LATBbits.LATB3

void DispInit()
{
	BYTE buf[4];

	TRISBbits.TRISB3 = 0;	// make RES line an output
	RES_LINE = 0;			// hold it reset
	TRISBbits.TRISB1 = 0;	// make E/RD line an output
	RD_LINE = 0;			// must be Low, D&R to fix
	TRISBbits.TRISB0 = 0;	// make C/D line an output
	CD_LINE = 0;			// start with CMD mode
	TRISFbits.TRISF12 = 0;	// make CS line an output
	CS_LINE = 1;			// deselect the chip for now
	TRISGbits.TRISG15 = 0;
	VDD_LINE = 0;			// no VDD yet
	TRISAbits.TRISA0 = 0;
	VCC_LINE = 0;			// no VCC yet

#ifdef	MANUAL_IO
	TRISFbits.TRISF5 = 0;	// make SDIN an output
	TRISFbits.TRISF13 = 0;	// make SCLK an output
	SCLK = 1;
#else
	SPI4STAT = 0x0000;               // power on state 
	SPI4CON = 0x0000;				// legacy mode
	SPI4CONbits.MODE32 = 0;		// 8 bit mode
	SPI4CONbits.MODE16 = 0;
	SPI4CONbits.MSTEN = 1;			// master mode
	SPI4CONbits.CKP = 0;			// this is the polarity, should be ok?
	SPI4CONbits.CKE = 1;
	SPI4CONbits.ENHBUF = 1;
	SPI4BRG = 25;					// 1 mhz

	SPI4CONbits.ON = 1;			// enable synchronous serial port
#endif

	DispPowerOn();

}

static void ByteOut(BYTE b)
{
#ifdef	MANUAL_IO
	int i;
	BYTE mask;

	CS_LINE=0;
	Nop();
	Nop();
	Nop();
	Nop();
	mask = 0x80;
	for (i=0; i<8; i++, mask >>= 1)
	{
		Nop();
		Nop();
		Nop();
		Nop();
		SCLK=0;
		SDIN = (b & mask) ? 1 : 0;
		Nop();
//		uDelay(1);
		SCLK=1;
//		uDelay(1);
	}
//	SCLK=0;
	Nop();
	Nop();
	Nop();
	Nop();
	CS_LINE=1;
#else
	Nop();
	Nop();
	CS_LINE = 0;		// select the chip
	Nop();
	Nop();
	SPI4BUF = b;
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	while(! SPI4STATbits.SPITBE);
	while(! SPI4STATbits.SRMT);
	Nop();
	Nop();
	CS_LINE = 1;		// de-select the chip
#endif

}

void DispCmd(BYTE b)
{
	CD_LINE = 0;		// command mode
	Nop();
	Nop();
	ByteOut(b);
}

static void DispCmd2(BYTE b1, BYTE b2)
{
	DispCmd(b1);
	DispCmd(b2);
}

void DispByte(BYTE b)
{
	CD_LINE = 1;		// data mode
	Nop();
	Nop();
	ByteOut(b);
}


void DispData(BYTE *ptr, int cnt)
{
	CD_LINE = 1;		// data mode
	Nop();
	Nop();
	while(cnt--)
		ByteOut(*ptr++);
}

static void Set_Start_Page(unsigned char d)
{
	DispCmd(0xb0|d);			// Set Page Start Address for Page Addressing Mode
						//   Default => 0xB0 (0x00)
}

static void Set_Start_Column(unsigned char d)
{
	DispCmd(0x00+(d & 0xf));		// Set Lower Column Start Address for Page Addressing Mode
						//   Default => 0x00
	DispCmd(0x10+(d >> 4));		// Set Higher Column Start Address for Page Addressing Mode
						//   Default => 0x10
}

void Set_Contrast_Control(BYTE d)
{
	BYTE ctab[] = { 0, 5, 10, 30, 80, 160 };
	DispCmd2(0x81, ctab[d]);			// Set Contrast Control
}

void Set_Display_On_Off(unsigned char d)	
{
	int tick;

	// display on?, then turn on VCC and wait a bit for stabilization
	if (d)
	{
		VCC_LINE = 1;
		tick = GetTickCount();
		while(! HasElapsed(tick, 100));	// 100 ms delay
	}
	
	DispCmd(0xAE|d);			// Set Display On/Off
						//   Default => 0xAE
						//     0xAE (0x00) => Display Off
						//     0xAF (0x01) => Display On
	if (!d)
		VCC_LINE = 0;		// turn off VCC since display is off
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//  Show Character (5x7)
//
//    a: Database
//    b: Ascii
//    c: Start Page
//    d: Start Column
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

static void SetPos(unsigned char c, unsigned char d)
{
	Set_Start_Page(c);
	Set_Start_Column(d);
}

static int ShowSpace(unsigned char c, unsigned char d, int n)
{
	int j;

	c *= 2;
	j = n;
	while(j--)
	{
		SetPos(c, d);
		DispByte(0x00);		// space
		SetPos(c + 1, d);
		DispByte(0x00);		// space
		d++;
	}
	return n;
}

static int ShowBlock(unsigned char c, unsigned char d, int n)
{
	int j;

	c *= 2;
	j = n;
	while(j--)
	{
		SetPos(c, d);
		DispByte(0xfe);		// line
		SetPos(c + 1, d);
		DispByte(0xff);		// space
		d++;
	}
	return n;
}


static int Show_Font(unsigned char b, unsigned char c, unsigned char d)
{
	BYTE *Src_Pointer;
	FONT_CHAR_INFO *font;
	int address;
	int width = 0;		// default to space width
	
	c *= 2;
	if (b == 0x7d)
	{
		width = ShowBlock(c / 2, d, 6);
		d += width;
	}
	else
	{
		if (b >= 0x21 && b <= 0x80)
		{
			if (b >= 'a')
				b -= 0x20;	// convert to upper case
			font = (FONT_CHAR_INFO *)&fontCharInfo[b - 0x21];
			width = font->charWidth;
			
			Src_Pointer=(BYTE *)fontBitmaps;
			address = font->address;
			Src_Pointer += address;
			Src_Pointer += width;
			SetPos(c, d);
			DispData(Src_Pointer, width);
	
			Src_Pointer -= width;
			SetPos(c + 1, d);
			DispData(Src_Pointer, width);
			d += width;
		}
		else
		{
			if (b == 0x20)	// space
			{
				SetPos(c, d);
				DispByte(0x00);		// space
				SetPos(c + 1, d);
				DispByte(0x00);		// space
				width = 1;
				d += width;
			}
		}
	}
	SetPos(c, d);
	DispByte(0x00);		// space
	SetPos(c + 1, d);
	DispByte(0x00);		// space

	return width + 1;
}

static void Show_Font57(unsigned char b, unsigned char c, unsigned char d)
{
	unsigned char *Src_Pointer;
	unsigned char i;

	Src_Pointer=Ascii_1[b-0x20];
	Set_Start_Page(c);
	Set_Start_Column(d);

	DispData(Src_Pointer, 5);

	DispByte(0x00);		// space
}


void Show_Bar(BYTE b, BYTE c, int maxlen, int len)
{
	int i;

	maxlen -= len;
	while(len > 0)
	{
		c += ShowBlock(b, c, 9);
		c += ShowSpace(b, c, 1);
		len--;
	}
	while(maxlen > 0)
	{
		c += ShowSpace(b, c, 10);
		maxlen--;
	}
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//  Show String
//
//    a: Database
//    b: Start Page
//    c: Start Column
//    * Must write "0" in the end...
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
int Show_String(unsigned char *Data_Pointer, unsigned char b, unsigned char c)
{
//	Show_Font57(96, b, c);			// No-Break Space
						//   Must be written first before the string start...
	while(*Data_Pointer)
	{
		c += Show_Font(*Data_Pointer++, b, c);
//		c+=6;
	}
	return c;
}

void Show_StringFS(unsigned char *Data_Pointer, unsigned char b, unsigned char c)
{
	int len;

	while(*Data_Pointer)
	{
		if (*Data_Pointer == ' ')
			c += ShowSpace(b, c, 10);
		else
		{		
			len = Show_Font(*Data_Pointer, b, c);
			if (len < 10)
				c += 10;
			else
				c += len;
		}
		Data_Pointer++;
	}
}

void ClearScreen()
{
	unsigned char i,j;
	BYTE barray[2];

	barray[0] = 0;
	barray[1] = 0;
	for(i=0;i<8;i++)
	{
		Set_Start_Page(i);
		Set_Start_Column(0x00);

		for(j=0;j<64;j++)
		{
/*			b[0] = 0x55;
			b[1] = 0xaa; */
			DispData(barray, 2);
		}
	}

}

void ClearRest(unsigned char line, int start)
{
	int i;

	line *= 2;

	Set_Start_Page(line);
	Set_Start_Column(start);
	for(i = start; i < 128; i++)
		DispByte(0);
	Set_Start_Page(line + 1);
	Set_Start_Column(start);
	for(i = start; i < 128; i++)
		DispByte(0);
}

void ClearLine(unsigned char line)
{
	ClearRest(line, 0);
}



void Checkerboard()
{
	unsigned char i,j;
	BYTE b[2];

	for(i=0;i<8;i++)
	{
		Set_Start_Page(i);
		Set_Start_Column(0x00);

		for(j=0;j<64;j++)
		{
			b[0] = 0x55;
			b[1] = 0xaa;
			DispData(b, 2);
		}
	}
}


void DispPowerOn()
{
	int tick;

	VDD_LINE = 1;		// power up VDD
	tick = GetTickCount();
	while(! HasElapsed(tick, 20));	// wait for it to be stable
	RES_LINE = 0;		// reset
	tick = GetTickCount();
	while(! HasElapsed(tick, 20));	// long reset
	RES_LINE = 1;
	tick = GetTickCount();
	while(! HasElapsed(tick, 50));	// long reset
	DispCmd(0xae);			// display off
	DispCmd2(0xd5, 0x80);	// set osc freq
	DispCmd2(0xa8, 0x3f);	// set mux ratio
	DispCmd2(0xd3, 0);		// set display offset
	DispCmd2(0x20, 2);		// set page addressing mode
	DispCmd(0x40);			// set display start line
	DispCmd2(0x8d, 0x10);	// disable charge pump regulator
	DispCmd(0xa1);			// set segment remap
	DispCmd(0xc8);			// set COM output scan direction
	DispCmd2(0xda, 0x12);		// set COM hardware pins
	Set_Contrast_Control(4);	// set contrast control
	DispCmd2(0xd9, 0x22);	// pre charge, 2 clocks & 2 clocks
	DispCmd2(0xdb, 0x40);	// set VCOM deselect level
	DispCmd(0xa4);			// disable entire display on
	DispCmd(0xa6);			// set normal display

	VCC_LINE = 1;
	tick = GetTickCount();
	while(! HasElapsed(tick, 250));	// 250 ms delay
	ClearScreen();		// clear display
}

void DispPowerOff()
{
	int tick;

	VCC_LINE = 0;
	tick = GetTickCount();
	while(! HasElapsed(tick, 250));	// 250 ms delay
	VDD_LINE = 0;
}
