/*
	flash programming helper routines
*/
#define _SUPPRESS_PLIB_WARNING 1 

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "logic.h"
#include "flash.h"

const PARAMS __attribute__ ((aligned (BYTE_PAGE_SIZE),section("PARAMS"),space(prog))) FlashParams =
	{ 0, 0, 0, 0, 0, 136, 10, 22, 22, 22 };

PARAMS Params;

const TOUCHPAD __attribute__ ((aligned (BYTE_PAGE_SIZE),section("TOUCH_PAD"),space(prog))) FlashTouchPad =
	{ 0x0 };

void UserFlashPageReadBytes(const void * userFlash, BYTE *buff, WORD numBytes)
{
	// Read numBytes from user page into buff.
	memcpy((void *)buff, userFlash, numBytes);
}
 
void UserFlashPageWriteBytes(DWORD userFlashPage, const WORD addr, BYTE *data, WORD numBytes)
{
	int numBytesLeft = (int)numBytes;
	WORD maxRows;

	while(numBytesLeft > 0)
	{
		// Erase one page (4096 Bytes) of User Program Flash
		NVMErasePage((void *)userFlashPage);
		
		// program this page
		maxRows = NUM_ROWS_PAGE;
		while (numBytesLeft > 0 && maxRows--)
		{
			// Write 512 bytes (one NVM row) from pageBuff. Writes a whole row always.
			NVMWriteRow((void *)(userFlashPage), (void*)data);
			numBytesLeft -= BYTE_ROW_SIZE;
			data += BYTE_ROW_SIZE;
			userFlashPage += BYTE_ROW_SIZE;
		}
	}
}

void SaveTouchPad(void *touchpad, int len)
{
	DWORD FlashPage;
	WORD FlashOffset;

	FlashPage = (DWORD)&FlashTouchPad;
	FlashOffset = FlashPage % BYTE_PAGE_SIZE;
	UserFlashPageWriteBytes((FlashPage / BYTE_PAGE_SIZE) * BYTE_PAGE_SIZE, FlashOffset, (BYTE *)touchpad, len);
}

void ReadParams()
{
	UserFlashPageReadBytes(&FlashParams, (BYTE *)&Params, sizeof(Params));
}

void SaveParams()
{
	DWORD FlashPage;
	WORD FlashOffset;

	FlashPage = (DWORD)&FlashParams;
	FlashOffset = FlashPage % BYTE_PAGE_SIZE;
	UserFlashPageWriteBytes((FlashPage / BYTE_PAGE_SIZE) * BYTE_PAGE_SIZE, FlashOffset, (BYTE *)&Params, sizeof(Params));
}

