/*
	keys.c
*/
#define _SUPPRESS_PLIB_WARNING 1 

#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "timer.h"
#include "keys.h"
#include "logic.h"
#include "display.h"
#include "lamps.h"
#include "thumbdrive.h"
#include "beepalert.h"
#include "flash.h"

/* globals */
key_func_t key_funcs[KEY_COUNT];
key_held_t key_held[KEY_COUNT];
key_func_t slide_func;
tmr_func_t tmr_func;
int currSlidePos;
BOOL keyPressed[KEY_COUNT];
BOOL keyHeld[KEY_COUNT];
BOOL specialKeys;		// for thumb drive we disable the normal keypad
BOOL refreshDisp = TRUE;
int setupState;
/* locals */
#define		SPEC_COUNT	4
#define		MESG_COUNT	8
static char currMesg[MESG_MAX+1];
//static char mesgList[MESG_COUNT][MESG_MAX+1];
static int tempIndex;

// current average
#define	CURR_AVERAGES	8
static int avgs[CURR_AVERAGES];

static float crap;

// used for traffic director display
static int trafficTick;
static int trafficStep = 0;
static int trafficPtr;
static int trafficLamp;
static int trafficLampPtr;
static int setupTick;
static int keyHeldTick[KEY_COUNT];
static int dispTick;
static int dispPtr = 0;
static enum TrafficState traffic = IDLE;

static char *arrowRight[] = { "-           ",  "->          ",  "->-         ", "->->        ",  "->->-       ", "->->->      ",
	"->->->-     ", "->->->->    ", "->->->->-   ", "->->->->->  ", "->->->->->- ","->->->->->->", NULL
};

static char *arrowLeft[] = { "           -", "          <-", "         -<-", "        <-<-", "       -<-<-", "      <-<-<-", 
	"     -<-<-<-","    <-<-<-<-","   -<-<-<-<-", "  <-<-<-<-<-", " -<-<-<-<-<-", "<-<-<-<-<-<-", NULL
};

static char *centerOut[] = { "     --     ", "    <-->    ", "   -<-->-   ", "  <-<-->->  ", " -<-<-->->- ", "<-<-<-->->->",
 NULL
};

static char *flash[] = { "\175          \175",  "\175          \175", "   \175    \175   ", "   \175    \175   ", NULL };
static char *flashDiamonds[] = { " \177       \177",  " \177       \177", "              ", NULL };
static char *chevLeft[] = { "          <", "     <    <", "<    <    <", "           ", NULL };
static char *chevRight[] = {">          ", ">    >     ", ">    >    >", "           ",NULL };
static char *chevOut[] = {  "   <  >    ", "   <  >    ", "<  <   >  >", "<  <   >  >", "           ", NULL };
static char *warningLine[] = { " ----------", " ----------", "            ", NULL };
static char *warningDots[] = { " \200       \200 ", "           ", NULL };

typedef struct
{
	IoPortId port;
	unsigned int bits;
} port_bit_t;

port_bit_t special_keys[SPEC_COUNT] = {
#ifdef AST5500
	{ IOPORT_E, 0x20 },		// RE5, is sw3 on AST5500
	{ IOPORT_E, 0x10 },		// RE4, sw4 on AST5500
	{ IOPORT_E, 0x8 },		// RE3
	{ IOPORT_E, 0x4 },		// RE2
#else
#ifdef RDS16
	{ IOPORT_D, 0x40 },		// RD6
#elif RDS17
	{ IOPORT_D, 0x40 },		// RD6
#endif
	{ IOPORT_F, 0x1 },		// RF0 row 2 col 1
	{ IOPORT_A, 0x80 },		// RA7
	{ IOPORT_A, 0x40 }		// RA6
#ifdef F5TP40NS
	,{ IOPORT_G, 0x1 }		// RG0
#elif F5TP40WS
	,{ IOPORT_G, 0x1 }		// RG0
#elif F5TP40ROT
	,{ IOPORT_G, 0x1 }		// RG0
#endif
#endif
};

port_bit_t key_ports[KEY_COUNT] = {
#ifdef F5TP40ROT
	{ IOPORT_C, 0x10 },		// RC4
	{ IOPORT_E, 0x1 },		// RE0
	{ IOPORT_F, 0x1 },		// RF0
	{ IOPORT_A, 0x80 },		// RA7
	{ IOPORT_A, 0x40 },		// RA6
	{ IOPORT_G, 0x1 },		// RG0
	{ IOPORT_G, 0x2 },		// RG1
	{ IOPORT_F, 0x2 },		// RF1
	{ IOPORT_D, 0x80 },		// RD7
	{ IOPORT_D, 0x40 },		// RD6
	{ IOPORT_D, 0x2000 },	// RD13
	{ IOPORT_D, 0x1000 },	// RD12
	{ IOPORT_D, 0x8 },		// RD3
	{ IOPORT_D, 0x4 },		// RD2
	{ IOPORT_E, 0x20 }		// RE5
#else
#ifdef RDS16
	{ IOPORT_E, 0x20 },		// RE5, is sw1 on RDS16
#endif
#ifdef AST5500
	{ IOPORT_F, 0x1 },		// RF0, sw1 on AST5500
	{ IOPORT_A, 0x80 },		// RA7, sw2 on AST5500
	{ IOPORT_E, 0x20 },		// RE5, is sw3 on AST5500
#endif
#ifndef RDS17
	{ IOPORT_E, 0x10 },		// RE4
#endif
	{ IOPORT_E, 0x8 },		// RE3
	{ IOPORT_E, 0x4 },		// RE2
	{ IOPORT_G, 0x2000 },	// RG13
	{ IOPORT_G, 0x4000 },	// RG14
	{ IOPORT_E, 0x2 },		// RE1
#ifndef AST5500
	{ IOPORT_E, 0x1 },		// RE0
#ifdef RDS16
	{ IOPORT_D, 0x40 },		// RD6
#elif RDS17
	{ IOPORT_D, 0x40 },		// RD6
#endif
	{ IOPORT_F, 0x1 },		// RF0
	{ IOPORT_A, 0x80 },		// RA7
	{ IOPORT_A, 0x40 },		// RA6
	{ IOPORT_G, 0x1 },		// RG0
	{ IOPORT_G, 0x2 },		// RG1
	{ IOPORT_F, 0x2 },		// RF1
	{ IOPORT_D, 0x80 },		// RD7
#ifndef RDS17
#ifndef RDS16
	{ IOPORT_D, 0x40 },		// RD6
	{ IOPORT_D, 0x2000 },	// RD13
	{ IOPORT_D, 0x1000 },	// RD12
	{ IOPORT_D, 0x8 },		// RD3
#ifdef F5TP40NS
	{ IOPORT_D, 0x4 },		// RD2
	{ IOPORT_E, 0x20 }		// RE5
#else
	{ IOPORT_D, 0x4 }		// RD2
#endif
#endif
#endif
#endif
#endif
};

port_bit_t slide_ports[SLIDE_COUNT] = {
// for the large no slide, these don't exist!
#ifdef F5TP40WS
	{ IOPORT_E, 0x0 },		// Dummy, Slide Pos off
	{ IOPORT_E, 0x80 },		// RE7, Slide pos 1
	{ IOPORT_E, 0x40 },		// RE6, Slide pos 2
	{ IOPORT_E, 0x20 }		// RE5, Slide pos 3
#elif RDS17
	{ IOPORT_E, 0x0 },		// Dummy, Slide Pos off
	{ IOPORT_E, 0x20 },		// RE5, Slide pos 3
	{ IOPORT_E, 0x40 },		// RE6, Slide pos 2
	{ IOPORT_E, 0x80 }		// RE7, Slide pos 1
#elif F5TP40ROT
	{ IOPORT_C, 0x2},		// Dummy, Rotary Pos off
	{ IOPORT_E, 0x10 },		// RE4, Rotary pos 1
	{ IOPORT_E, 0x2 },		// RE1, Rotary pos 2
	{ IOPORT_G, 0x4000 },	// RG14 Rotary pos 3 
	{ IOPORT_E, 0x8 },		// RE3, Rotary pos 4
	{ IOPORT_E, 0x4 },		// RE2, Rotary pos 5
	{ IOPORT_G, 0x2000 },	// RG13 Rotary pos 6 
	{ IOPORT_G, 0x1000 }	// RG12 Rotary pos 7 
#endif
};

static int keyTick;		// main line timer tick
static int key;
static int lastKey;
static int setTick;		// setup mode tick
static BOOL lastBacklight;

void KeyInit( void )
{
	int i;

	key = 0;
	lastKey = 0;
	setupState = 0;
	currSlidePos = 0;		// current slide position is off
	*currMesg = '\0';

//	for(i = 0; i < MESG_COUNT; i++)
//		*mesgList[i] = '\0';

	for(i = 0; i < CURR_AVERAGES; i++)
		avgs[i] = 0;
	for(i = 0; i < KEY_COUNT; i++)
	{
		key_funcs[i] = NULL;	// no function assigned yet
		key_held[i] = NULL;		// no held function yet
		keyPressed[i] = FALSE;	// all keys are released to start
		keyHeldTick[i] = GetTickCount();	// for more than 2 second held keys
	}
	slide_func = NULL;
	tmr_func = NULL;

	keyTick = GetTickCount();
	setTick = GetTickCount();
	specialKeys = FALSE;
}

int GetKS()
{
	return key;
}

void ClearKey()
{
	key = 0;
}

int GetK()
{
	int k;

	while(! GetKS());
	k = key;
	ClearKey();
	return k;
}

static void AddMesg(char *msg)
{
	CopyMesg(msg, currMesg, MESG_MAX);
/*
	int i, ndx;
	BOOL fnd = FALSE;


	ndx = -1;
	for(i = 0; i < MESG_COUNT; i++)
	{
		if (*mesgList[i])
		{
			if (! strcmp(mesgList[i], msg))
			{
				fnd = TRUE;
				break;
			}
			else
				ndx = i;
		}
		else
		{
			CopyMesg(msg, mesgList[i], MESG_MAX);
			fnd = TRUE;
			break;
		}
	}
	if (ndx != -1 && ! fnd)		// if we don't already have this same message
		CopyMesg(msg, mesgList[ndx], MESG_MAX); */
}

static void RemoveMesg(char *msg)
{
	if (! strcmp(currMesg, msg))
		*currMesg = '\0';
 
/*	int i;
	for(i = 0; i < MESG_COUNT; i++)
	{
		if (*mesgList[i])
		{
			if (! strcmp(mesgList[i], msg))
			{
				mesgList[i][0] = '\0';
				break;
			}
		} 
	} */
}

void SetTraffic(enum TrafficState t, char *msg)
{
	if (t != IDLE)
	{
		refreshDisp = TRUE;
		if (t == CUSTOM)
			AddMesg(msg);
		else
		{
			trafficTick = GetTickCount();
			trafficStep = 0;
			traffic = t;
		}
	}
}

void ResetTraffic(enum TrafficState t, char *msg)
{
	if (t != IDLE)
	{
		refreshDisp = TRUE;
		if (t == CUSTOM)
			RemoveMesg(msg);
		else
		{
			trafficTick = GetTickCount();
			trafficStep = 0;
			traffic = IDLE;
		}
	}

}

/*
	average 8 readings of current as per D & R
*/
static int Average(int n)
{
	int i, tot;

	for(i = 0; i < CURR_AVERAGES - 1; i++)
		avgs[i] = avgs[i+1];
	avgs[CURR_AVERAGES-1] = n;	
	tot = 0;
	for(i = 0; i < CURR_AVERAGES; i++)
		tot += avgs[i];
	return n;
//	return tot / CURR_AVERAGES;			
}

static void DisplayTasks()
{
	char s[25];
	double d;
	char **text = NULL;
	char *msg;
	int line, n, cnt;

	if (refreshDisp)
	{
		ClearScreen();
		if (! *currMesg && TouchPad.Analog[0])
#ifdef USE_CURRENT
			Show_String("TEMP", 1, 0);
		Show_String("AMPS", 2, 0);
#else
			Show_String("TEMP", 2, 0);
#endif
		Show_String("VOLTS", 3, 0);
		trafficStep = 0;
		dispPtr = 0;
	}

	// handle traffic director
	switch(trafficStep)
	{
		case 0:
			ClearLine(0);
			trafficStep++;
			trafficPtr = 0;
			trafficLamp = 0;
			trafficLampPtr = 0;
			trafficTick = GetTickCount();
			break;
		case 1:
			if (Repeat(&trafficTick, 300))
			{
				switch(traffic)
				{
					case ARROWRIGHT:
						if (trafficLamp >= 7)
							trafficLamp = 0;
						else
						{
							trafficLamp >>= 1;
							trafficLamp |= 4;
						}
						text = arrowRight;
						break;
					case ARROWLEFT:
						if (trafficLamp >= 7)
							trafficLamp = 0;
						else
						{
							trafficLamp <<= 1;
							trafficLamp |= 1;
						}
						text = arrowLeft;
						break;
					case CENTEROUT:
						if (trafficLamp == 0)
							trafficLamp = 2;
						else
							if (trafficLamp == 2)
								trafficLamp = 7;
							else
								trafficLamp = 0;
						text = centerOut;
						break;
					case FLASH:
						if (trafficLamp == 0 || trafficLamp == 4)
							trafficLamp = 1;
						else
							trafficLamp = 4;
						text = flash;
						break;
					case CHEVRONLEFT:
						if (trafficLamp >= 7)
							trafficLamp = 0;
						else
						{
							trafficLamp <<= 1;
							trafficLamp |= 1;
						}
						text = chevLeft;
						break;
					case CHEVRONRIGHT:
						if (trafficLamp >= 7)
							trafficLamp = 0;
						else
						{
							trafficLamp >>= 1;
							trafficLamp |= 4;
						}
						text = chevRight;
						break;
					case CHEVRONCENTER:
						if (trafficLamp == 0)
							trafficLamp = 2;
						else
							if (trafficLamp == 2)
								trafficLamp = 7;
							else
								trafficLamp = 0;
						text = chevOut;
						break;
					case DIAMONDFLASH:
						if (trafficLamp == 0)
							trafficLamp = 5;
						else
							trafficLamp = 0;
						text = flashDiamonds;
						break;
					case WARNINGLINE:
						if (trafficLamp == 0)
							trafficLamp = 7;
						else
							trafficLamp = 0;
						text = warningLine;
						break;
					case WARNINGDOTS:
						if (trafficLamp == 0)
							trafficLamp = 5;
						else
							trafficLamp = 0;
						text = warningDots;
						break;
												
				}
				ArrowLamp(trafficLamp);	// update traffic arrow lamps
				if (text != NULL)
				{
					msg = text[trafficPtr++];
					if (text[trafficPtr] == NULL)
						trafficPtr = 0;
					Show_StringFS(msg, 0, 4);
				}
			}
			break;
	}

	if (Repeat(&dispTick, 1000) || refreshDisp)
	{
		// device is full scale +- 100 amps
		// therefore 1/2 scale is 100 amps / 512 counts * 100 (X 10 since we want to display Amps to one decimal) 
		// at zero amps, the counts should be 512 (1/2 scale)
		// D&R change, they got 3.3 A when should be 4 so X 1.2121
		// i was using 2.367423
		d = (rawCurrent - 512) * Params.CurrentCal;
		n = (int)d;
		if (n > 0)
			n = 0;
		n = -n;		// invert because of hardware
#ifdef USE_CURRENT
		itoan(Average(n), s, 4);
		insdec(s, 1);
		strcpy(s + 5, " A");	// right pad


		ClearRest(2, Show_String(s, 2, 60));
#endif
		// voltage calibration
		// for now assume full scale is 14 volts so 14/1024 = 0.01367
		d = rawVoltage * Params.VoltageCal;
		n = (int)d;
		itoan(n, s, 4);
		insdec(s, 1);
		strcpy(s + 5, " V");

		ClearRest(3, Show_String(s, 3, 60));

		line = 1;
		if (*currMesg)
			ClearRest(1, Show_String(currMesg, 1, 0));
#ifndef USE_CURRENT
		else
		{
#endif
			// display temperature
			if (TouchPad.Analog[0])
			{
				itoan((int)temperature[0], s, 4);
				if (TouchPad.TempC)
					strcpy(s + 4, " C");
				else
					strcpy(s + 4, " F");
#ifdef USE_CURRENT
				ClearRest(1, Show_String(s, 1, 60));
#else
				ClearRest(2, Show_String(s, 2, 60));
#endif
			}
			else
#ifdef USE_CURRENT
				ClearLine(2);
#else
				ClearLine(1);
#endif
#ifndef USE_CURRENT
		}
#endif
	}
	refreshDisp = FALSE;
}

static BOOL ShowTempName()
{
	char s[10];

	while(++tempIndex < ANALOG_CARDS)
		if (TouchPad.Analog[tempIndex])
			break;
	if (tempIndex < ANALOG_CARDS)
	{
		ClearScreen();
		Show_String("Temperature", 1, 0);
		sprintf(s, "Val#%d", tempIndex + 1);
		Show_String(s, 2, 0);
		sprintf(s, "Raw#%d", tempIndex + 1);
		Show_String(s, 3, 0);
	}
	return tempIndex < ANALOG_CARDS;
}

static void ShowInputs(int rbox)
{
	ClearScreen();
	if (rbox)
		Show_String("RELAY BOX", 0, XLevel);
	else
		Show_String("LIGHTBAR", 0, XLevel);
	Show_String("INPUTS", 1, XLevel);
}
		
void KeyTasks()
{
	int i, k;
	unsigned int ioStat;
	BOOL dwn;
	char s[25];
	DWORD dw;

	if (! Repeat(&keyTick, 100))	// only every 100 ms
		return;

	if (specialKeys)		// for setup functions and thumb drive keys
	{
		if (lastKey)
		{
			if (PORTReadBits(special_keys[lastKey - 1].port, special_keys[lastKey - 1].bits))
			{
				lastKey = 0;
				setupTick = GetTickCount();
			}
			else
			{
				if (setupState < 100)		// are we already in calibration mode?
				{
					if (! PORTReadBits(special_keys[0].port, special_keys[0].bits))
						if (HasElapsed(setupTick, 3000))
							setupState = 100;		// calibration mode
				}
			}
		}
		else
		{
			setupTick = GetTickCount();	
			for(i = 0; i < SPEC_COUNT; i++)
			{
				if (! PORTReadBits(special_keys[i].port, special_keys[i].bits))
				{
					lastKey = i + 1;
					key = i + 1;
					break;
				}
			}
		}
	}
	else
	{
		if (! thumbState)
		{
			DisplayTasks();
			// setup mode?
			if (! PORTReadBits(special_keys[0].port, special_keys[0].bits) && ! PORTReadBits(special_keys[2].port, special_keys[2].bits) )
			{
				if (HasElapsed(setupTick, 3000))
				{
					setupState = 1;
					lastKey = 1;
					specialKeys = TRUE;
					setTick = GetTickCount();
					setupTick = GetTickCount();
					// turn off any running keys!
					AllKeysOff();
				}
			}
			else
				setupTick = GetTickCount();
		}
	}

	if (setupState)
	{
		switch(setupState)
		{
			case 1:
				DisplayLamp(TRUE);
				ClearScreen();
				Show_String("Setup Mode", 0, XLevel);
				Show_String("Back Light", 1, XLevel);
				Show_Bar(2, XLevel, 6, blBrightness + 1);
				lastBacklight = backLight;
				backLight = TRUE;
				setupState++;
				setTick = GetTickCount();
				break;
			case 2:
				if(k = GetKS())
				{
					setTick = GetTickCount();
					ClearKey();
					switch(k)
					{
						case 1:
							ClearLine(1);
							Show_String("Key Bright", 1, XLevel);
							ClearLine(2);
							Show_Bar(2, XLevel, 6, btBrightness + 1);
							backLight = lastBacklight;
							setupState++;
							break;
						case 2:
							if (blBrightness)
								Show_Bar(2, XLevel, 6, --blBrightness + 1);
							break;
						case 3:
							if (blBrightness < 4)
								Show_Bar(2, XLevel, 6, ++blBrightness + 1);
							break;
					}
				}
				break;
			case 3:
				if(k = GetKS())
				{
					setTick = GetTickCount();
					ClearKey();
					switch(k)
					{
						case 1:
							ClearLine(1);
							Show_String("Disp Bright", 1, XLevel);
							ClearLine(2);
							Show_Bar(2, XLevel, 6, dispBrightness + 1);
							backLight = lastBacklight;
#ifdef RDS16
							setupState = 9;		// RDS16 or 17, we're done
#elif RDS17
							setupState = 9;
#else
							setupState++;
#endif
							break;
						case 2:
							if (btBrightness)
								Show_Bar(2, XLevel, 6, --btBrightness + 1);
							break;
						case 3:
							if (btBrightness < 2)
								Show_Bar(2, XLevel, 6, ++btBrightness + 1);
							break;
					}
				}
				break;
			case 4:
				if(k = GetKS())
				{
					setTick = GetTickCount();
					ClearKey();
					switch(k)
					{
						case 1:
							setupState = 8;
							break;
						case 2:
							if (dispBrightness)
							{
								Show_Bar(2, XLevel, 6, --dispBrightness + 1);
								Set_Contrast_Control(dispBrightness);
							}
							break;
						case 3:
							if (dispBrightness < 5)
							{
								Show_Bar(2, XLevel, 6, ++dispBrightness + 1);
								Set_Contrast_Control(dispBrightness);
							}
							break;
					}
				}
				break;
			case 8:
				ClearScreen();
				Show_String("Goodbye...", 0, XLevel);
				setTick = GetTickCount();
				setupState = 9;
				break;
			case 9:
				if (HasElapsed(setTick, 1000))
				{
					// save settings to flash
					if (ValidTouchPad())
					{
						TouchPad.ButtonIntensity = btBrightness;
						TouchPad.BacklightIntensity = blBrightness;
						TouchPad.DisplayIntensity = dispBrightness;
						SaveTouchPad(&TouchPad, sizeof(TouchPad));
					}

					setupState = 0;
					specialKeys = FALSE;
					refreshDisp = TRUE;
					backLight = lastBacklight;
					DisplayLamp(FALSE);
				}
				break;

			case 100:
				ClearKey();
				setupState = 102;
				if (ValidTouchPad())
					if(TouchPad.RBExists)
					{
						ShowInputs(1);
						setupState = 101;
					}
				break;
			case 101:		// relay box inputs
				if (k = GetKS())
				{
					ClearKey();
					setTick = GetTickCount();
					if (k == 1)
						setupState = 102;
				}
				else
				{
					setTick = GetTickCount();
					strcpy(s, "");
					dw = lastRelayInputs;
					for(i = 0; i < 8; i++)
					{
						k = dw & 3;
						if (k == 1)
							strcat(s, "L ");
						else
							if (k == 2)
								strcat(s, "H ");
							else
								strcat(s, "O ");
						dw >>= 2;
					}
					ClearRest(2, Show_String(s, 2, 0));
				}
				break;
			case 102:
				setupState = 104;
				if (ValidTouchPad())
					if(TouchPad.LBExists)
					{
						ShowInputs(0);
						setupState = 103;
					}
				break;
			case 103:		// light bar inputs
				if (k = GetKS())
				{
					ClearKey();
					setTick = GetTickCount();
					if (k == 1)
					{
						setupState = 104;
					}
				}
				else
				{
					setTick = GetTickCount();
					strcpy(s, "");
					dw = lastLightBarInputs;
					for(i = 0; i < 8; i++)
					{
						k = dw & 3;
						if (k == 1)
							strcat(s, "L ");
						else
							if (k == 2)
								strcat(s, "H ");
							else
								strcat(s, "O ");
						dw >>= 2;
					}
					ClearRest(2, Show_String(s, 2, 0));
					strcpy(s, "");
					for(i = 8; i < 16; i++)
					{
						k = dw & 3;
						if (k == 1)
							strcat(s, "L ");
						else
							if (k == 2)
								strcat(s, "H ");
							else
								strcat(s, "O ");
						dw >>= 2;
					}
					ClearRest(3, Show_String(s, 3, 0));
				}
				break;
			case 104:
				setupState = 106;
				if (ValidTouchPad())
				{
					tempIndex = -1;
					if (ShowTempName())
						setupState = 105;
				}
				break;
			case 105:		// temperature data
				if (k = GetKS())
				{
					ClearKey();
					setTick = GetTickCount();

					switch(k)
					{
						case 1:
							if (rawTemp[tempIndex])		// cannot be zero!
							{
								crap = (float)(Params.tempCalValue[tempIndex] + 55) / (float)rawTemp[tempIndex];
								Params.TempCal[tempIndex] = crap;
								SaveParams();
							}
							if (! ShowTempName())
								setupState = 106;
							break;
						case 2:
							if (++Params.tempCalValue[tempIndex] > 30)
								Params.tempCalValue[tempIndex] = 10;
							break;
						case 3:
							if (--Params.tempCalValue[tempIndex] < 10)
								Params.tempCalValue[tempIndex] = 30;
							break;
					}

				}
				else
				{
					itoan((int)Params.tempCalValue[tempIndex], s, 3);
					ClearRest(2, Show_String(s, 2, 80));
/*					itoan((int)temperature[tempIndex], s, 4);
					if (TouchPad.TempC)
						strcpy(s + 4, " C");
					else
						strcpy(s + 4, " F");
					ClearRest(2, Show_String(s, 2, 80)); */
					itoan((int)rawTemp[tempIndex], s, 4);
					ClearRest(3, Show_String(s, 3, 80));
				}
				break;
			case 106:
				setupState = 108;
				if (ValidTouchPad())
				{
					if (TouchPad.RBExists)
					{	
						ClearScreen();
						Show_String("CALIBRATION", 0, XLevel);
						Show_String("VOLTAGE", 1, XLevel);
						Show_String("VALUE", 2, XLevel);
						Show_String("RAW", 3, XLevel);
						setupState = 107;
					}
				}
				break;
			case 107:
				if (k = GetKS())
				{
					setTick = GetTickCount();
					ClearKey();
					switch(k)
					{
						case 1:
							if (rawVoltage)		// cannot be zero!
							{
								Params.VoltageCal = (float)Params.voltCalValue / (float)rawVoltage;
								SaveParams();
							}
							setupState = 108;
							break;
						case 2:
							if (++Params.voltCalValue > 140)
								Params.voltCalValue = 110;
							break;
						case 3:
							if (--Params.voltCalValue < 110)
								Params.voltCalValue = 140;
							break;
					}
					
				}
				else
				{
					itoan((int)Params.voltCalValue, s, 3);
					insdec(s, 1);
					ClearRest(2, Show_String(s, 2, 80));
					itoan((int)rawVoltage, s, 4);
					ClearRest(3, Show_String(s, 3, 80));
				}
				break;
			case 108:
				setupState = 8;
#ifdef USE_CURRENT
				if (ValidTouchPad())
				{
					if (TouchPad.RBExists)
					{	
						ClearScreen();
						Show_String("CALIBRATION", 0, XLevel);
						Show_String("CURRENT", 1, XLevel);
						Show_String("VALUE", 2, XLevel);
						Show_String("RAW", 3, XLevel);
						setupState = 109;
					}
				}
#endif
				break;
			case 109:
				if (k = GetKS())
				{
					setTick = GetTickCount();
					ClearKey();
					switch(k)
					{
						case 1:
							k = 512 - rawCurrent;
							if (k < 0)
								k = 0;
							if (k)		// cannot be zero!
							{
								Params.CurrentCal = (float)Params.currCalValue / (float)k;
								SaveParams();
							}
							setupState = 8;
							break;
						case 2:
							if (++Params.currCalValue > 100)
								Params.currCalValue = 2;
							break;
						case 3:
							if (--Params.currCalValue < 2)
								Params.currCalValue = 100;
							break;
					}
					
				}
				else
				{
					itoan((int)Params.currCalValue, s, 3);
					insdec(s, 1);
					ClearRest(2, Show_String(s, 2, 80));
					itoan((int)rawCurrent, s, 4);
					ClearRest(3, Show_String(s, 3, 80));
				}
				break;
		}
		// time out of special mode
		if (HasElapsed(setTick, 45000))
		{
			setTick = GetTickCount();
			setupState = 9;		// abort setup
		}
	}

	if (! specialKeys && ! setupState)
	{
		// slide switch logic
// the rotary position is open!
		for(i = 1; i < SLIDE_COUNT; i++)
		{
			if (i != currSlidePos)
			{
				if(! PORTReadBits(slide_ports[i].port, slide_ports[i].bits))
				{
					if (slide_func != NULL)
					{
						slide_func(currSlidePos, FALSE);
						slide_func(i, TRUE);
					}
					currSlidePos = i;
					break;
				}
			}
			else
			{
				if (PORTReadBits(slide_ports[i].port, slide_ports[i].bits))
				{
					if (slide_func != NULL)
					{
						slide_func(currSlidePos, FALSE);
						slide_func(0, TRUE);
					}
					currSlidePos = 0;
					break;
				}
			}
		}
	
		for(i = 0; i < KEY_COUNT; i++)
		{
			if (key_funcs[i] != NULL)
			{
				if (keyPressed[i])
				{
					if (PORTReadBits(key_ports[i].port, key_ports[i].bits))
					{
						key_funcs[i](i, FALSE);
						keyPressed[i] = FALSE;
					}
					else
					{
						if (! keyHeld[i])
							if (HasElapsed(keyHeldTick[i], 2000))	// 2 seconds
							{
								keyHeld[i] = TRUE;
								key_held[i](i);		// fire the event
							}
					}
				}
				else
				{
					if (! PORTReadBits(key_ports[i].port, key_ports[i].bits))
					{
						key_funcs[i](i, TRUE);
						keyPressed[i] = TRUE;
						keyHeldTick[i] = GetTickCount();
						keyHeld[i] = FALSE;
						Buzzer(25);
					}
				}	
			}
		}
	}
	// 100 ms timer event
	if (tmr_func != NULL && ! thumbState)
		tmr_func();
}

/*
	used by pb and ign logic
*/

BOOL IsSlideOn(int key)
{
	return (! PORTReadBits(slide_ports[key].port, slide_ports[key].bits));
}

