/*
	logic.h
*/

#ifndef _logic_h_
#define _logic_h_

#define	MAX_NAME_LEN	40
#define	MAX_PROGS		15
/* this touch pads personality */
#define	EXP_CARDS		2
#define	ANALOG_CARDS	3
// for large touch pad with no slide switch
#ifdef F5TP40WS
#define	KEY_COUNT		19
#define	SLIDE_COUNT		4
#define	TOUCHPAD_TYPE	0
#elif F5TP40NS
#define	KEY_COUNT		20
#define	SLIDE_COUNT		0
#define	TOUCHPAD_TYPE	1
#elif RDS17
#define	KEY_COUNT		14
#define	SLIDE_COUNT		4
#define	TOUCHPAD_TYPE	2
#elif RDS16
#define	KEY_COUNT		16
#define	SLIDE_COUNT		0
#define	TOUCHPAD_TYPE	3
#elif F5TP40ROT
#define	KEY_COUNT		15
#define	SLIDE_COUNT		8
#define	TOUCHPAD_TYPE	4
#elif AST5500
#define	KEY_COUNT		9
#define	SLIDE_COUNT		0
#define	TOUCHPAD_TYPE	5
#endif
#define	VIRT_COUNT		16
#define	RELAYBOX_INPUTS	11
#define	OUTPUT_COUNT	20
#define	EXP_OUTPUTS		10
#define	LIGHTBAR_INPUTS	16
#define	MESG_MAX		16

enum OutState
{
	STATE_NONE,
	STATE_ON,
	STATE_OFF
};

enum KeyState
{
	MOMENTARY,
	TOGGLE,
	CYCLE,
	CYCLEOFF,
	TIMER,
	TEMP_TRIGGER
};

enum TriggerState
{
	POSITIVE,
	NEGATIVE
};

enum TrafficState
{
	IDLE,
	ARROWRIGHT,
	ARROWLEFT,
	CENTEROUT,
	FLASH,
	CHEVRONLEFT,
	CHEVRONRIGHT,
	CHEVRONCENTER,
	DIAMONDFLASH,
	WARNINGLINE,
	WARNINGDOTS,
	CUSTOM
};

enum Condition
{
	IGNORE,
	ACTIVE,
	INACTIVE
};

typedef struct
{
	enum KeyState KeyType;	// 0-momentary, 1-toggle, 2-Cycle, 3-Cycle w/off, 4-timer
	long TimerValue;
	BOOL Active;		// this input is currently activated
	long RunTimer;		// current active running timer 
	enum TriggerState InputTrigger;		// positive or negative trigger state
	int SirenMode;			// zero is none, 1 - siren, 2 - standby
	int SirenTone[2];		// zero is none, Note: now 2 siren outputs
	double VoltSetpoint;
	BOOL VoltActiveAbove;
	double TempSetpoint;
	int TempDevice;		// temperature device
	BOOL TempActiveAbove;
	BOOL RelayBox;		// this system has a relay box
	int LightBarPrograms[MAX_PROGS];	// light bar program indexes
	int LightIndex;		// current selected program
	int Priority;			// light bar program priority
	BOOL Dimmed;
	BOOL CanRunFast;		// light bar program can run fast
	int TrafficProgram;		// traffic director program index
	int TrafficPriority;
	enum TrafficState TrafficDisplay;
	char TrafficMesg[MESG_MAX+1];
	BOOL DispFlag;
	char DispMesg[MESG_MAX+1];
	enum TrafficState KeyDisplay;
	char KeyMesg[MESG_MAX+1];
	enum Condition IgnCondition;
	enum Condition PBCondition;
	BOOL SoftwareDimmer;	// dimmer function
	BOOL FastPatterns;		// fast pattern function
	int FuncCondition;	// new, function conditional

	enum OutState Buttons[KEY_COUNT];
	enum OutState VirtButtons[VIRT_COUNT];
	BOOL Outputs[OUTPUT_COUNT];
	BOOL OutExp1[EXP_OUTPUTS];
	BOOL OutExp2[EXP_OUTPUTS];
} INPUT_DATA;

typedef struct
{
	BYTE chkByte1;
	char Name[MAX_NAME_LEN + 1];
	char TPName[MAX_NAME_LEN + 1];
	BOOL RBExists;
	BOOL LBExists;
	BOOL Exp[EXP_CARDS];
	BOOL Analog[ANALOG_CARDS];
	BOOL TempC;
	int BeepAlert;
	int BattVoltInput;
	int DisplayIntensity;
	int ButtonIntensity;
	int BacklightIntensity;
	int IgnTimeout;		// ignition timeout
	int LightBarType;	// zero is standard, above that is Odyssey 
	int AlwaysStandby;	// permanent standby 3 is air horn, 4 is manual, 0 is off

	INPUT_DATA KeyData[KEY_COUNT];
	INPUT_DATA VirtData[VIRT_COUNT];
	INPUT_DATA SlideData[SLIDE_COUNT];
	INPUT_DATA RelayBoxInputs[RELAYBOX_INPUTS];
	INPUT_DATA AnalogData[ANALOG_CARDS];
	INPUT_DATA LightBarInputs[LIGHTBAR_INPUTS];
	BYTE chkByte2;
} TOUCHPAD;

extern TOUCHPAD TouchPad;
extern int rawCurrent;		// used for system current
extern int rawVoltage;		// user for system voltage
extern DWORD lastRelayInputs;
extern DWORD lastLightBarInputs;
extern double temperature[];	// converted temperature reading
extern int rawTemp[];		// raw temperature
void LogicInit();
void AllKeysOff();

#endif
