/********************************************************************
 FileName:     main.c
 Dependencies: See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Description
  1.0   Initial release
  2.1   Updated for simplicity and to use common
                     coding style
********************************************************************/
#define _SUPPRESS_PLIB_WARNING 1 
#include "USB/usb.h"
#include "USB/usb_host_msd.h"
#include "USB/usb_host_msd_scsi.h"
#include "MDD File System/FSIO.h"
#include "lamps.h"
#include "timer.h"
#include "thumbdrive.h"
#include "display.h"
#include "keys.h"
#include "logic.h"
#include "comm.h"

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

#if defined( __PIC32MX__ )
    #pragma config UPLLEN   = ON            // USB PLL Enabled
    #pragma config FPLLMUL  = MUL_15        // PLL Multiplier
    #pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
    #pragma config FPLLODIV = DIV_1         // PLL Output Divider
    #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    #pragma config FWDTEN   = ON           	// Watchdog Timer
    #pragma config WDTPS    = PS1024        // Watchdog Timer Postscale, about 1 sec timeout
    //#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
    #pragma config OSCIOFNC = OFF           // CLKO Enable
    #pragma config POSCMOD  = HS            // Primary Oscillator
    #pragma config IESO     = OFF           // Internal/External Switch-over
    #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
    #pragma config FNOSC    = PRIPLL        // Oscillator Selection
    #pragma config CP       = OFF           // Code Protect
    #pragma config BWP      = OFF           // Boot Flash Write Protect
    #pragma config PWP      = OFF           // Program Flash Write Protect
    #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
   
#else
    #error Cannot define configuration bits.
#endif

int main(void)
{
	int tick;
	int  value;
	BYTE b[2];

	value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
	
	// Enable the cache for the best performance
	CheKseg0CacheOn();

	INTEnableSystemMultiVectoredInt();
	
	value = OSCCON;
	while (!(value & 0x00000020))
	{
		 value = OSCCON;    // Wait for PLL lock to stabilize
	}
	AD1PCFG = 0xffff;		// make all analog inputs be digital lines

	Timer3Init();

	LampInit();

    //Initialize the stack
//	ODCBbits.ODCB5 = 1;		// make USB Power Open Drain
//    TRISBbits.TRISB5 = 0;	// make USB pwr an output
//	LATBbits.LATB5 = 0;		// turn on USB power
	USBInitialize(0);

    #if defined(DEBUG_MODE)
        // PPS - Configure U2RX - put on pin 49 (RP10)
        RPINR19bits.U2RXR = 10;

        // PPS - Configure U2TX - put on pin 50 (RP17)
        RPOR8bits.RP17R = 5;

        UART2Init();
    #endif


	// init buzzer
	BeepAlertInit();

	DispInit();
	ClearWDT();
	
	KeyInit();
	LogicInit();
	CommInit();

	ReadParams();		// read calibration

	Set_Display_On_Off(1);
	ClearScreen();
	ForceReset(RELAY_ADDRESS);		// force Relay panel to reset
	ForceReset(LIGHT_ADDRESS);		// force light bar to reset

	tick = GetTickCount();
	value = 0;
    while(1)
    {
/*		if (Repeat(&tick, 10000))
		{
			if (value)
				U1PWRC = 1;
			else
				U1PWRC = 0;
			value = ! value;
		} */
		ClearWDT();
		KeyTasks();
		ThumbTasks();
		BeepAlertTasks();
    }
    return 0;
}

