/*
	thumbdrive.c
*/
#define _SUPPRESS_PLIB_WARNING 1 

#include "USB/usb.h"
#include "USB/usb_host_msd.h"
#include "USB/usb_host_msd_scsi.h"
#include "MDD File System/FSIO.h"
#include "logic.h"
#include "util.h"
#include "flash.h"
#include "comm.h"
#include "display.h"
#include "keys.h"

#define	MAX_LINE  700
#define	MAX_FIELDS	 75
#define	MAX_FILES	32

static int state = 0;
static int rptr, rcnt;
static FSFILE *myFile;
static BOOL eof;
static volatile BOOL deviceAttached = FALSE;

static BYTE lineBuf[MAX_LINE];
static int errCode;
static char *fields[MAX_FIELDS];
static int fldCnt;
static int inpNdx;
static WORD crcMain;
static char lastfield[64];
static TOUCHPAD TmpTouch;
// for directories
static int fileCount = 0;			// number of files found on the thumb drive
static char **fileNames;		// pointer to a list of file names found
static int fileNdx;
static int topLine;
int thumbState = 0;

const WORD crc16tab[] = {
	0x0000,0x1021,0x2042,0x3063,0x4084,0x50a5,0x60c6,0x70e7,
	0x8108,0x9129,0xa14a,0xb16b,0xc18c,0xd1ad,0xe1ce,0xf1ef,
	0x1231,0x0210,0x3273,0x2252,0x52b5,0x4294,0x72f7,0x62d6,
	0x9339,0x8318,0xb37b,0xa35a,0xd3bd,0xc39c,0xf3ff,0xe3de,
	0x2462,0x3443,0x0420,0x1401,0x64e6,0x74c7,0x44a4,0x5485,
	0xa56a,0xb54b,0x8528,0x9509,0xe5ee,0xf5cf,0xc5ac,0xd58d,
	0x3653,0x2672,0x1611,0x0630,0x76d7,0x66f6,0x5695,0x46b4,
	0xb75b,0xa77a,0x9719,0x8738,0xf7df,0xe7fe,0xd79d,0xc7bc,
	0x48c4,0x58e5,0x6886,0x78a7,0x0840,0x1861,0x2802,0x3823,
	0xc9cc,0xd9ed,0xe98e,0xf9af,0x8948,0x9969,0xa90a,0xb92b,
	0x5af5,0x4ad4,0x7ab7,0x6a96,0x1a71,0x0a50,0x3a33,0x2a12,
	0xdbfd,0xcbdc,0xfbbf,0xeb9e,0x9b79,0x8b58,0xbb3b,0xab1a,
	0x6ca6,0x7c87,0x4ce4,0x5cc5,0x2c22,0x3c03,0x0c60,0x1c41,
	0xedae,0xfd8f,0xcdec,0xddcd,0xad2a,0xbd0b,0x8d68,0x9d49,
	0x7e97,0x6eb6,0x5ed5,0x4ef4,0x3e13,0x2e32,0x1e51,0x0e70,
	0xff9f,0xefbe,0xdfdd,0xcffc,0xbf1b,0xaf3a,0x9f59,0x8f78,
	0x9188,0x81a9,0xb1ca,0xa1eb,0xd10c,0xc12d,0xf14e,0xe16f,
	0x1080,0x00a1,0x30c2,0x20e3,0x5004,0x4025,0x7046,0x6067,
	0x83b9,0x9398,0xa3fb,0xb3da,0xc33d,0xd31c,0xe37f,0xf35e,
	0x02b1,0x1290,0x22f3,0x32d2,0x4235,0x5214,0x6277,0x7256,
	0xb5ea,0xa5cb,0x95a8,0x8589,0xf56e,0xe54f,0xd52c,0xc50d,
	0x34e2,0x24c3,0x14a0,0x0481,0x7466,0x6447,0x5424,0x4405,
	0xa7db,0xb7fa,0x8799,0x97b8,0xe75f,0xf77e,0xc71d,0xd73c,
	0x26d3,0x36f2,0x0691,0x16b0,0x6657,0x7676,0x4615,0x5634,
	0xd94c,0xc96d,0xf90e,0xe92f,0x99c8,0x89e9,0xb98a,0xa9ab,
	0x5844,0x4865,0x7806,0x6827,0x18c0,0x08e1,0x3882,0x28a3,
	0xcb7d,0xdb5c,0xeb3f,0xfb1e,0x8bf9,0x9bd8,0xabbb,0xbb9a,
	0x4a75,0x5a54,0x6a37,0x7a16,0x0af1,0x1ad0,0x2ab3,0x3a92,
	0xfd2e,0xed0f,0xdd6c,0xcd4d,0xbdaa,0xad8b,0x9de8,0x8dc9,
	0x7c26,0x6c07,0x5c64,0x4c45,0x3ca2,0x2c83,0x1ce0,0x0cc1,
	0xef1f,0xff3e,0xcf5d,0xdf7c,0xaf9b,0xbfba,0x8fd9,0x9ff8,
	0x6e17,0x7e36,0x4e55,0x5e74,0x2e93,0x3eb2,0x0ed1,0x1ef0
};

// error codes
#define	FIELD_COUNT			1
#define	INVALID_VERSION		2
#define	WRONG_TOUCHPAD		3
#define	UNEXPECTED_RECORD	4
#define	LIGHTBAR_COMM		5
#define	RELAY_COMM			6
#define	FILE_ERR			7
#define	USB_ERR				8
#define	MEMORY_ERR			9
#define	FILE_NOT_FOUND		10

#define	FILE_OK				15


static WORD CalcCRC16(WORD crc, BYTE b)
{
	return ((crc << 8) ^ crc16tab[((crc >> 8) ^ b) & 0x00FF]);
}

int GetFileCount(const char *path)
{
	int cnt = 0;
	SearchRec rec;

	if(FindFirst(path, 0xffff, &rec) == 0)		// found a matching file?
	{
		ClearWDT();
		cnt++;
		while(FindNext(&rec) == 0)
			cnt++;
	}

	return cnt;
}

int GetFileList(const char *path)
{
	int i;
	SearchRec rec;
	fileCount = GetFileCount(path);
	
	if(fileCount)
	{
		// make room for the file names
		if((fileNames = (char **)malloc((sizeof (char *)) * fileCount)) != NULL)
		{
			for(i = 0; i < fileCount; i++)
				fileNames[i] = NULL;		// mark them not taken yet
			if(FindFirst(path, 0xffff, &rec) == 0)
			{
				if((fileNames[0] = (char *)malloc(FILE_NAME_SIZE + 2)) != NULL)
					strcpy(fileNames[0], rec.filename);
				for(i = 1; i < fileCount; i++)
					if(FindNext(&rec) == 0)
					{
						ClearWDT();
						if((fileNames[i] = (char *)malloc(FILE_NAME_SIZE + 2)) != NULL)
							strcpy(fileNames[i], rec.filename);
						else
							break;
					}
					else
						break;
				// memory check, make sure we didn't run out!!
				for(i = 0; i < fileCount; i++)
					if(fileNames[i] == NULL)
					{
						fileCount = i;
						break;
					}
				if(! fileCount)
				{
					free(fileNames);
					fileCount = -1;		// out of memory!!
				}
			}
			else
			{
				free(fileNames);
				fileCount = -1;		// out of memory!!
			}
		}
		else
			fileCount = -1;		// out of memory!!
	}

	return fileCount;
}
				
void FreeFileNames(void)
{
	int i;

	if(fileCount > 0)
	{
		for(i = 0; i < fileCount; i++)
			if(fileNames[i] != NULL)
				free(fileNames[i]);
		free(fileNames);
	}
	fileCount = 0;
}



static char ReadChar(BOOL *eof)
{
	char c = 0;

	if (rptr >= rcnt)
	{
		if (rcnt < MAX_LINE)
			*eof = TRUE;
		else
		{
			rcnt = FSfread(lineBuf, 1, MAX_LINE, myFile); 
			rptr = 0;
			if (rptr >= rcnt)
				*eof = TRUE;
		}
	}

	return lineBuf[rptr++];
}

static BOOL ReadLine(char *buf, char *buf2)
{
	char c;
	int cnt = 0;
	BOOL eof = FALSE;
	fldCnt = 0;
	char *currFld = buf;
	BOOL crcFlag = FALSE;
	WORD crcLine = 0;
	WORD crcTmp = 0;

	ClearWDT();
	while(! eof)
	{
		c = ReadChar(&eof);
		
		if (c == '\r')
			break;
		if (c != '\n')
		{
			if (cnt < MAX_LINE - 1)
			{
				if (c == '|' && fldCnt < MAX_FIELDS - 1)
				{
					fields[fldCnt++] = currFld;	// save previous field
					buf[cnt] = '\0';		// null terminate previous field
					buf2[cnt++] = c;
					currFld = buf + cnt;	// point to next field
				}
				else
				{
					buf[cnt] = c;
					buf2[cnt++] = c;
					if (c == '^')
						crcFlag = TRUE;
				}
				if (! crcFlag)
				{
					crcLine = CalcCRC16(crcLine, (BYTE) c);
					crcMain = CalcCRC16(crcMain, (BYTE) c);
				}
				else
				{
					if (c != '^')
					{
						crcTmp *= 10;		// parse the CRC and convert to int
						crcTmp += c & 0xf;
					}
				}
			}
		}	
	}
//	if (crcTmp != crcLine)	// is the line CRC ok?
//		errCode = 30;		// bad CRC error code
	if (fldCnt)
		fields[fldCnt++] = currFld;
	buf[cnt] = '\0';
	buf2[cnt] = '\0';
	return eof;
}

static long LongValue(int ndx)
{
	return atol(fields[ndx]);
}

static int IntValue(int ndx)
{
	return atoi(fields[ndx]);
}

static double DblValue(int ndx)
{
	return atof(fields[ndx]);
}

static BOOL BoolValue(int ndx)
{
	return *fields[ndx] == 'T';
}

static void RecordTypeC(char *line)
{
	int i;

	if (*line == 'C')
	{
		if (fldCnt >= 21)
		{
			if (IntValue(1) == 1)	// check version number
			{
				if (IntValue(3) == TOUCHPAD_TYPE)	// check touch pad type index
				{
					copymax(TmpTouch.Name, fields[2], MAX_NAME_LEN);
					copymax(TmpTouch.TPName, fields[4], MAX_NAME_LEN);
					TmpTouch.LBExists = BoolValue(5);
					TmpTouch.RBExists = BoolValue(6);
					TmpTouch.IgnTimeout = IntValue(7);
					TmpTouch.BeepAlert = IntValue(8);
					TmpTouch.BattVoltInput = IntValue(9);
					TmpTouch.DisplayIntensity = IntValue(10);
					TmpTouch.ButtonIntensity = IntValue(11);
					TmpTouch.BacklightIntensity = IntValue(12);
					for(i = 0; i < EXP_CARDS; i++)
						TmpTouch.Exp[i] = BoolValue(i + 13);
					for(i = 0; i < ANALOG_CARDS; i++)
						TmpTouch.Analog[i] = BoolValue(i + EXP_CARDS + 13);
					TmpTouch.TempC = BoolValue(ANALOG_CARDS + EXP_CARDS + 13);
					TmpTouch.LightBarType = IntValue(ANALOG_CARDS + EXP_CARDS + 14);
					TmpTouch.AlwaysStandby = IntValue(ANALOG_CARDS + EXP_CARDS + 15);
					state++;
				}
				else
					errCode = WRONG_TOUCHPAD;		// wrong touch pad type
			}
			else
				errCode = INVALID_VERSION;	// invalid version
		}
		else
			errCode = FIELD_COUNT;	// invalid field count record type C
	}
}

void CopyMesg(char *src, char *msg, int len)
{
	memmove(msg, src, len);
	msg[len] = '\0';
}

static void RecordTypeX(char *line, char *line2, char lineType, INPUT_DATA *id)
{
	int i, ndx, iTmp;
	int btnNdx = 0;

	if (*line == lineType)
	{
		if (fldCnt >= 44)
		{
			id->Active = FALSE;		// not active yet
			id->RunTimer = 0;		// active timer
			id->KeyType = (enum KeyState)IntValue(3);
			id->TimerValue = LongValue(4) * 2;
			id->InputTrigger = (enum TriggerState)IntValue(5);
			id->SirenMode = IntValue(6);
			id->SirenTone[0] = IntValue(7) + 1;		// since zero no longer applies
			id->SirenTone[1] = IntValue(8);
			id->VoltSetpoint = DblValue(9);
			id->VoltActiveAbove = BoolValue(10);
			id->TempDevice = IntValue(11);			
			id->TempSetpoint = DblValue(12);
			id->TempActiveAbove = BoolValue(13);
			ndx = 14;
			for(i = 0; i < MAX_PROGS; i++)
				id->LightBarPrograms[i] = IntValue(ndx++);
			id->LightIndex = 0;		// select 1st pattern
			id->Priority = IntValue(ndx++);
			// for compatiblity these two fields are together
			id->Dimmed = BoolValue(ndx);
			id->CanRunFast = *(fields[ndx++] + 1) == 'T';		// two booleans together in the same column
			id->KeyDisplay = (enum TrafficState)IntValue(ndx++);
			CopyMesg(fields[ndx++], id->KeyMesg, MESG_MAX);
			id->TrafficProgram = IntValue(ndx++);
			id->TrafficPriority = IntValue(ndx++);
			id->TrafficDisplay = (enum TrafficState)IntValue(ndx++);
			// custom traffic message
			CopyMesg(fields[ndx++], id->TrafficMesg, MESG_MAX);
			id->DispFlag = BoolValue(ndx++);
			CopyMesg(fields[ndx++], id->DispMesg, MESG_MAX);
			id->IgnCondition = (enum Condition)IntValue(ndx++);
			id->PBCondition = (enum Condition)IntValue(ndx++);
			id->SoftwareDimmer = BoolValue(ndx++);	// software dimmer function
			id->FastPatterns = BoolValue(ndx++);	// fast patterns function
			id->FuncCondition = IntValue(ndx++);	// new function conditional

			ReadLine(line, line2);
			if (fldCnt >= KEY_COUNT + 1)
			{
				if (*line == 'B')
				{
					for(i = 0; i < KEY_COUNT; i++)
						id->Buttons[i] = IntValue(i + 1);
				}
				else
					errCode = UNEXPECTED_RECORD;	// unexpected record type
			}
			else
				errCode = FIELD_COUNT;	// invalid B field count

			ReadLine(line, line2);
			if (fldCnt >= VIRT_COUNT + 1)
			{
				if (*line == 'b')
				{
					for(i = 0; i < VIRT_COUNT; i++)
						id->VirtButtons[i] = IntValue(i + 1);
				}
				else
					errCode = UNEXPECTED_RECORD;	// unexpected record type
			}
			else
				errCode = FIELD_COUNT;	// invalid b field count

			ReadLine(line, line2);
			if (fldCnt >= OUTPUT_COUNT + 1)
			{
				if (*line == 'O')
				{
					for(i = 0; i < OUTPUT_COUNT; i++)
						id->Outputs[i] = BoolValue(i + 1);
				}
				else
					errCode = UNEXPECTED_RECORD;	// unexpected record type
			}
			else
				errCode = FIELD_COUNT;	// invalid B field count

			ReadLine(line, line2);
			if (fldCnt >= EXP_OUTPUTS + 1)
			{
				if (*line == 'X')
				{
					for(i = 0; i < EXP_OUTPUTS; i++)
						id->OutExp1[i] = BoolValue(i + 1);
				}
				else
					errCode = UNEXPECTED_RECORD;	// unexpected record type
			}
			else
				errCode = FIELD_COUNT;	// invalid B field count

			ReadLine(line, line2);
			if (fldCnt >= EXP_OUTPUTS + 1)
			{
				if (*line == 'X')
				{
					for(i = 0; i < EXP_OUTPUTS; i++)
						id->OutExp2[i] = BoolValue(i + 1);
				}
				else
					errCode = UNEXPECTED_RECORD;	// unexpected record type
			}
			else
				errCode = FIELD_COUNT;	// invalid B field count

		}
		else
			errCode = FIELD_COUNT;	// invalid field count record type T
	}
	else
		errCode = UNEXPECTED_RECORD;
}

static void RecordTypeToSend(char *line2, char *types)
{
	char rc, addr;
	BOOL okToSend;

	if (strchr(types, *line2) != NULL)
	{
		if (*line2 == 'Z')		// send eof to both if available
		{
			if (TmpTouch.LBExists)
			{
				rc = SendLineEx(LIGHT_ADDRESS, line2, strlen(line2), 400);
				if (rc != 'A')		// sucess sending?
					errCode = LIGHTBAR_COMM;
			}
			if (TmpTouch.RBExists)
			{
				rc = SendLineEx(RELAY_ADDRESS, line2, strlen(line2), 400);
				if (rc != 'A')		// sucess sending?
					errCode = RELAY_COMM;
			}
		}
		else
		{	
			if (*line2 > 'Z')		// lower case? then send to relay panel
			{
				addr = RELAY_ADDRESS;
				okToSend = TmpTouch.RBExists;
			}
			else
			{
				addr = LIGHT_ADDRESS;
				okToSend = TmpTouch.LBExists;
			}
			if (okToSend)
			{
				rc = SendLine(addr, line2, strlen(line2));
				if (rc != 'A')		// sucess sending?
					errCode = (addr == RELAY_ADDRESS) ? RELAY_COMM : LIGHTBAR_COMM;
			}
		}
	}
	if (*line2 == 'Z')		// the end?
	{
		if (fldCnt > 1)
		{
			if (*line2 != 'Z')
				errCode = UNEXPECTED_RECORD;		// invalid end of file
			else
			{
				if (errCode == 0)	// make sure this one's valid first
				{
					errCode = FILE_OK;		// done!
					// mark touch pad as valid
					TmpTouch.chkByte1 = 0x58;
					TmpTouch.chkByte2 = 0x9b;
					// save to flash for power up
					SaveTouchPad(&TmpTouch, sizeof(TmpTouch));					
					// make this one the active touch pad
					memmove(&TouchPad, &TmpTouch, sizeof(TmpTouch));
				}
			}
		}
		else
			errCode = FIELD_COUNT;
	}
}


static void ProcessLine(char *line, char *line2)
{
	switch(state)
	{
		case 0:		// need 'C' record type
			RecordTypeC(line);
			inpNdx = 0;
			break;
		case 1:		// now we need 'T' record types
			RecordTypeX(line, line2, 'T', &TmpTouch.KeyData[inpNdx]);
			if (++inpNdx >= KEY_COUNT)
			{
				inpNdx = 0;
				state++;
			}
			break;
		case 2:		// now we need 'V' record types
			RecordTypeX(line, line2, 'V', &TmpTouch.VirtData[inpNdx]);
			if (++inpNdx >= VIRT_COUNT)
			{
				inpNdx = 0;
				state++;
#ifdef F5TP40NS
				state++;		// since no slide switches
#elif RDS16
				state++;		// since no slide switches
#elif AST5500
				state++;		// since no slide switches
#endif
			}
			break;
		case 3:		// now we need 'S' record types
			RecordTypeX(line, line2, 'S', &TmpTouch.SlideData[inpNdx]);
			if (++inpNdx >= SLIDE_COUNT)
			{
				inpNdx = 0;
				state++;
			}
			break;
		case 4:		// now we need 'E' record types
			RecordTypeX(line, line2, 'E', &TmpTouch.RelayBoxInputs[inpNdx]);
			// special case for horn ring, MUST be positive active to work properly in the logic
			// even though it could be set to active high or low in the windows app
			if (inpNdx == 8)	// horn ring?
				TmpTouch.RelayBoxInputs[inpNdx].InputTrigger = POSITIVE;
			if (++inpNdx >= RELAYBOX_INPUTS)
			{
				inpNdx = 0;
				state++;
			}
			break;
		case 5:		// now we need 'A' record types
			RecordTypeX(line, line2, 'A', &TmpTouch.AnalogData[inpNdx]);
			if (++inpNdx >= ANALOG_CARDS)
			{
				inpNdx = 0;
				state++;
			}
			break;
		case 6:		// now we need 'I' record types
			RecordTypeX(line, line2, 'I', &TmpTouch.LightBarInputs[inpNdx]);
			if (++inpNdx >= LIGHTBAR_INPUTS)
			{
				inpNdx = 0;
				state++;
			}
			break;
		case 7:
			RecordTypeToSend(line2, "LFGPNZtfgpn");
			break;
	}
}

static int ReadNewFile(char *fn)
{
	BOOL eof = FALSE;
	char line[MAX_LINE];
	char line2[MAX_LINE];

	myFile = FSfopen(fn, "r");
	state = 0;
	errCode = FILE_ERR;
	if (myFile != NULL)
	{
		errCode = 0;
		rptr = MAX_LINE;
		rcnt = MAX_LINE;
		while(! eof && ! errCode)
		{
			eof = ReadLine(line, line2);
			if (fldCnt > 1)
				ProcessLine(line, line2);
		}

		ClearLine(2);
		switch(errCode)
		{
			case FILE_OK:
				Show_String("Done...", 2, 0);
				break;
			case WRONG_TOUCHPAD:
				Show_String("Invalid TP", 2, 0);
				break;
			case LIGHTBAR_COMM:
				Show_String("L Comm Err", 2, 0);
				break;
			case RELAY_COMM:
				Show_String("R Box Err", 2, 0);
				break;
			default:
				sprintf(line, "Error %d", errCode);
				Show_String(line, 2, 0);
				break;
		}
		
		//Always make sure to close the file so that the data gets
		FSfclose(myFile);
	}
	else
	{
		ClearLine(2);
		Show_String("No File!", 2, 0);
	}

	Show_String("Remove Drive", 3, 0);

	ErrLamp(errCode, TRUE);

	return errCode;
}

static void DispSelected()
{
	int i, ndx;
	char s[20];

	ndx = topLine;
	for(i = 0; ndx < fileCount && i < 3; i++, ndx++)
	{
		ClearLine(i + 1);
		if (ndx == fileNdx)
			sprintf(s, "<%s>", fileNames[ndx]);
		else
			sprintf(s, "       %s", fileNames[ndx]);
		Show_String(s, i + 1, 0);
	}
}

void ThumbTasks()
{
	int i, k;

	//USB stack process function
	USBTasks();

	switch(thumbState)
	{
		case 0:
			//if thumbdrive is plugged in
			if(USBHostMSDSCSIMediaDetect())
			{
				ClearScreen();
				Show_String("Drive Detect", 0, 0);
				Show_String("Initializing...", 1, 0);
				deviceAttached = TRUE;
				//now a device is attached
				//See if the device is attached and in the right format
				if(FSInit())
				{
					// turn off any running keys!
					AllKeysOff();
					thumbState++;
					// turn off the watch dog reset on the relay panel and lightbar
					WatchReset(LIGHT_ADDRESS, FALSE);
					WatchReset(RELAY_ADDRESS, FALSE);
				}
				else
				{
					ErrLamp(USB_ERR, TRUE);
					thumbState = 9;
					Show_String("Init Failed!", 2, 0);
					Show_String("Remove Drive", 3, 0);
				}
			}
			break;
		case 1:
			if (GetFileList("*.tpp") == -1)
			{
				ErrLamp(MEMORY_ERR, TRUE);
				Show_String("Memory Fault", 1, 0);
			}
			else
			{
				if (fileCount == 0)
				{
					ErrLamp(FILE_NOT_FOUND, TRUE);
					Show_String("No TP Files", 1, 0);
				}
				else
				{
// touch pads without a display
#ifdef RDS16
					ReadNewFile("rds16.tpp");
					thumbState = 9;
#elif RDS17
					ReadNewFile("rds17.tpp");
					thumbState = 9;
#else
					ClearScreen();
					Show_String("Select File", 0, 0);
					fileNdx = 0;
					topLine = 0;
					DispSelected();
					thumbState++;
					specialKeys = TRUE;		// enable special keyboard
					setupState = 0;
					DisplayLamp(TRUE);
#endif
				}
			}
			if (fileCount <= 0)
			{
				thumbState = 9;
				Show_String("Remove Drive", 3, 0);
			}
			break;
		case 2:
			if (! deviceAttached)
				thumbState = 9;
			else
			{
				if(k = GetKS())
				{
					ClearKey();
					switch(k)
					{
						case 1:
							ClearScreen();
							Show_String("File Selected", 0, 0);
							Show_String("Reading File..", 2, 0);
							// read in the new file
							ReadNewFile(fileNames[fileNdx]);
							break;
						case 2:
							if (fileNdx < fileCount - 1)
							{
								if (++fileNdx >= 3)		// max lines?
									if (fileNdx - topLine >= 3)									
										topLine++;
								DispSelected();
							}
							break;
						case 3:
							if (fileNdx > 0)
							{
								if (--fileNdx < topLine)
									topLine--;
								DispSelected();
							}
							break;
					}
				}
			}
			break;
		//Just sit here until the device is removed.
		case 9:
			if (! deviceAttached)
			{
				DisplayLamp(FALSE);
				ErrLamp(0, FALSE);		// turn off error lamps
				FreeFileNames();
				thumbState = 0;
				specialKeys = FALSE;
				refreshDisp = TRUE;
				// turn on the watch dog reset on the relay panel and lightbar
				WatchReset(LIGHT_ADDRESS, TRUE);
				WatchReset(RELAY_ADDRESS, TRUE);

#ifndef __DEBUG
				while(1);		// force a reset!
#endif
			}
			break;
	}
}

/****************************************************************************
  Function:
    BOOL USB_ApplicationEventHandler( BYTE address, USB_EVENT event,
                void *data, DWORD size )

  Summary:
    This is the application event handler.  It is called when the stack has
    an event that needs to be handled by the application layer rather than
    by the client driver.

  Description:
    This is the application event handler.  It is called when the stack has
    an event that needs to be handled by the application layer rather than
    by the client driver.  If the application is able to handle the event, it
    returns TRUE.  Otherwise, it returns FALSE.

  Precondition:
    None

  Parameters:
    BYTE address    - Address of device where event occurred
    USB_EVENT event - Identifies the event that occured
    void *data      - Pointer to event-specific data
    DWORD size      - Size of the event-specific data

  Return Values:
    TRUE    - The event was handled
    FALSE   - The event was not handled

  Remarks:
    The application may also implement an event handling routine if it
    requires knowledge of events.  To do so, it must implement a routine that
    matches this function signature and define the USB_HOST_APP_EVENT_HANDLER
    macro as the name of that function.
  ***************************************************************************/

BOOL USB_ApplicationEventHandler( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    switch( event )
    {
        case EVENT_VBUS_REQUEST_POWER:
            // The data pointer points to a byte that represents the amount of power
            // requested in mA, divided by two.  If the device wants too much power,
            // we reject it.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // Turn off Vbus power.
            // The PIC24F with the Explorer 16 cannot turn off Vbus through software.

            //This means that the device was removed
            deviceAttached = FALSE;
            return TRUE;
            break;

        case EVENT_HUB_ATTACH:
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            //UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            //UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            //UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            //UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;
}
