/*
	Our utility functions
	Copyright (c) DM&T Services Ltd  2010
*/
#define _SUPPRESS_PLIB_WARNING 1 
#include "Compiler.h"
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"

/*****************************************************************************
* Function: itoan, int to ascii string n bytes in length
*
* Input: i is int to convert, s is n bytes + 1 long, n is dest length
*
* Output: None.
*
******************************************************************************/
void itoan(int i, char *s, int n)
{
	BOOL minus;

	if((minus = i < 0))
		i = -i;
	s += n;
	*s-- = '\0';	// null terminate
	do
	{
		*s-- = i % 10 + '0';
		i /= 10;
		n--;	
	}
	while(i && (n > 0));

	if(n > 0 && minus)
	{
		*s-- = '-';
		n--;
	}
	while(n--)
		*s-- = ' ';
}


/*****************************************************************************
* Function: utoan, unsigned int to ascii string n bytes in length
*
* Input: i is int to convert, s is n bytes + 1 long, n is dest length
*
* Output: None.
*
******************************************************************************/
void utoan(unsigned int i, char *s, int n)
{
	s += n;
	*s-- = '\0';	// null terminate
	do
	{
		*s-- = i % 10 + '0';
		i /= 10;
		n--;	
	}
	while(n > 0);
}

/*****************************************************************************
* Function: ltoan, long to ascii string n bytes in length
*
* Input: l is long to convert, s is n bytes + 1 long, n is dest length
*
* Output: None.
*
******************************************************************************/
void ltoan(long l, char *s, int n)
{
	BOOL minus;

	if((minus = l < 0))
		l = -l;
	s += n;
	*s-- = '\0';	// null terminate
	while(l && (n > 0))
	{
		*s-- = l % 10 + '0';
		l /= 10;
		n--;	
	}
	if(n > 0 && minus)
	{
		*s-- = '-';
		n--;
	}
	while(n--)
		*s-- = ' ';
}

/*****************************************************************************
* Function: insdec, insert a decimal place
*
* Input: *s is string to insert into, i is the position
*
* Output: None.
*
******************************************************************************/
void insdec(char *s, int i)
{
	char *ptr;
	int n;

	if(! i)
		return;
	ptr = s;
	if((n = strlen(ptr) - i) > 0)
	{
		ptr += n;
		memmove(ptr + 1, ptr, i + 1);
		*ptr = '.';
	}
}

void strpchr(char *s, char c)
{
	char *ptr;

	ptr = s;
	while(*ptr)
		if(*ptr++ == c)
			memmove(ptr - 1, ptr, strlen(ptr) + 1);
}


/*****************************************************************************
* Function: trim a string
*
* Input: *s is string to trim the spaces
*
* Output: None.
*
******************************************************************************/
void trim(char *s)
{
	int len;

	len = strlen(s);
	while(len > 0)
	{
		len--;
		if(s[len] != ' ')
			break;
		s[len] = '\0';
	}
}


/*****************************************************************************
* Function: trim a string
*
* Input: *s is source, *d destination, len is length to pad with spaces
*
* Output: None.
*
******************************************************************************/
void rpad(char *s, char *d, int len)
{
	int n;

	memset(d, ' ', len);
	d[len] = '\0';
	if((n = strlen(s)) > len)
		n = len;	
	memmove(d, s, n);
}

void strrepl(char look, char repl, char *s)
{
	while(*s)
	{
		if (*s == look)
			*s = repl;
		s++;
	}
}

/*****************************************************************************
* Function: FtoC, convert to Celsius from Fahrenheit
*
* Input: f is degress fah * 8
*
* Output: degrees celsius rounded to 0.5 C
*
******************************************************************************/
int FtoC(int f)
{
	f -= 256;		// 8 * 32 degs. f
	f = (f * 5) / 9;
	f = (f * 10) / 8;		// there temp if * 8 so divide
	f = (f / 5) * 5;		// round to lowest 0.5 C
	return f;
}

/*
  f is temperature * 10
*/
int FtoCInt(int f)
{
	return ((f - 320) * 50) / 90;
}

void FlattenWord(WORD w, BYTE *b)
{
	memmove(b, (BYTE *)&w, 2);
}

void FlattenLong(long l, BYTE *b)
{
//	long *lp;

	memmove(b, (BYTE *)&l, 4);
//	lp = (long *)b;
//	*lp = l;
}

void FlattenFloat(float f, BYTE *b)
{
	memmove(b, (BYTE *)&f, 4);
}

int SwapInt(BYTE *b)
{
	return((b[0] << 8) | b[1]);
}

void Swap2Bytes(BYTE *b)
{
	WORD w;

	w = SwapInt(b);

	FlattenWord(w, b);
}

int IntPtr(BYTE *b)
{
	return((b[1] << 8) | b[0]);
}

long SwapLong(BYTE *b)
{
	long Result;
	BYTE *p;

	p = (BYTE *)&Result;
	*p++ = b[3];
	*p++ = b[2];
	*p++ = b[1];
	*p = *b;

	return Result;
}

long SwapLong3(BYTE *b)
{
	long Result;
	BYTE *p;

	p = (BYTE *)&Result;
	*p++ = b[2];
	*p++ = b[1];
	*p++ = *b;
	*p = 0;

	return Result;
}

BYTE NibToHex(BYTE b)
{
	b &= 0xf;
	if(b > 9)
		b += 7;
	return b + '0';
}

void ByteToHex(BYTE b, char *dest)
{
	*dest++ = NibToHex(b >> 4);
	*dest = NibToHex(b);
}
	
BYTE HexToNib(char c)
{
	if(c >= 'A')
		c -= 7;
	return(c & 0xf);
}

BYTE HexToByte(char *s)
{
	BYTE Result = HexToNib(*s++) << 4;
	return Result + HexToNib(*s);
}

WORD HexToWord(char *s)
{
	return (HexToByte(s) << 8) | HexToByte(s + 2);
}

DWORD HexToDWord(char *s)
{
	DWORD Result = (DWORD)HexToByte(s) << 24UL;
	Result |= (DWORD)HexToByte(s + 2) << 16UL;
	Result |= (DWORD)HexToByte(s + 4) << 8UL;
	Result |= (DWORD)HexToByte(s + 6);
	return Result;
}

unsigned int beToUInt16(unsigned char *buf, int pos)
{
	unsigned int result;
	result = (*(buf + pos + 1) << 8) | *(buf + pos);
	return result;
}

unsigned long beToUInt32(BYTE *buf, int pos)
{
	unsigned long result;
	unsigned long l;
	l = *(buf + pos + 3);
	result = l << 24;
	l = *(buf + pos + 2);
	result |= l << 16;
	l = *(buf + pos + 1);
	result |= l << 8;
	result |= *(buf + pos);

	return result;
}

unsigned int ToUInt16(unsigned char *buf, int pos)
{
	unsigned int result;
	result = (*(buf + pos) << 8) | *(buf + pos + 1);
	return result;
}

void copymax(char *dest, char *src, int maxLen)
{
	memmove(dest, src, maxLen);
	dest[maxLen] = '\0';
}
